import time
from django.core.management.base import BaseCommand
from django.conf import settings
from django.db import connections


class Command(BaseCommand):
    help = 'Fix sequences after sync'

    PREFIXES = ['chat_', 'website_', 'visitors_']

    def handle(self, *args, **options):

        if connections["default"].vendor != 'postgresql':
            raise ConnectionError()
        with connections["default"].cursor() as cursor:

            cursor.execute("""SELECT t.oid::regclass AS table_name,
                a.attname AS column_name,
                s.relname AS sequence_name
            FROM pg_class AS t
            JOIN pg_attribute AS a
                ON a.attrelid = t.oid
            JOIN pg_depend AS d
                ON d.refobjid = t.oid
                    AND d.refobjsubid = a.attnum
            JOIN pg_class AS s
                ON s.oid = d.objid
            WHERE d.classid = 'pg_catalog.pg_class'::regclass
            AND d.refclassid = 'pg_catalog.pg_class'::regclass
            AND d.deptype IN ('i', 'a')
            AND t.relkind IN ('r', 'P')
            AND s.relkind = 'S';""")
            items =  cursor.fetchall()
            for table_name, pk_field, sequence_name in items:
                use_table = False
                for prefix in self.PREFIXES:
                    if table_name.startswith(prefix):
                        use_table = True
                        break
                if not use_table:
                    continue
                cursor.execute(f"select max({pk_field}) from {table_name};")
                (max_id,) = cursor.fetchone()
                cursor.execute("select last_value from pg_sequences where sequencename = %s;", [sequence_name])
                (seq_id,) = cursor.fetchone()
                # print(table_name, pk_field, sequence_name, max_id, seq_id)# , max_id > seq_id)

                if not max_id:
                    continue

                if not seq_id:
                    print("WARN", f"Not found next_value for {sequence_name} for table {table_name}")
                    continue

                if max_id > seq_id:
                    print("Found unsync values at ", sequence_name, "=", seq_id, "; at table", table_name, "max", max_id, "; delta ", max_id - seq_id)
                    cursor.execute("SELECT nextval(%s) from generate_series(1, %s)", [sequence_name, max_id - seq_id]);

    if __name__ == "__main__":
        handle()
