from django.db.models.signals import post_save, post_delete, pre_delete, pre_save, m2m_changed
from django.dispatch import receiver
from django.conf import settings
from .models import ArbiterHostMap, ArbiterHostDomains, ArbiterHostInfo

import json
import socket
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
channel_layer = get_channel_layer()


@receiver(post_save, sender=ArbiterHostMap)
def hostmap_post_save(sender, instance=None, **kwargs):
    wsock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    wsock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    wsock.sendto(b'rereadmap', ('127.0.0.1', 5006))
    wsock.close()


@receiver(post_save, sender=ArbiterHostInfo)
def hostinfo_post_save(sender, instance=None, **kwargs):
    async_to_sync(channel_layer.group_send)("host-pool", {"type": "websocket.send",  "text":  json.dumps({
        "address": instance.address,
        "mode": instance.mode,
        "timestamp": instance.timestamp,
        "hostname": instance.hostname,
        "isok": instance.isok,
        "host": instance.host_id,
    })})
