import time

from django.conf import settings
from nicechat.celery import app
from nicechat.utils.arbiter import update_data_workernode
from nicechat.utils.arbiter_sync import get_user_relations, get_website_relations


def sync_user(user, target=None):
    if settings.NODE_MODE == settings.NODE_MODE_STANDALONE:
        return
    tgt = user.website_set.values_list('node2website__node', flat=True) if not target else [target.pk]
    return update_data_workernode(
        tgt,
        'user',
        get_user_relations(user)
    )

@app.task
def sync_user_delayed(user_pk):
    from apps.users.models import User
    user = User.objects.filter(pk=user_pk).first()
    sync_user(user)

def sync_website(website, target=None):
    if settings.NODE_MODE == settings.NODE_MODE_STANDALONE:
        return
    tgt = website.node2website.values_list('node_id', flat=True) if not target else [target.pk]
    return update_data_workernode(
        tgt,
        'website',
        get_website_relations(website)
    )

@app.task
def sync_website_delayed(website_pk):
    from apps.webiste.models import Website
    website = Website.objects.filter(pk=website_pk).first()
    sync_website(website)

def _check_is_same_exists(ct):
    from celery import current_app

    active_tasks = current_app.control.inspect().active()
    for worker, running_tasks in active_tasks.items():
        for _task in running_tasks:
            if _task.get('name') == ct.name and _task['id'] != ct.request.id:
                # exists the same task with different requestid
                return True
    return False

@app.task(bind=True)
def website_transfer_executor(self):
    if _check_is_same_exists(self):
        print("Task already active. Skip.")
        return
    from apps.balancer.models import ArbiterWebsiteTransfer
    for awt in ArbiterWebsiteTransfer.objects.exclude(step=100):
        awt.run()
