from apps.balancer.transfer.utils import apply_sync_data, default_data_writing_call
from nicechat.utils.arbiter_sync import scenario_store
from django.core import serializers


def sync_clients(t_pk, website, node_from, node_to, log):
    return apply_sync_data('sync_clients', t_pk, website, node_from, node_to, log)

def sync_visitors(t_pk, website, node_from, node_to, log):
    return apply_sync_data('sync_visitors', t_pk, website, node_from, node_to, log)

def sync_products(t_pk, website, node_from, node_to, log):
    return apply_sync_data('sync_products', t_pk, website, node_from, node_to, log)

def sync_forms(t_pk, website, node_from, node_to, log):
    return apply_sync_data('sync_forms', t_pk, website, node_from, node_to, log)

def sync_quickanswers(t_pk, website, node_from, node_to, log):
    return apply_sync_data('sync_quickanswers', t_pk, website, node_from, node_to, log)

def sync_orders(t_pk, website, node_from, node_to, log):
    return apply_sync_data('sync_orders', t_pk, website, node_from, node_to, log)


def do_read_sync_clients(website):
    obj_pack = []

    cli_scenario = {
        'Client': {
            'ignore_pk': True,
        },
        'ClientEntrance': {
            'ignore_pk': True,
            'set_fields': {'client': 'Client'}
        },
        'ClientPromotion': {
            'ignore_pk': True,
            'set_fields': {'client': 'Client'}
        }

    }

    from apps.visitors.models import Client

    _recs = []
    for cli in Client.objects.filter(website=website):
        data = [cli]
        for ce in cli.entrances.all():
            data.append(ce)
        for cp in cli.promotions.all():
            data.append(cp)
        _recs.append(serializers.serialize('json', data))

        if len(_recs) > 49:
            if not scenario_store(cli_scenario, _recs, obj_pack, website.pk, 'sync_clients'):
                return False
            _recs = []

    if len(_recs):
        if not scenario_store(cli_scenario,_recs, obj_pack, website.pk, 'sync_clients'):
            return False

    return obj_pack


def do_read_sync_visitors(website):
    obj_pack = []

    cli_scenario = {
    }

    from apps.chat.models import Idempotency
    
    _recs = []
    for idp in Idempotency.objects.filter(visitor__website=website):
        data = [idp]

        for peer in idp.peers.all():
             data.append(peer)

        _recs.append(serializers.serialize('json', data))

        if len(_recs) > 200:
            if not scenario_store(cli_scenario, _recs, obj_pack, website.pk, 'sync_visitors'):
                return False
            _recs = []

    if len(_recs):
        if not scenario_store(cli_scenario, _recs, obj_pack, website.pk, 'sync_visitors'):
            return False

    return obj_pack

def do_read_sync_forms(website):
    obj_pack = []

    cli_scenario = {
        'Form': {
            'ignore_pk': True,
        },
        'FormResult': {
            'ignore_pk': True,
            'set_fields': {'form': 'Form'}
        },
        'FormRow': {
            'ignore_pk': True,
            'set_fields': {'form': 'Form'}
        },
    }

    from apps.website.models import Form
    from apps.visitors.models import FormResult
    
    _recs = []
    for frm in Form.objects.filter(website=website):
        data = [frm]

        for row in frm.formrow_set.all():
             data.append(row)

        _recs.append(serializers.serialize('json', data))

        for frs in FormResult.objects.filter(form_id=frm.pk):
            data.append(frs)

        if len(_recs) > 20:
            if not scenario_store(cli_scenario, _recs, obj_pack, website.pk, 'sync_forms'):
                return False
            _recs = []

    if len(_recs):
        if not scenario_store(cli_scenario, _recs, obj_pack, website.pk, 'sync_forms'):
            return False

    return obj_pack


def do_read_sync_products(website):
    obj_pack = []

    cli_scenario = {
        'ProductsCategory': {
            'ignore_pk': True,
            'set_fields': {'parent': 'ProductsCategory'}
        },
        'Product': {
            'ignore_pk': True,
            'set_fields': {'category': 'ProductsCategory'}
        }
    }

    from apps.website.models import Product, ProductsCategory

    _data = []
    items = [cc for cc in ProductsCategory.objects.filter(website=website, parent__isnull=True)]
    while items:
        cc = items.pop()
        _data.append(cc)
        for _c in cc.productscategory_set.all():
            items.insert(0, _c)
        if len(_data) > 200:
            if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_products'):
                return False
            _data = []

    for cc in Product.objects.filter(website=website.pk):
        _data.append(cc)
        if len(_data) > 200:
            if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_products'):
                return False
            _data = []

    if len(_data):
        if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_products'):
            return False
        _data = []

    return obj_pack


def do_read_sync_quickanswers(website):
    obj_pack = []

    cli_scenario = {
        'QuickReply': {
            'ignore_pk': True,
        }
    }

    from apps.website.models import QuickReply

    _data = []

    for cc in QuickReply.objects.filter(website=website.pk):
        _data.append(cc)
        if len(_data) > 500:
            if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_quickanswers'):
                return False
            _data = []

    if len(_data):
        if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_quickanswers'):
            return False
        _data = []

    return obj_pack


def do_read_sync_orders(website):
    obj_pack = []

    cli_scenario = {
        'Order': {
            'ignore_pk': True,
            'set_fields': {'client': 'Client'},
            'set_m2m': {'products': 'Product'}
        }
    }

    from apps.visitors.models import Order

    _data = []

    for cc in Order.objects.filter(website=website.pk):
        _data.append(cc)
        if len(_data) > 100:
            if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_quickanswers'):
                return False
            _data = []

    if len(_data):
        if not scenario_store(cli_scenario, [serializers.serialize('json', _data)], obj_pack, website.pk, 'sync_quickanswers'):
            return False
        _data = []

    return obj_pack



do_write_sync_clients = default_data_writing_call
do_write_sync_visitors = default_data_writing_call
do_write_sync_products = default_data_writing_call
do_write_sync_forms = default_data_writing_call
do_write_sync_quickanswers = default_data_writing_call
do_write_sync_orders = default_data_writing_call
