import json
from nicechat.utils.arbiter import update_data_workernode
from django.core import serializers


def apply_sync_data(action, t_pk, website, node_from, node_to, log):
    readed = update_data_workernode([node_from], 'doread', {
        'website_id': website.pk,
        'action': action,
    })
    if not readed:
        log(f'Read error at {action}')
        return False
    if isinstance(readed, str):
        readed = json.loads(readed)
    readed = readed.get('dumps', [])
    log(f"Reading complete. {len(readed)} task in queue.")
    print("!!!!", readed)
    for rec in readed:
        wrote = update_data_workernode([node_to], 'dowrite', {
            'website_id': website.pk,
            'action': action,
            't_pk': t_pk,
            'data_path': rec
        })
        if isinstance(wrote, str):
            wrote = json.loads(wrote)
        if not wrote:
            log(f'Write error at {action}')
            return False
        log('; '.join([f"{o} - {c}" for o, c in wrote.items()]))
    log('Writing complete')
    return True


def default_data_writing_call(website, data, transform, t_pk):
    from apps.balancer.models import ArbiterWebsiteTransferPK

    affected = {}

    for rec in data:
        objs = serializers.deserialize('json', rec)

        for _obj in objs:
            obj = _obj.object
            c_name = obj.__class__.__name__
            tf = transform.get(c_name, {})
            ignore_pk = tf.get('ignore_pk', False)
            ignore_error = tf.get('ignore_error', False)
            clr_fields = tf.get('clr_fields', [])
            set_fields = tf.get('set_fields', {})
            set_m2m = tf.get('set_m2m', {})
            data_m2m = _obj.m2m_data

            old_pk = obj.pk
            
            if ignore_pk:
                a_id_obj = ArbiterWebsiteTransferPK.objects.filter(
                    transfer = t_pk,
                    class_name = c_name,
                    old_id = old_pk,
                ).first()
                obj.pk = None if not a_id_obj else a_id_obj.new_id

            if set_fields:
                ret_set_items = {}
                for f_name, f_tgt in set_fields.items():
                    _rel_old_id = getattr(obj, f"{f_name}_id", None)
                    if _rel_old_id is None:
                        ret_set_items[f_name] = None
                        continue
                    a_id_obj_rel = ArbiterWebsiteTransferPK.objects.filter(
                        transfer = t_pk,
                        class_name = f_tgt,
                        old_id = _rel_old_id,
                    ).first()
                    if a_id_obj_rel:
                        ret_set_items[f_name] = a_id_obj_rel.new_id
                        setattr(obj, f"{f_name}_id", a_id_obj_rel.new_id)

                if list(ret_set_items.keys()) != list(set_fields.keys()):
                    continue

            for clr in clr_fields:
                setattr(obj, clr, None)

            try:
                obj.save()
            except Exception as e:
                if not ignore_error:
                    raise e

            if data_m2m:
                for attr_name, rec_data in data_m2m.items():
                    m2m_mgr = getattr(obj, attr_name, None)
                    if m2m_mgr:
                        model = m2m_mgr.model
                        model_name = model.__name__
                        for rec in rec_data:
                            use_id = None
                            if attr_name in set_m2m.keys():
                                a_id_obj_rel = ArbiterWebsiteTransferPK.objects.filter(
                                    transfer = t_pk,
                                    class_name = model_name,
                                    old_id = rec,
                                ).first() 
                                use_id = a_id_obj_rel.new_id if a_id_obj_rel else None
                            else:
                                use_id = rec
                            m2m_rec = model.objects.filter(pk=use_id).first()
                            if m2m_rec:
                                m2m_mgr.add(m2m_rec)
                                print(' > ', attr_name, m2m_rec)  

            if ignore_pk and not a_id_obj:
                ArbiterWebsiteTransferPK.objects.create(
                    transfer = t_pk,
                    class_name = c_name,
                    old_id = old_pk,
                    new_id = obj.pk
                )

            affected[c_name] = affected.get(c_name, 0) + 1
            
            print(obj)
    return affected

