from django.urls import path, re_path, include
from apps.balancer.views import NodesInviteView, DashboardView, ArbiterLoginView, ArbiterLogoutView, NodeDetailView, \
    NodesListView, HostsListView, WebsitesListView, ManagersListView, AgentsListView, ManagerDetailView, \
    AgentDetailView, ManagerAddView, AssignHostView, HostDetailView, WebsitesDetailView, \
    AgentSwitchView, ManagerSwitchView, ManagerPwdView, ManagerDelView, AgentPwdView, ManagerOtpView, \
    NodeDeleteView, NodesCredentailsView, RouteMapView, NodeForNewSwitchView, WebsitesStartTransferView, \
    ArbiterWebsiteTransferDetailView, ArbiterWebsiteTransfersListView, GoToNodeAdminView, \
    WebsitesStartClearView, WebsitesProcessClearView, AppearanceView, HostDeleteView, HostEmailView, \
    AppearancePermissionsView, CustomerListView, CustomerDetailView, FirstRunView, ArbiterCredentailsView, \
    ArbiterDomainEditView, ArbiterDomainsView, ArbiterDomainDeleteView, HostInfoView

from django.urls import path

urlpatterns = [
    path("nodes/<int:pk>/", NodeDetailView.as_view(), name="node_det"),
    path("nodes/<int:pk>/delete/", NodeDeleteView.as_view(), name="node_delete"),
    path("nodes/<int:pk>/creds/", NodesCredentailsView.as_view(), name="node_creds"),    
    path("nodes/<int:pk>/for_new/", NodeForNewSwitchView.as_view(), name="node_fornew"),
    path("nodes/<int:pk>/node_admin/", GoToNodeAdminView.as_view(), name="node_admin_go"),
    path("nodes/invite/", NodesInviteView.as_view(), name="invite_node"), 
    
    path("nodes/", NodesListView.as_view(), name="nodes_list"),
    path("appearance-permissions/", AppearancePermissionsView.as_view(), name="system_appearance_perm"),
    path("appearance/", AppearanceView.as_view(), name="system_appearance"),
    
    path("danger/arbiter/", ArbiterCredentailsView.as_view(), name="arbiter_settings"),
    path("danger/domains/", ArbiterDomainsView.as_view(), name="domains_list"),
    path("danger/domains/new/", ArbiterDomainEditView.as_view(), name="domains_new"),
    path("danger/domains/<str:domain>/", ArbiterDomainEditView.as_view(), name="domains_edit"),
    path("danger/domains/<str:domain>/delete/", ArbiterDomainDeleteView.as_view(), name="domains_delete"),
    

    path("hosts/<int:pk>/", HostDetailView.as_view(), name="host_det"),
    path("hosts/assign/<str:address>/", AssignHostView.as_view(), name="host_assign"),
    path("hosts/info/<str:address>/", HostInfoView.as_view(), name="host_info"),
    
    path("hosts/<int:pk>/delete/", HostDeleteView.as_view(), name="host_delete"),
    path("hosts/<int:pk>/test_email/", HostEmailView.as_view(), name="host_test_email"),
    path("hosts/", HostsListView.as_view(), name="hosts_list"),

    path("route_map/", RouteMapView.as_view(), name="route_map"),

    path("customer/", CustomerListView.as_view(), name="customers_list"),
    path("customer/<int:pk>/", CustomerDetailView.as_view(), name="customers_det"),

    path("websites/", WebsitesListView.as_view(), name="websites_list"),
    path("websites/<int:pk>/", WebsitesDetailView.as_view(), name="website_det"),
    path("website/<int:pk>/transfer/", WebsitesStartTransferView.as_view(), name="website_move"),
    path("website/<int:pk>/clear-start/<int:node>/", WebsitesStartClearView.as_view(), name="website_clear"),
    path("website/<int:pk>/clear-process/<int:node>/", WebsitesProcessClearView.as_view(), name="website_clear_exec"),
    path("website/<int:pk>/transfer-logs/", ArbiterWebsiteTransfersListView.as_view(), name="website_moves_list"),
    path("website/<int:site_id>/transfer-logs/<int:pk>/", 
         ArbiterWebsiteTransferDetailView.as_view(), name="website_moves_det"),


    path("agents/", AgentsListView.as_view(), name="agents_list"),
    path("agents/<int:pk>/", AgentDetailView.as_view(), name="agent_det"),
    path("agents/<int:pk>/pwd/", AgentPwdView.as_view(), name="agent_pwd"),
    path("agents/<int:pk>/<str:action>/", AgentSwitchView.as_view(), name="agent_sw"),

    path("managers/", ManagersListView.as_view(), name="managers_list"),
    path("managers/invite/", ManagerAddView.as_view(), name="invite_manager"),
    path("managers/<int:pk>/", ManagerDetailView.as_view(), name="manager_det"),
    path("managers/<int:pk>/del/", ManagerDelView.as_view(), name="manager_rem"),
    path("managers/<int:pk>/pwd/", ManagerPwdView.as_view(), name="manager_pwd"),
    path("managers/<int:pk>/otp/", ManagerOtpView.as_view(), name="manager_otp"),
    path("managers/<int:pk>/<str:action>/", ManagerSwitchView.as_view(), name="manager_sw"),

    path("login/", ArbiterLoginView.as_view(), name="login"),
    path("logout/", ArbiterLogoutView.as_view(), name="logout"),

    path("first-run/", FirstRunView.as_view(), name="first_run"),
    path("first-run/<str:step>/", FirstRunView.as_view(), name="first_run_step"),
    
    path("", DashboardView.as_view(), name="main"),
]
