from datetime import datetime
import PIL
# from PIL import Image
import cairosvg
from rest_framework import status
from django.conf import settings
from django.core.files.base import ContentFile
from django.core.files.storage import get_storage_class


def save_site_icon(content, site_id=None):

    size = len(content)

    if size > settings.AGENT_ATTACH_LIMIT * 1048576:
        return False, status.HTTP_413_REQUEST_ENTITY_TOO_LARGE

    if b'<svg' in content:
        try:
            content = cairosvg.svg2png(bytestring=content)
        except Exception:
            pass

    try:
        storage = get_storage_class(settings.CHAT_UPLOAD_STORAGE)()
    except ImportError as ie:
        print(ie)
        return False, status.HTTP_507_INSUFFICIENT_STORAGE

    try:
        
        import io
        from uuid import uuid4
        image = PIL.Image.open(io.BytesIO(content))

        MAX_SIZE = (200, 200)
        
        image.thumbnail(MAX_SIZE)
        
        path2u = "icons/preview/%s-%s/%s.png" % (datetime.now().year, datetime.now().month, str(uuid4()))
        # path2 = "%s/%s" % (settings.MEDIA_ROOT, path2u)
        thumb_io = io.BytesIO()
        image.save(thumb_io, format="PNG")

        thumb_io.seek(0)
        result = storage.save(path2u, ContentFile(thumb_io.read()))
        
        preview = '%s%s' % (settings.MEDIA_URL, path2u)
    except PIL.UnidentifiedImageError:
        return False, status.HTTP_415_UNSUPPORTED_MEDIA_TYPE
    except Exception as x:
        import traceback
        print(traceback.print_exc())
        return False, status.HTTP_415_UNSUPPORTED_MEDIA_TYPE

    return True, preview

