import os
import json
import time
import socket


def balancer_packet(data, addr, settings):
    if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
        process_aribiter(data, addr, settings)
    if settings.NODE_MODE == settings.NODE_MODE_WORKER:
        process_worker(data, addr, settings)

def process_worker(data, addr, settings):
    
    if data == b'nc_meetyou' or data == b'iamworker': # or data == b'iamnewnode'
        msg = b"nclist:" + ":".join(settings.NODE_ARBITER_HOST).encode()
        settings.WSOCKA.sendto(msg, (addr[0], 5005))
    elif data == b'iamarbiter':
        settings.NODE_ARBITER_HOST.add_host(addr[0])
    elif data and data.startswith(b'nclist'):
        nclist = data.split(b":")[1:]
        for rec in nclist:
            settings.NODE_ARBITER_HOST.add_host(rec.decode())

def process_aribiter(data, addr, settings):
    from .models import ArbiterHostInfo, ArbiterNodeHost
    address = addr[0]
    try:
        hostname, _, _ = socket.gethostbyaddr(address)
    except socket.error:
        hostname = address
    host = None
    isok = True
    stats = {'status': 'unknown'}
    mode = 'unknown'
    print(">>>>", data, addr)
    if data == b'boot' or data == b'exit' or data == b'nc_meetyou':
        stats['status'] = 'stop' if data == b'exit' else 'starting'
        isok = False
    elif data == b'iamnewnode' or (data and data.startswith(b'nc_newnode_')):
        stats['status'] = 'new'
        mode = 'new'
    elif data == b'iamarbiter' or data == b'up_arbiter':
        stats['status'] = 'up'
        mode = 'arbiter'
        host = ArbiterNodeHost.objects.filter(node__mode=settings.NODE_MODE_ARBITER).first()
    elif data == b'iambalancer':
        stats['status'] = 'up'
        mode = 'balancer'
    elif data and data.startswith(b'iamworker'):
        stats['status'] = 'up'
        mode = 'worker'
        host = ArbiterNodeHost.objects.filter(
            node__mode=settings.NODE_MODE_WORKER,
            host_pubkey = data[10:].decode()
        ).first()
    elif data and data.startswith(b'nc_error_'):
        stats['status'] = 'error'
        edata = data
        isok = False
        
        edata = edata.split(b":", 1).pop()
        try:
            _edata = json.loads(edata).split(":", 3)
        except json.JSONDecodeError:
            _edata = edata
        if len(_edata) == 4:
            mode = _edata[1]
            stats['error'] = _edata[3]
            if mode == 'worker':
                host = ArbiterNodeHost.objects.filter(
                    node__mode=settings.NODE_MODE_WORKER,
                    host_pubkey = _edata[2]
                ).first()
        else:
            stats['error'] = f"{edata}"
            mode = 'error'

    else:
        return

    n, _ = ArbiterHostInfo.objects.update_or_create(
        address = address,
        defaults = {
            'mode': mode,
            'hostname': hostname,
            'isok': isok,
            'host': host,
            'timestamp': int(time.time()),
            'stats': json.dumps(stats) if stats else '{}'
        }
        # host = FK
    )
