from nicechat.admins import ncadmin
from django.contrib import admin
from django.utils.translation import gettext_lazy as _

from .models import Plan, Transaction, CompanyPlan, PlanFeature, PaymentMethod, \
    PromoCode, PromoCodeUsage, PlanSettings


class BillingPlanInline(admin.StackedInline):
    model = CompanyPlan
    can_delete = False


class PaymentMethodInline(admin.TabularInline):
    model = PaymentMethod


class PromoCodeUsageInline(admin.TabularInline):
    model = PromoCodeUsage
    readonly_fields = ('company', 'date_used',)
    can_delete = False
    classes = ['collapse']

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False



class PlanAdmin(admin.ModelAdmin):
    list_display = ('name', 'month_price', 'annual_price', 'is_trial', 'is_free')


class PlanSettingsAdmin(admin.ModelAdmin):
    list_display = ('prop', 'plan')


class CompanyPlanAdmin(admin.ModelAdmin):
    list_display = ('company', 'plan', 'agents_count', 'begin', 'end', 'payment_period',)
    list_filter = ('plan', 'payment_period', 'begin', 'end',)


class PlanFeatureAdmin(admin.ModelAdmin):
    list_display = ('name', 'title',)
    list_editable = ('title',)


class PromoCodeAdmin(admin.ModelAdmin):
    inlines = [PromoCodeUsageInline]
    fieldsets = (
        (None, {
            'fields': ('code', 'discount_percent', 'is_active', 'billing_id')
        }),
        ('Promo code limitations', {
            'description': _('For each set of objects select those you wish '
                             'current promo code to be availbale exclusively. '
                             'Leave blank if no limitation is desired.'),
            'classes': ('collapse',),
            'fields': ('companies', 'plans', 'payment_period'),
        }),
    )


ncadmin.register(Transaction)
ncadmin.register(Plan, PlanAdmin)
ncadmin.register(PlanSettings, PlanSettingsAdmin)
ncadmin.register(CompanyPlan, CompanyPlanAdmin)
ncadmin.register(PlanFeature, PlanFeatureAdmin)
ncadmin.register(PromoCode, PromoCodeAdmin)
