# Generated by Django 4.2.1 on 2023-05-20 18:01

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('company', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BillingProps',
            fields=[
                ('k', models.CharField(max_length=25, primary_key=True, serialize=False)),
                ('v', models.CharField(max_length=256)),
            ],
        ),
        migrations.CreateModel(
            name='PaymentMethod',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('card_number', models.CharField(blank=True, max_length=64, null=True)),
                ('is_primary', models.BooleanField(default=False)),
                ('type', models.CharField(choices=[('credit_card', 'Credit card'), ('paypal', 'PayPal')], default='credit_card', max_length=16)),
                ('billing_token', models.CharField(blank=True, max_length=32, null=True)),
                ('billing_provider', models.CharField(default='nicechat_mocks', max_length=32)),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='payment_methods', to='company.company')),
            ],
            options={
                'ordering': ['-is_primary'],
            },
        ),
        migrations.CreateModel(
            name='Plan',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=25)),
                ('month_price', models.DecimalField(decimal_places=2, help_text='Price per agent seat per month if client chooses monthly billing', max_digits=10)),
                ('annual_price', models.DecimalField(blank=True, decimal_places=2, help_text='Price per agent seat per month if client chooses yearly billing', max_digits=10, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('order', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('sites_num', models.PositiveSmallIntegerField(blank=True, help_text='The max number of sites available', null=True)),
                ('agents_num', models.PositiveSmallIntegerField(blank=True, help_text='The max number of agent seats available for each company', null=True)),
                ('products_num', models.PositiveSmallIntegerField(blank=True, help_text='The max number of products that can be purchased from chat', null=True)),
                ('chat_stats_period', models.PositiveSmallIntegerField(blank=True, help_text='The max period for chat statistics to be available (in days)', null=True)),
                ('sales_stats_period', models.PositiveSmallIntegerField(blank=True, help_text='The max period for sales statistics to be available (in days)', null=True)),
                ('visitors_stats_period', models.PositiveSmallIntegerField(blank=True, help_text='The max period for sales statistics to be available (in days)', null=True)),
                ('chat_history_period', models.PositiveSmallIntegerField(blank=True, help_text='The max period for chats history to be available (in days)', null=True)),
                ('visitor_data_period', models.PositiveSmallIntegerField(blank=True, help_text='The max period for visitors data to be available (in days)', null=True)),
                ('billing_prefix', models.CharField(blank=True, max_length=25, null=True)),
                ('promotion', models.TextField(default='', help_text='Each row is a separate record at plans list')),
            ],
        ),
        migrations.CreateModel(
            name='PlanFeature',
            fields=[
                ('name', models.CharField(max_length=25, primary_key=True, serialize=False)),
                ('title', models.CharField(max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name='PromoCode',
            fields=[
                ('code', models.CharField(help_text='Textual representation of a code', max_length=128, primary_key=True, serialize=False)),
                ('discount_percent', models.PositiveIntegerField(default=1, help_text='The discount in percents', validators=[django.core.validators.MaxValueValidator(100), django.core.validators.MinValueValidator(1)])),
                ('is_active', models.BooleanField(default=True)),
                ('billing_id', models.CharField(max_length=128, validators=[django.core.validators.MinLengthValidator(1)])),
                ('billing_provider', models.CharField(default='nicechat_mocks', max_length=32)),
                ('payment_period', models.CharField(blank=True, choices=[('month', 'Monthly'), ('year', 'Annually')], max_length=8, null=True)),
                ('companies', models.ManyToManyField(blank=True, related_name='allowed_promo_codes', to='company.company')),
                ('plans', models.ManyToManyField(blank=True, related_name='allowed_promo_codes', to='billing.plan')),
            ],
        ),
        migrations.CreateModel(
            name='PromoCodeUsage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_used', models.DateTimeField(auto_now_add=True)),
                ('code', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='billing.promocode')),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='company.company')),
            ],
        ),
        migrations.AddField(
            model_name='promocode',
            name='usages',
            field=models.ManyToManyField(related_name='used_promo_codes', through='billing.PromoCodeUsage', to='company.company'),
        ),
        migrations.CreateModel(
            name='PlanSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('prop', models.CharField(choices=[('free', 'Free'), ('trial', 'Trial')], max_length=8, unique=True)),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='settings', to='billing.plan')),
            ],
        ),
        migrations.AddField(
            model_name='plan',
            name='features',
            field=models.ManyToManyField(blank=True, to='billing.planfeature'),
        ),
        migrations.CreateModel(
            name='CompanyPlan',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('agents_count', models.PositiveSmallIntegerField()),
                ('begin', models.DateTimeField()),
                ('end', models.DateTimeField()),
                ('payment_period', models.CharField(choices=[('month', 'Monthly'), ('year', 'Annually')], default='month', max_length=8)),
                ('billing_subscription_id', models.CharField(blank=True, max_length=32, null=True)),
                ('billing_provider', models.CharField(default='nicechat_mocks', max_length=32)),
                ('notes', models.TextField(default='{}')),
                ('company', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='billing_plan', to='company.company')),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='billing.plan')),
            ],
        ),
        migrations.CreateModel(
            name='Transaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('plan_period', models.CharField(choices=[('month', 'Monthly'), ('year', 'Annually')], max_length=8)),
                ('timestamp', models.DateTimeField()),
                ('result', models.TextField()),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('sysinfo', models.TextField(null=True)),
                ('status', models.CharField(choices=[('success', 'Success'), ('pending', 'Pending'), ('reqact', 'Require action'), ('failed', 'Failed')], default='pending', max_length=7)),
                ('billing_id', models.CharField(db_index=True, max_length=32)),
                ('billing_provider', models.CharField(default='nicechat_mocks', max_length=32)),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='transactions', to='company.company')),
                ('payment_method', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='billing.paymentmethod')),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='billing.plan')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('billing_id', 'billing_provider')},
            },
        ),
        migrations.CreateModel(
            name='Invoice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('plan_period', models.CharField(choices=[('month', 'Monthly'), ('year', 'Annually')], max_length=8)),
                ('number', models.CharField(max_length=32)),
                ('link', models.CharField(default='', max_length=256)),
                ('timestamp', models.DateTimeField()),
                ('result', models.TextField()),
                ('amount', models.DecimalField(decimal_places=2, max_digits=10)),
                ('sysinfo', models.TextField(null=True)),
                ('status', models.CharField(choices=[('success', 'Success'), ('cancel', 'Canceled'), ('pending', 'Pending'), ('draft', 'Draft'), ('reqact', 'Require action'), ('failed', 'Failed')], default='pending', max_length=7)),
                ('billing_id', models.CharField(db_index=True, max_length=32)),
                ('billing_provider', models.CharField(default='nicechat_mocks', max_length=32)),
                ('company', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='invoices', to='company.company')),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='billing.plan')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'unique_together': {('billing_id', 'billing_provider')},
            },
        ),
    ]
