from rest_framework.permissions import BasePermission
from rest_framework.views import APIView

from ..website.models import Website
from .models import Plan, PlanFeature, PlanSettings
from nicechat.utils.billing import execute_billing


def get_trial_plan():

    trial = PlanSettings.objects.filter(prop=PlanSettings.TRIAL).first()
    if trial:
        return trial.plan

    trial_plan = Plan.objects.create(
        order=0,
        defaults={
            "name": "Trial",
            "month_price": "0.00",
            "annual_price": "0.00",
            "description": "Initial trial",
            "order": 0,
        }
    )

    PlanSettings.objects.create(
        prop=PlanSettings.TRIAL,
        plan=trial_plan
    )

    return trial_plan


def sync_company_billing_transactions(company):
    return execute_billing('sync', [company])


class FeatureAccessAPIView(APIView):
    def get_permissions(self):
        """
        Instantiates and returns the list of permissions that this view requires.
        """
        def _permission_class_instance(perm_class_or_instance):
            """
            Returns either permission class instance if incoming parameter is already an instance of HasAccessToFeatures
            or instantiated `perm_class_or_instance` class instance otherwise              
            """
            if isinstance(perm_class_or_instance, HasAccessToFeatures):
                return perm_class_or_instance
            else:
                return perm_class_or_instance()

        return [_permission_class_instance(permission) for permission in self.permission_classes]


class HasAccessToFeatures(BasePermission):
    """
    Allows access only to users currently working in a company that has
    an active billing plan including all the features provided thorough constructor list argument

    """
    features_required = []

    def __init__(self, features_required=[]):
        assert type(features_required) in [list, tuple]
        self.features_required = features_required

        super(HasAccessToFeatures, self).__init__()

    def has_permission(self, request, view):
        if not request.user.is_authenticated:
            return False

        try:
            if PlanFeature.objects.filter(
                        name__in=self.features_required,
                    pk__in=request.current_site.company.billing_plan.plan.features.all().values_list('pk', flat=True)
                    ).count() == len(self.features_required):
                return True
            else:
                return False

        except Website.DoesNotExist:
            return False
