import json
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
channel_layer = get_channel_layer()
from channels.generic.websocket import WebsocketConsumer
from apps.chat.models import Chat
from nicechat.utils.arbiter_sync import WatcherMiddleware


class MessagesConsumer(WebsocketConsumer):
    method_mapping = {
        "websocket.connect": "connect",
        "websocket.receive": "receive",
        "websocket.send": "websocket_send",
        "websocket.disconnect": "disconnect",
    }

    def connect(self):
        self.kwargs = self.scope['url_route'].get('kwargs', {})
        self.chat_id = self.kwargs.get('chat_id')
        if not self.chat_id:
            self.close()
            return
        chat = Chat.objects.get(pk=self.chat_id)
        if not chat.website or not chat.website.online:
            self.close()
            return
        self.accept()
        self.scope["session"]['site_id'] = chat.website_id
        async_to_sync(channel_layer.group_add)('chat-{}'.format(self.chat_id), self.channel_name)
        async_to_sync(self.channel_layer.group_add)("website_global-%i" % chat.website.id, self.channel_name)

        self.send(text_data='{"type": "welcome"}')

    def receive(self, text_data="", bytes_data=None):
        async_to_sync(channel_layer.group_send)('chat-{}'.format(self.chat_id), {
            'text': '{"type": "new message", "msg_text": "text"}'})
        site_id = self.scope['session'].get('site_id')
        WatcherMiddleware.stats[f"ws:{site_id}"] = 1 + WatcherMiddleware.stats.get(f"ws:{site_id}", 0)

    def websocket_send(self, message):
        if message.get('text') == "setoffline":
            self.close()
            return
        self.send(text_data=message.get('text'))

    def disconnect(self, close_code):
        async_to_sync(channel_layer.group_discard)('chat-{}'.format(self.chat_id), self.channel_name)
        chat = Chat.objects.filter(pk=self.chat_id).first()
        if chat:
            async_to_sync(self.channel_layer.group_discard)("website_global-%i" % chat.website.id, self.channel_name)
