import time
import threading

from django.conf import settings
from nicechat.celery import app
from .utils import do_chat_activity_wdt, do_celery_chat_call_wdt, batam_assign_agents_to_chat
from .notificator import process_notificator_sender, email_message_parser


def chat_activity_wdt(chat_pk, props):
    try:
        first = int(props.get('noagent_first', '0'))
    except ValueError:
        first = None
    if not first:
        first = settings.CHAT_AGENT_CHECK_FIRST

    try:
        second = int(props.get('noagent_second', '0'))
    except ValueError:
        second = None
    if not second:
        second = settings.CHAT_AGENT_CHECK_SECOND

    # for debug run as local thread, for production as celery task
    if settings.IS_DJANGO_DEBUG_RUN:
        t = threading.Thread(target=threading_chat_activity_wdt, args=[chat_pk, first, second])
        t.daemon = True
        t.start()
        batam_assign_agents_to_chat_task.run(args=[chat_pk], kwargs={'small_wait': True})
    else:
        celery_chat_activity_wdt.apply_async(args=[chat_pk, False], countdown=first)
        celery_chat_activity_wdt.apply_async(args=[chat_pk, True], countdown=second)
        batam_assign_agents_to_chat_task.apply_async(args=[chat_pk], kwargs={'small_wait': True})


@app.task
def batam_assign_agents_to_chat_task(chat_pk, small_wait=False):
    return batam_assign_agents_to_chat(chat_pk, small_wait)


@app.task
def move_chat_to_archive_delayed(chat_pk, force_del=False):
    from .models import Chat
    from .utils import move_chat_to_archive
    chat = Chat.objects.filter(pk=chat_pk).first()
    move_chat_to_archive(chat, force_del=force_del)


@app.task
def celery_chat_activity_wdt(chat_pk, is_final=False):
    do_chat_activity_wdt(chat_pk, is_final)

@app.task
def celery_chat_call_wdt(chat_pk):
    do_celery_chat_call_wdt(chat_pk)

@app.task
def process_notificator_sender_task():
    process_notificator_sender()

@app.task
def email_message_parser_task():
    email_message_parser()

def threading_chat_activity_wdt(chat_pk,  first, second):
    time.sleep(first)
    do_chat_activity_wdt(chat_pk, False)
    time.sleep(second - first)
    do_chat_activity_wdt(chat_pk, True)


@app.task
def clean_chatrooms():
    from nicechat.utils.periodical_task import clean_chatrooms
    clean_chatrooms(False)
