from unittest import skip

from rest_framework import status
from rest_framework.test import APIClient

from nicechat.utils.tests import TestCaseWithUsers
from ..chat.models import Message, Chat

from channels.tests import ChannelTestCase
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
channel_layer = get_channel_layer()


@skip('Need to fix the tests')
class ChatAPITest(TestCaseWithUsers, ChannelTestCase):
    class TestData:
        SITE_ID = 'site_id={}'
        WEBSITE = {'title': 'dat.com', 'company': 'http://testserver/api/v1/companies/1/'}
        WEBSITE_V2 = {'title': 'dat1.com', 'company': 'http://testserver/api/v1/companies/2/'}
        COMPANY = {'owner': 'http://testserver/api/v1/users/2/', 'company': 'http://testserver/api/v1/companies/1/',
                   'label': 'dat.com'}
        COMPANY_V2 = {'owner': 'http://testserver/api/v1/users/2/', 'company': 'http://testserver/api/v1/companies/2/',
                      'label': 'dat2.com'}
        MESSAGE = {'text': '0', 'data': 'hello'}
        MESSAGE_V2 = {'text': '1', 'data': 'hello'}

    def setUp(self):
        super(ChatAPITest, self).setUp()
        self.site_api_url = self.api_url + 'websites/'
        self.company_api_url = self.api_url + 'companies/'
        self.messages_api_url = self.api_url + 'messages/?site_id={}'
        self.chat_api_url = self.api_url + 'chat/{}/messages/'
        self.chat_list_url = self.api_url + 'chat/'
        self.chat_current_url = self.api_url + 'chat/{}/'
        self.join_chat_api_url = self.api_url + 'chat/{}/join/'
        self.leave_chat_api_url = self.api_url + 'chat/{}/leave/'
        self.invite_chat_api_url = self.api_url + 'chat/{}/invite/'

        self.client.post(self.company_api_url, self.TestData.COMPANY, format='json')
        self.client.post(self.company_api_url, self.TestData.COMPANY_V2, format='json')
        self.client.post(self.site_api_url, self.TestData.WEBSITE, format='json')
        self.client.post(self.site_api_url, self.TestData.WEBSITE_V2, format='json')

    def test_messages_views(self):
        msg = {'text': '1', 'data': 'test data', 'chat': '2'}
        msg_count = 2

        # Login by user1
        self.login_user(self.user1)

        # Create new messages
        response = self.client.post(self.messages_api_url.format(1), self.TestData.MESSAGE, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        chat_id = response.data['chat_id']

        # Check response data
        response = self.client.get(self.chat_api_url.format(chat_id), format='json')
        self.assertEqual(response.data['results'][0]['data'], self.TestData.MESSAGE['data'])
        self.assertEqual(response.data['results'][0]['text'], self.TestData.MESSAGE['text'])

        # Creating a message with incorrect chat_id
        response = self.client.post(self.chat_api_url.format(chat_id), msg, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        self.assertEqual(Message.objects.count(), msg_count)
        self.assertEqual(Message.objects.all().values('chat_id')[0], Message.objects.all().values('chat_id')[1])

    def test_ws_channels(self):
        # Login by user1
        self.login_user(self.user1)

        # Create new chat
        response = self.client.post(self.messages_api_url.format(1), self.TestData.MESSAGE, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        chat_id_1 = response.data['chat_id']
        self.assertTrue(chat_id_1)

        # Subscribe to active chat
        async_to_sync(channel_layer.group_add)('chat-{}'.format(chat_id_1), 'test-channel_{}'.format(chat_id_1))
        response = self.client.post(self.chat_api_url.format(chat_id_1), self.TestData.MESSAGE, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        # Sent data verification
        result_chan1 = self.get_next_message('test-channel_{}'.format(chat_id_1), require=True)
        self.assertIn('new message', result_chan1.content['text'])
        self.assertIn(self.TestData.MESSAGE['text'], result_chan1.content['text'])

        # Login by user2
        self.login_user(self.user2)

        # Create new chat
        response = self.client.post(self.messages_api_url.format(2), self.TestData.MESSAGE_V2, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        chat_id_2 = response.data['chat_id']
        self.assertTrue(chat_id_2)

        # Subscribe to active chat
        async_to_sync(channel_layer.group_add)('chat-{}'.format(chat_id_2), 'test-channel_{}'.format(chat_id_2))
        self.client.post(self.chat_api_url.format(chat_id_1), self.TestData.MESSAGE, format='json')
        self.client.post(self.chat_api_url.format(chat_id_2), self.TestData.MESSAGE_V2, format='json')

        # Sent data verification
        result_chan2 = self.get_next_message('test-channel_{}'.format(chat_id_2), require=True)
        self.assertIn('new message', result_chan2.content['text'])
        self.assertIn(self.TestData.MESSAGE_V2['text'], result_chan2.content['text'][14:15])
        with self.assertRaises(AssertionError):
            self.get_next_message('test-channel_{}'.format(chat_id_2), require=True)

        self.assertNotEqual(result_chan1.channel, result_chan2.channel)

    def test_permission_views(self):
        # Create chat
        self.login_user(self.user5)
        response = self.client.post(self.messages_api_url.format(1), self.TestData.MESSAGE, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        chat_id_1 = response.data['chat_id']
        # Join to chat by company 1 admin
        response = self.client.post(self.join_chat_api_url.format(chat_id_1), format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # Join to chat by company 1 employee
        self.login_user(self.user1)
        response = self.client.post(self.join_chat_api_url.format(chat_id_1), format='json')
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        # Join to chat by company 2 employee
        self.login_user(self.user2)
        response = self.client.post(self.messages_api_url.format(2), self.TestData.MESSAGE, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        chat_id_2 = response.data['chat_id']
        response = self.client.post(self.join_chat_api_url.format(chat_id_2))
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        # Join to chat by company employee from other company
        response = self.client.post(self.join_chat_api_url.format(chat_id_1))
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_invite(self):
        operator_id_1 = {'operator_id': 1}

        # Create chat and join correct user
        self.login_user(self.user1)
        response = self.client.post(self.messages_api_url.format(1), self.TestData.MESSAGE, format='json')
        chat_id = response.data['chat_id']
        self.client.post(self.invite_chat_api_url.format(chat_id), operator_id_1, format='json')
        response = self.client.get(self.chat_list_url, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        # Invite correct user again
        response = self.client.post(self.invite_chat_api_url.format(chat_id), operator_id_1, format='json')
        self.assertEqual(response.status_code, status.HTTP_302_FOUND)

        # Send empty 'operator_id'
        response = self.client.post(self.invite_chat_api_url.format(chat_id), format='json')
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        # Send incorrect user
        operator_id_2 = {'operator_id': 2}
        response = self.client.post(self.invite_chat_api_url.format(chat_id), operator_id_2, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # Invite admin to chat
        operator_id_3 = {'operator_id': 5}
        self.login_user(self.user5)
        response = self.client.post(self.invite_chat_api_url.format(chat_id), operator_id_3, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # Leave from chat
        response = self.client.get(self.leave_chat_api_url.format(1), format='json')
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        response = self.client.get(self.chat_current_url.format(chat_id), format='json')
        self.assertEqual(response.data['operators_current'], [1])
        self.assertEqual(Chat.objects.filter(operators_current=self.user1, pk=chat_id).count(), 1)

        # Join by employee of another company
        operator_id_4 = {'operator_id': 5}
        self.login_user(self.user2)
        response = self.client.post(self.invite_chat_api_url.format(chat_id), operator_id_4, format='json')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
