import json
import time
import random
import hashlib
import traceback
from django.views.generic import View
from django.template.loader import get_template
from .models import Chat, Message, Attachment, ArchivedChat, Website
from django.http import HttpResponse, HttpResponseNotFound, StreamingHttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.utils.encoding import smart_str
from django.conf import settings
from django.core.files.storage import get_storage_class
from nicechat.utils.helpers import auth_jwt
from nicechat.utils.attach import verify_seed


class AttachDownload(View):

    def options(self, request, *args, **kwargs):
        resp = HttpResponse(status=204)
        setattr(resp, 'is_rendered', True)
        return resp


    def get(self, request, *args, **kwargs):

        msg_id = self.kwargs.get('msg_id')
        if request.META and request.META.get('HTTP_AUTHORIZATION'):
            tok = request.META.get('HTTP_AUTHORIZATION').split(' ')
            if len(tok) == 2:
                result, user = auth_jwt(tok[1])
                if result:
                    msg = Message.objects.filter(pk=msg_id, chat__website__users=user).first()
                    if not msg:
                        msgx = Message.objects.filter(pk=msg_id).first()
                        if msgx and msgx.chat.website_b:
                            if Website.objects.filter(pk=msgx.chat.website_b, users=user).exists():
                                msg = msgx
                    if msg:
                        # if chat is (Message is present) work in old style
                        attach = Attachment.objects.filter(message=msg).first()
                        if attach:
                            tok = self.make_hash(msg_id, self.kwargs.get('chat_id'))
                            resp = HttpResponse(json.dumps({'key': tok}), content_type='application/json')
                            setattr(resp, 'is_rendered', True)
                            return resp
                    else:
                        # if chat is Archived - try to get ArchivedChat object and check message_id in it
                        chat_id = self.kwargs.get('chat_id')
                        achat = ArchivedChat.objects.filter(pk=chat_id, website__users=user).first()
                        if not achat:
                            achatx =  ArchivedChat.objects.filter(pk=chat_id).first()
                            if achatx and achatx.website_b:
                                if Website.objects.filter(pk=achatx.website_b, users=user).exists():
                                    achat = achatx
                        if achat:
                            tok = self.make_hash(msg_id, self.kwargs.get('chat_id'))
                            resp = HttpResponse(json.dumps({'key': tok}), content_type='application/json')
                            setattr(resp, 'is_rendered', True)
                            return resp
        elif self.request.GET.get('ta', None):
            t = self.request.GET.get('ta', None)
            if not self.check_hash(t, msg_id, self.kwargs.get('chat_id')):
                return HttpResponseNotFound()
            msg = Message.objects.filter(pk=msg_id).first()
            if msg:
                # if chat is (Message is present) work in old style
                attach = Attachment.objects.filter(message=msg).first()
                if attach:
                    return self.render_attach_response(msg.text, attach.url, attach.size)
            else:
                # if chat is Archived - try to get ArchivedChat object and check message_id in it
                chat_id = self.kwargs.get('chat_id')
                achat = ArchivedChat.objects.filter(pk=chat_id).first()
                if achat:
                    try:
                        messages = json.loads(achat.messages)
                        attachments = json.loads(achat.attachments)
                    except ValueError:
                        messages = []
                        attachments = []
                    msg = {}
                    attach = {}
                    for _msg in messages:
                        if str(_msg.get('id', '-')) == str(msg_id):
                            msg = _msg
                            break
                    for _attach in attachments:
                        if str(_attach.get('message_id', '-')) == str(msg_id):
                            attach = _attach
                            break
                    if attach and msg:
                        return self.render_attach_response(msg.get('text'), attach.get('url'), attach.get('size'))

        elif self.request.GET.get('t', None) or self.request.GET.get('s', None):
            t = self.request.GET.get('t', None)
            seed = self.request.GET.get('s', None)
            msg = Message.objects.filter(pk=msg_id).first()
            print(t, msg)

            if msg:
                hs = hashlib.sha512(f"{self.request.path}{msg.chat.client_id}".encode()).hexdigest()
                if hs == t or verify_seed(msg.pk, seed):
                    attach = Attachment.objects.filter(message=msg).first()
                    print(attach)
                    if attach:
                        return self.render_attach_response(msg.text, attach.url, attach.size, file_type=attach.file_type, force=False)
        elif self.request.nice_chat_client_id:
            # Visitor can load only active attachments (old-style links)
            msg = Message.objects.filter(pk=msg_id, chat__client_id=self.request.nice_chat_client_id).first()
            if msg:
                attach = Attachment.objects.filter(message=msg).first()
                if attach:
                    return self.render_attach_response(msg.text, attach.url, attach.size)

        return HttpResponseNotFound()

    @staticmethod
    def make_hash(msg_id, chat_id, delta=0, seed=None):
        ts = int(time.time() / 1800 + delta)
        if not seed:
            seed = ''.join(random.choice('abcedf0123456789') for i in range(12))
        sign = "%s-%s-%s-%s-%s" % (seed, settings.SECRET_KEY, ts, msg_id, chat_id)
        return "%s%s" % (seed, hashlib.sha512(sign.encode()).hexdigest())

    def check_hash(self, hash, msg_id, chat_id):
        if len(hash) != 140:
            return False
        seed = hash[:12]
        if self.make_hash(msg_id, chat_id, 0, seed) == hash or self.make_hash(msg_id, chat_id, 1, seed) == hash:
            return True
        return False

    @staticmethod
    def render_attach_response(msg_text, attach_url, attach_size, force=True, file_type=None):
        return render_attach_response(msg_text, attach_url, attach_size, force=force, file_type=file_type)

def render_attach_response(msg_text, attach_url, attach_size, force=True, file_type=None):
    if True:
        if attach_url and isinstance(attach_url, str):
            if ":" in attach_url:
                module, path = attach_url.split(":", 2)
            else:
                module = settings.CHAT_UPLOAD_STORAGE
                path = attach_url

            if module not in settings.ALLOWED_UPLOAD_MODULES:
                return HttpResponseNotFound()

            storage = get_storage_class(module)()

            response = StreamingHttpResponse(storage.open(path))
            if force:
                response['Content-Type'] = 'application/force-download'
                response['Content-Disposition'] = 'attachment; filename=%s' % smart_str(msg_text)
            elif file_type:
                response['Content-Type'] = file_type
            else:
                response['Content-Type'] = 'application/binary'
            return response

        return HttpResponseNotFound()
 