import six

from django.conf import settings
from django.db import models


class Company(models.Model):
    label = models.CharField(max_length=100)
    owner = models.OneToOneField(settings.AUTH_USER_MODEL, related_name='own_company', on_delete=models.CASCADE)
    name = models.CharField(max_length=255, blank=True, null=True)
    itin_vat_num = models.CharField(max_length=30, blank=True, null=True)
    mail_address = models.TextField(blank=True, null=True)
    bill_name = models.CharField(max_length=255, blank=True, null=True)
    bill_country = models.CharField(max_length=255, blank=True, null=True)
    bill_state = models.CharField(max_length=20, blank=True, null=True)
    bill_city = models.CharField(max_length=50, blank=True, null=True)
    bill_address1 = models.CharField(max_length=255, blank=True, null=True)
    bill_address2 = models.CharField(max_length=255, blank=True, null=True)
    bill_address3 = models.CharField(max_length=255, blank=True, null=True)
    bill_zip = models.CharField(max_length=10, blank=True, null=True)
    contact_person = models.TextField(blank=True, null=True)
    contact_email_main = models.EmailField(blank=True, null=True)
    contact_phone_main = models.CharField(max_length=50, blank=True, null=True)
    contact_info = models.TextField(blank=True, null=True)
    billing_id = models.CharField(max_length=32, blank=True, null=True)
    billing_provider = models.CharField(max_length=32, blank=True, null=True)
    partner_id = models.CharField(max_length=64, blank=True, null=True)
    allow_video = models.BooleanField(default=False)
    allow_branding = models.BooleanField(default=False)

    class Meta:
        verbose_name_plural = 'companies'

    def __str__(self):
        return self.label

    def has_access_to_feature(self, feature_name):
        assert isinstance(feature_name, six.string_types)

        return feature_name in self.billing_plan.plan.features.all().values_list('name', flat=True)

    @property
    def primary_payment_method(self):
        # return the first record with is_primary=True OR the first payment method if no primary set
        pm = self.payment_methods.filter(is_primary=True).first()
        if not pm:
            pm = self.payment_methods.first()

        return pm
