import asyncio
from apps.connector.client import ConnectorLibAsyncBase
from .consts import CMD_START, CMD_TXT, CMD_JOIN, CMD_LEAVE, CMD_COMMIT, CMD_STOP, CMD_PRODUCT


class ConnectorLib(ConnectorLibAsyncBase):

    def __init__(self, host, site_id, connector, key=None, cmd_stream=None, recv_stream=None, node_id=None):
        super().__init__(host, site_id, connector, node_id, key)
        self.cmd_stream=cmd_stream
        self.recv_stream=recv_stream
        self.run_read_stream = True

    async def __del__(self):
        self.run_read_stream = False

    async def read_stream(self):
        while self.run_read_stream:
            msg = await self.cmd_stream.get()
            if msg['type'] == CMD_TXT:
                if msg.get('payload') and isinstance(msg['payload'], str) and '%' in msg['payload']:
                    reply_event, reply_cmd = msg['payload'].split('%', 1)
                    await self.receive_message(msg['id'], msg['text'], msg['name'], reply_event=reply_event, reply_cmd=reply_cmd)
                else:
                    await self.receive_message(msg['id'], msg['text'], msg['name'])
            elif msg['type'] == CMD_COMMIT:
                await self.commit(msg['event_id'])
            elif msg['type'] == CMD_START:
                await self.new_chat_room(msg['id'], msg['name'])
            elif msg['type'] == CMD_STOP:
                await self.close_chat_room(msg['id'])

    async def on_connect(self):
        await super().on_connect()
        asyncio.create_task(self.read_stream())

    async def on_send_message(self, room_id, event_id, text, sender_name, buttons):
        await self.recv_stream.put({
            "type": CMD_TXT,
            "room_id": room_id,
            "event_id": event_id,
            "text": text,
            "buttons": buttons
        })

    async def on_agent_join(self, room_id, event_id, agent_name, agent_img):
        await self.recv_stream.put({
            "type": CMD_JOIN,
            "room_id": room_id,
            "event_id": event_id,
            "agent_name": agent_name,
            "agent_img": agent_img,
        })

    async def on_agent_leave(self, room_id, event_id, agent_name):
        await self.recv_stream.put({
            "type": CMD_LEAVE,
            "room_id": room_id,
            "event_id": event_id,
            "agent_name": agent_name,
        })

    async def on_suggestion(self, room_id, event_id, product, sender_name):
        await self.recv_stream.put({
            "type": CMD_PRODUCT,
            "room_id": room_id,
            "event_id": event_id,
            "sender_name": sender_name,
            "product": product
        })
