import json
import time
import multiprocessing
from django.core.management.base import BaseCommand

from .tg import run_instance

class Command(BaseCommand):
    help = 'Connector for telegram'

    def add_arguments(self, parser):
        parser.add_argument('config_file', nargs=1, type=str)

    def handle(self, *args, **options):
        config_file = options.get('config_file').pop()
        f = open(config_file, 'r')
        config = json.loads(f.read())
        f.close()

        process = dict()

        for _site_id, tg_key in config.items():
            if ":" in _site_id:
                site_id, node_id = _site_id.split(":", 1)
            else:
                site_id = _site_id
                node_id = ''
            process[site_id] = {
                'p': multiprocessing.Process(target=run_instance, args=(site_id, tg_key, node_id)),
                'c': (site_id, tg_key, node_id,)
            }
            process[site_id]['p'].start()

        while True:
            try:
                time.sleep(1)
                for rec in process.values():
                    if not rec['p'].is_alive():
                        rec['p'] = multiprocessing.Process(target=run_instance, args=rec['c'])
                        rec['p'].start()
            except KeyboardInterrupt:
                for rec in process.values():
                    if rec['p'].is_alive():
                        rec['p'].terminate()
                break
        print("finishing")
        exit()


    if __name__ == "__main__":
        handle()
