from django.db import models

from apps.chat.models import Connector
from apps.connector.constants import ConnectorType
from apps.website.models import Website


class WebsiteConnectorSettings(models.Model):

    is_active = models.BooleanField(default=False)
    data = models.JSONField(default=dict)

    connector = models.ForeignKey(Connector, related_name='+', on_delete=models.CASCADE)
    website = models.ForeignKey(Website, related_name='integration_settings', on_delete=models.CASCADE)

    class Meta:
        unique_together = ('website', 'connector')
        verbose_name_plural = 'Integration Settings'

    def __str__(self):
        return f'[{self.connector.name}] Integration settings for {self.website}'


class WebhookEvent(models.Model):

    event_id = models.CharField(max_length=255, unique=True, primary_key=True)
    data = models.JSONField(default=dict)
    source = models.CharField(max_length=255, choices=ConnectorType.CHOICES)

    timestamp = models.DateTimeField(auto_now_add=True)

    def __str__(self):
        return self.event_id

