import json
from datetime import datetime
from nicechat.utils.constants import WebsocketEventTypes
from apps.chat.models import Message, Chat, Visitor, VisitedPage
from apps.website.models import Website, Form
from apps.website.serializers import WebsiteSmallSerializer, FormForWidgetSerializer
from apps.chat.serializers import ChatBotSerializer, MessageSerializer
from apps.visitors.utils.conn_info import set_visitor_connection_info
from apps.visitors.ws_utils.monitor import monitor_do_cliopen
from apps.visitors.ws_utils.room import chat_ws_close, chat_ws_message



def cnn_monitor_packet(dt, kwargs, connector_id, connector_name, ch_send):
    # print('QQ>', dt, kwargs, connector_id, connector_name, ch_send)
    if isinstance(dt, dict):
        _type = dt.get('type', None)
        site_id = kwargs.get('site_id')
        # print(dt)
        if _type == WebsocketEventTypes.CH_RCVD_MSG:
            ws_get_msg_room(site_id, dt, connector_id, connector_name, ch_send)
        elif _type == WebsocketEventTypes.CH_NEW_ROOM:
            ws_get_room(site_id, dt, connector_id, connector_name, ch_send)
        elif _type == WebsocketEventTypes.CH_COMMIT:
            ws_commit(site_id, dt, connector_id, connector_name, ch_send)
        elif _type == WebsocketEventTypes.CH_CLOSE_ROOM:
            ws_close_room(site_id, dt, connector_id, connector_name, ch_send)
            
def noop(*args, **kwargs):
    pass

def ws_get_room(site_id, dt, connector_id, connector_name, ch_send):

    session = dt.get('client_id')
    session = f"{connector_name}_{session}"
    token = "cli_wait-%s-%s" % (site_id, session)    
    params = dt.get('params', {})
    params['client_label'] = dt.get('name', 'Guest')
    opened_chat = set_visitor_connection_info(site_id, token, params, session, connector=connector_id)
    if opened_chat == "disabled":
        ch_send(text_data=json.dumps({
            'type': WebsocketEventTypes.CH_NEW_ROOM, 
            'error': 'Cannot create room'
        }))
    chat_id = monitor_do_cliopen(site_id, session, noop, connector=connector_name)

    ch_send(text_data=json.dumps({
        'type': WebsocketEventTypes.CH_NEW_ROOM, 
        'chat_id': chat_id
    }))

def ws_get_msg_room(site_id, dt, connector_id, connector_name, ch_send):
    session = dt.get('client_id')
    session = f"{connector_name}_{session}"
    chat = Chat.objects.filter(
        client=session,
        website_id = site_id,
        is_open=True,
        is_connector=True,
        connector_name=connector_name
    ).first()
    if not chat:
        token = "cli_wait-%s-%s" % (site_id, session)    
        params = dt.get('params', {})
        params['client_label'] = dt.get('name', 'Guest')
        chat_id = monitor_do_cliopen(site_id, session, noop, connector=connector_name)
        chat = Chat.objects.filter(pk=chat_id).first()

    opts = dt.get('opts', {})
    opts['x_tag'] = '1000'
    opts['sender_id'] = session
    msg_id = chat_ws_message(opts, session, {'chat_id': chat.pk}, ch_send)
    if msg_id:
        ch_send(text_data=json.dumps({
            'type': WebsocketEventTypes.CH_RCVD_MSG, 
            'msg_id': msg_id,
            'chat_id': chat.pk
        }))
    else:
        ch_send(text_data=json.dumps({
            'type': WebsocketEventTypes.CH_RCVD_MSG, 
            'error': 'Cannot create message',
            'chat_id': chat.pk
        }))


def ws_close_room(site_id, dt, connector_id, connector_name, ch_send):
    session = dt.get('client_id')
    session = f"{connector_name}_{session}"
    chat = Chat.objects.filter(
        client=session,
        website_id = site_id,
        is_connector=True,
        is_open=True,
        connector_name=connector_name
    ).first()
    if chat:
        chat_ws_close(dt, session, {'chat_id': chat.pk}, ch_send)


def ws_commit(site_id, dt, connector_id, connector_name, ch_send):
    event_id = dt.get('event_id')
    msg = Message.objects.filter(
        pk=event_id,
        chat__website_id=site_id,
        chat__is_connector=True,
        chat__connector_name=connector_name
    ).first()
    if msg:
        if not msg.is_processed:
            msg.is_processed = True
            msg.save()
            ch_send(text_data=json.dumps({
                'type': WebsocketEventTypes.CH_COMMIT, 
                'event_id': msg.pk,
                'processed': True
            }))
        else:
            ch_send(text_data=json.dumps({
                'type': WebsocketEventTypes.CH_COMMIT, 
                'event_id': msg.pk,
                'processed_before': True,
                'processed': True
            }))
    else:
        ch_send(text_data=json.dumps({
            'type': WebsocketEventTypes.CH_COMMIT, 
            'event_id': msg.pk,
            'error': 'Event not raised',
            'processed': False
        }))

