from django.db.models.signals import pre_delete, post_save
from django.dispatch import receiver
import json

from apps.connector.models import WebhookEvent, ConnectorType
from apps.chat.models import Message
#from apps.connector.clients.google import process_google, send_data_google
#from apps.connector.clients.facebook import process_facebook, send_data_facebook
from apps.connector.dispatch import connector_receive_message_from, connector_prepare_message_to, connector_commit_message_event
from apps.connector.consumers import put_ws_cnn


@receiver(connector_commit_message_event)
def commit_message_event(event_id, connector_name, **kwargs):
    msg = Message.objects.filter(
        pk=event_id,
        chat__is_connector=True,
        chat__connector_name=connector_name
    ).first()
    if msg:
        if not msg.is_processed:
            msg.is_processed = True
            msg.save()


@receiver(connector_receive_message_from)
def connector_receive(data, connector, website, **kwargs):
    def _send(text_data):
        connector_prepare_message_to.send_robust(
            sender=website,
            data=json.loads(text_data),
            connector_name=connector.name,
            website_id=website.pk,
        )

    put_ws_cnn(data, {
        'connector_id': connector.pk,
        'connector_name': connector.name,
        'site_id': website.pk,
        'connector_id': connector.pk,
        'send': _send
    })

@receiver(connector_prepare_message_to)
def receive_prepared_message(data={}, connector_name=None, website_id=None, **kwargs):
    #if connector_name == ConnectorType.GOOGLE:
    #    send_data_google(data, website_id, connector_name)
    #if connector_name == ConnectorType.FACEBOOK:
    #    send_data_facebook(data, website_id, connector_name)
    pass


@receiver(post_save, sender=WebhookEvent)
def webhook_event_post_save(sender, instance: WebhookEvent = None, created=False, **kwargs):
    pass
    #print(sender, instance, created)
    #if instance and created:
    #    if instance.source == ConnectorType.GOOGLE:
    #        process_google(instance.data)
    #    if instance.source == ConnectorType.FACEBOOK:
    #        process_facebook(instance.data)
