import json
from threading import Thread
from django.db.models.signals import post_save, pre_delete, m2m_changed
from django.dispatch import receiver

from apps.chat.models import Chat, Message, ArchivedChat
from .models import Search
from .utils import store_archived_chat, store_active_chat, add_row_active_chat


@receiver(post_save, sender=Chat)
def chat_post_save(sender, instance=None, **kwargs):
    store_active_chat(instance)


@receiver(pre_delete, sender=Chat)
def chat_pre_delete(sender, instance=None, **kwargs):
    Search.objects.filter(obj_type=Search.SEARCH_TYPE_CHAT, obj_id=instance.pk).delete()


@receiver(post_save, sender=Message)
def message_post_save(sender, instance=None, **kwargs):
    add_row_active_chat(instance)


@receiver(post_save, sender=ArchivedChat)
def achat_post_save(sender, instance=None, **kwargs):
    store_archived_chat(instance)


@receiver(m2m_changed, sender=Chat.operators_current.through)
def chat_m2m_operators_current_change(instance, action, **kwargs):
    if action in ['post_add', 'post_remove']:
        store_active_chat(instance)
