import json
import sys
from django.db import connection
from .models import Search


def store_archived_chat(achat):
    if not achat:
        return
    text = u''
    companies = set()
    for user in achat.operators.all():
        text += str(user.get_full_name)
        text += "\n"
        text += str(user)
        text += "\n"
        text += str(user.contacts)
        text += "\n"
        companies.add(user.company.pk)

        messages = json.loads(achat.messages)
        for msg in messages:
            _type = msg.get('type') 
            if _type and _type != 'tech':
                text += str(msg.get('text'))
                text += "\n"

    for cmp in list(companies):
        Search.objects.update_or_create(
            obj_id = achat.pk,
            obj_type = Search.SEARCH_TYPE_ARCHIVE,
            company_id = cmp,
            defaults={
                'content': text
            }
        )


def add_row_active_chat(msg):
    if not msg or not msg.chat or msg.type == 'tech':
        return
    text = msg.text + '\n'
    for s in Search.objects.filter(obj_id=msg.chat.pk, type=Search.SEARCH_TYPE_CHAT):
        s.content += text
        s.save(update_fields=('content',))


def store_active_chat(chat):
    if not chat:
        return
    text = u''
    companies = set()
    for user in chat.operators_current.all():
        text += str(user.get_full_name)
        text += "\n"
        text += str(user)
        text += "\n"
        text += str(user.contacts)
        text += "\n"
        companies.add(user.company.pk)

        for msg in chat.message_set.exclude(type='text'):
            text += str(msg.text)
            text += "\n"

    for cmp in list(companies):
        Search.objects.update_or_create(
            obj_id = chat.pk,
            obj_type = Search.SEARCH_TYPE_CHAT,
            company_id = cmp,
            defaults={
                'content': text
            }
        )


def do_search(query, _type, company):
    sql = """
    select
		ts_rank_cd(to_tsvector('english', content), phraseto_tsquery('english', %s)),
		obj_id       
		from fts_search
		where
	    company_id = %s
		AND
		obj_type = %s
		and 
  	to_tsvector('english', content) @@ phraseto_tsquery('english', %s)
		order by 1 desc
    limit 100
    """
    with connection.cursor() as cursor:
        cursor.execute(sql, [query, company,_type, query])
        rows = cursor.fetchall()

    return rows
