from rest_framework import serializers

from ..website.models import Product, ProductsCategory
from ..visitors.models import Order
from nicechat.serialization import ImageUrlSerializer


class MarketplaceOrderSerializer(serializers.ModelSerializer):
    deal = serializers.JSONField(required=False)
    contacts = serializers.JSONField(required=False)

    class Meta:
        model = Order
        fields = ['id', 'deal', 'contacts']
        read_only = True


class MarketplaceProductSerializer(serializers.ModelSerializer):
    id = serializers.SerializerMethodField()
    category = serializers.SerializerMethodField()
    image_url = ImageUrlSerializer(required=False)

    def get_id(self, obj):
        return obj.original_id

    def get_category(self, obj):
        if obj.category:
            return obj.category.original_id
        else:
            return obj.category

    class Meta:
        model = Product
        fields = ('id', 'category', 'title', 'price', 'currency', 'props',
                  'description', 'url', 'image_url', 'in_stock', 'version')
        read_only_fields = ['website']


class MarketplaceCategorySerializer(serializers.ModelSerializer):
    id = serializers.SerializerMethodField()
    parent = serializers.SerializerMethodField()

    def get_id(self, obj):
        return obj.original_id

    def get_parent(self, obj):
        return obj.parent.original_id if obj.parent else None

    class Meta:
        model = ProductsCategory
        fields = ('id', 'name', 'is_enabled', 'parent', 'version')
        read_only_fields = ['website']
