import threading
import json
import time
from django.db.models import Q

from apps.ncbot.client import EVENT_CALLBACK, EVENT_CLOSE_CHAT, EVENT_MESSAGE, EVENT_NEW_CHAT, \
    CMD_SWITCH_AI, CMD_CALLNEXT, CMD_WAITUSER, CMD_CLOSEBOT


class BotProcessor():
    def process_next_step(bot_state, event_type, event_data):

        from apps.chat.models import Message

        print('QQQ', event_type, event_data)

        if event_type == EVENT_NEW_CHAT:
            from apps.ncbot.models import AIAgent
            agent = AIAgent.objects.filter(
                website = bot_state.chat.website,
                ready=True,
                enabled=True
            ).first()
            
            if not agent:
                return CMD_CLOSEBOT

            Message.objects.create(
                chat=bot_state.chat,
                sender_id=0,
                sender_name=agent.name,
                text=agent.greeting,
                is_synthetic=True,
                type=Message.TYPE_OPERATOR
            )

            bot_state.state['agent_name'] = agent.name
            bot_state.state['agent_a_id'] = agent.assistant_id
            from apps.ncbot.integration import openaicli
            bot_state.state['thread_id'] = openaicli.create_thread()
            bot_state.save()
            reply_to = bot_state.chat.message_set.all().order_by('timestamp').first()
            reply_to = reply_to.pk if reply_to else None
            brief = bot_state.chat.obj_props.get('brief')
            ws_cnn_t = threading.Thread(target=process_ai_reply, args=[
                brief, bot_state.chat.pk, bot_state.chat.website_id,
                bot_state.state.get('thread_id'), bot_state.state.get('agent_a_id'),
                bot_state.state.get('agent_name'), False, reply_to])
            ws_cnn_t.isDaemon = True
            ws_cnn_t.start()

            return CMD_WAITUSER

        if event_data.get('datatype') == Message.DATA_ANSWERS_REPLY:
            return CMD_WAITUSER
        
        bot_state.state['cnt'] = 1 + bot_state.state.get('cnt', 0)
        bot_state.save()

        close_bot = True if bot_state.state['cnt'] > 2 else False
        txt = event_data.get('text')
        if txt and bot_state.state.get('agent_a_id'):
            ws_cnn_t = threading.Thread(target=process_ai_reply, args=[
                txt, bot_state.chat.pk, bot_state.chat.website_id,
                bot_state.state.get('thread_id'), bot_state.state.get('agent_a_id'),
                bot_state.state.get('agent_name'), close_bot, event_data.get('pk', None)])
            ws_cnn_t.isDaemon = True
            ws_cnn_t.start()

        if close_bot:
            ws_cnn_t_w = threading.Thread(target=process_ai_wdt, args=[
                bot_state.chat.pk, bot_state.chat.website_id])
            ws_cnn_t_w.isDaemon = True
            ws_cnn_t_w.start()
            return CMD_WAITUSER
        
        return CMD_WAITUSER


def process_ai_reply(txt, chat_id, site_id, thread_id, ass_id, ass_name, close_bot, reply_to):

    from apps.website.ws import website_monitor_packet
    from apps.ncbot.models import BotChat
    from nicechat.utils.constants import WebsocketEventTypes
    from apps.visitors.ws import ws_msg_q
    
    website_monitor_packet({'chatId': chat_id, 'type': WebsocketEventTypes.BACKEND_SITE_START_TYPING}, {'site_id' : site_id}, ass_name, ass_name)
    from apps.ncbot.integration import openaicli

    reply = openaicli.make_reply(txt, thread_id, ass_id)

    ws_msg_q.put(({
        'txt': reply,
        'ass_name': ass_name,
        'chat_id': chat_id,
        'reply_to': reply_to,
        'close_bot': close_bot
    }, 'from_ai'))

    website_monitor_packet({'chatId': chat_id, 'type': WebsocketEventTypes.BACKEND_SITE_STOP_TYPING}, {'site_id' : site_id}, ass_name, ass_name)

    time.sleep(0.1)

    if close_bot:
        BotChat.objects.filter(chat_id=chat_id).delete()

def process_ai_wdt(chat_id, site_id): 
    from apps.ncbot.models import BotChat
    time.sleep(15)
    BotChat.objects.filter(chat_id=chat_id).delete()

