import json
import os
from channels.generic.websocket import WebsocketConsumer
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
from threading import Thread
from queue import SimpleQueue
channel_layer = get_channel_layer()
from apps.ncbot.monitor import bot_monitor_packet, ws_list_rooms, ws_get_website, ws_list_forms
from nicechat.utils.arbiter_sync import WatcherMiddleware

ws_bot_q = SimpleQueue()

def process_ws_bot():
    from nicechat import watcher
    print("Start connector processing", os.getpid())
    while True:
        try:
            text_data, socket = ws_bot_q.get()
            if text_data == "stop_all" and not watcher.can_i_run():
                break

            try:
                dt = json.loads(text_data)
            except ValueError:
                continue
            bot_monitor_packet(dt, socket.scope['url_route'].get('kwargs', {}), socket.send)
        except Exception as e:
            print(e)
    print("Stop connector processing", os.getpid())

ws_bot_t = Thread(target=process_ws_bot)
ws_bot_t.isDaemon = True
ws_bot_t.start()


class BotConsumer(WebsocketConsumer):
    method_mapping = {
        "websocket.connect": "connect",
        "websocket.receive": "receive",
        "websocket.send": "websocket_send",
        "websocket.disconnect": "disconnect",
    }

    def connect(self, *args, **kwargs):
        kwargs = self.scope['url_route'].get('kwargs', {})
        self.scope['session']['site_id'] = kwargs.get('site_id')
        # TODO Auth
        self.accept()
        async_to_sync(self.channel_layer.group_add)("bot_site-%s" % kwargs.get('site_id'), self.channel_name)
        ws_get_website(kwargs.get('site_id'), self.send)
        ws_list_rooms(kwargs.get('site_id'), self.send)
        ws_list_forms(kwargs.get('site_id'), self.send)

    def receive(self, text_data="", bytes_data=None):
        ws_bot_q.put((text_data, self))
        site_id = self.scope['url_route'].get('kwargs', {}).get('site_id')
        WatcherMiddleware.stats[f"ws:{site_id}"] = 1 + WatcherMiddleware.stats.get(f"ws:{site_id}", 0)

    def websocket_send(self, message):
        self.send(text_data=message.get('text'))

    def disconnect(self, *args, **kwargs):
        kwargs = self.scope['url_route'].get('kwargs', {})
        async_to_sync(self.channel_layer.group_discard)("bot_site-%s" % kwargs.get('site_id'), self.channel_name)
