import openai
import re
from django.conf import settings

client = openai.Client(api_key=settings.OPENAI_APIKEY) if settings.OPENAI_APIKEY else None


def upload_file(data):
    datafile = client.files.create(
      file=data,
      purpose='assistants'
    )
    return datafile.id, datafile.status


def get_file_status(file_id):
    file = client.files.retrieve(file_id)
    return file.status if file else None


def create_thread():
    th = client.beta.threads.create()
    return th.id


def make_reply(text, thread_id, assistant_id):

    client.beta.threads.messages.create(
        thread_id=thread_id,
        role="user",
        content=text,
    )

    my_run = client.beta.threads.runs.create(
        thread_id=thread_id,
        assistant_id=assistant_id,
    )
    print(f"This is the run object: {my_run} \n")

    # Step 6: Periodically retrieve the Run to check on its status to see if it has moved to completed
    while my_run.status in ["queued", "in_progress"]:
        keep_retrieving_run = client.beta.threads.runs.retrieve(
            thread_id=thread_id,
            run_id=my_run.id
        )
        print(f"Run status: {keep_retrieving_run.status}")

        if keep_retrieving_run.status == "completed":
            all_messages = client.beta.threads.messages.list(
                thread_id=thread_id
            )
            print(all_messages.data[0].content[0])
            val = str(all_messages.data[0].content[0].text.value)
            val = re.sub(r'【\d+†source】', '', val)
            return val
        elif keep_retrieving_run.status == "failed":
            return 'Sorry'
        elif keep_retrieving_run.status == "queued" or keep_retrieving_run.status == "in_progress":
            pass
        # elif keep_retrieving_run.status == 'requires_action':
        #     tool_calls = keep_retrieving_run.required_action.submit_tool_outputs.tool_calls

        #     tool_outputs = []
        #     for q in tool_calls:
        #         print(q.function.name)
        #         print(q.function.arguments)
        #         import json

        #         tool_outputs.append({
        #             'tool_call_id': q.id,
        #             'output': "Successfull. Check number is UA-123123123" 
        #         })

        #     client.beta.threads.runs.submit_tool_outputs(
        #         thread_id=thread_id, 
        #         run_id=keep_retrieving_run.id,
        #         tool_outputs = tool_outputs,
        #     )
        else:
            print(f"Run status: {keep_retrieving_run.status}")
            break


def create_assistant(name, instructions, files, log):
    log('Try to create Assistant')
    try:
        assistant = client.beta.assistants.create(
            model="gpt-3.5-turbo-1106",
            instructions=instructions,
            name=name,
            file_ids=files,
            tools=[{"type": "retrieval"}]
        )
    except openai.OpenAIError as e:
        log(str(e))
        return None, None
    log(f'Success {assistant.id}')
    return assistant.id, 'ready'


def update_assistant(assistant_id, name, instructions, files, log):
    log(f'Try to modify Assistant {assistant_id}')
    try:
        assistant = client.beta.assistants.update(
            assistant_id=assistant_id,
            instructions=instructions,
            name=name,
            file_ids=files,
        )
    except openai.OpenAIError as e:
        log(str(e))
        return None, None
    log(f'Success')
    return assistant.id, 'ready'
