from django.db import models
from django.conf import settings
from io import BytesIO
import json
from django.utils.text import slugify


class BotChat(models.Model):
    chat = models.ForeignKey('chat.Chat', on_delete=models.CASCADE, related_name='joinedbot')
    botname = models.CharField(max_length=30, db_index=True)
    website = models.ForeignKey('website.Website', on_delete=models.CASCADE)
    state = models.JSONField(default=dict)

    class Meta:
        unique_together = ['chat', 'botname']


class AIPreset(models.Model):
    company = models.ForeignKey('company.Company', on_delete=models.CASCADE)
    website = models.ForeignKey('website.Website', on_delete=models.CASCADE, null=True, blank=True)
    title = models.CharField(max_length=100)
    file_id = models.CharField(max_length=100, blank=True)
    qa_mode = models.BooleanField(default=True)
    state = models.CharField(max_length=30, default='new')
    ready = models.BooleanField(default=False)
    enabled = models.BooleanField(default=True, db_index=True)

    def get_data(self):
        if self.qa_mode == False:
            file = getattr(self, 'aipresetfile', None)
            if file:
                bt = BytesIO(file.data)
                bt.name = file.filename
                return bt
        else:
            data = BytesIO()
            data.write(b'{')
            c = 0
            for qa in self.aipresetqa_set.all():
                if c > 0:
                    data.write(b',')
                data.write(json.dumps({'Question': qa.question, 'Answer': qa.answer}).encode())
                c += 1
            data.write(b'}')
            data.name = slugify(self.title) + ".json"
            data.seek(0)
            return data
        return False


class AIPresetFile(models.Model):
    preset = models.OneToOneField(AIPreset, on_delete=models.CASCADE)
    data = models.BinaryField()
    filename = models.CharField(max_length=300, default='unknown.txt')


class AIPresetQA(models.Model):
    preset = models.ForeignKey(AIPreset, on_delete=models.CASCADE)
    question = models.TextField()
    answer = models.TextField()
    notes = models.CharField(max_length=100, blank=True, default='')


class AIAgent(models.Model):
    name = models.CharField(max_length=100)
    instructions = models.TextField(default='')
    greeting = models.TextField(default='Hello! How can I help you?')
    assistant_id = models.CharField(max_length=100, unique=True)
    website = models.ForeignKey('website.Website', on_delete=models.CASCADE)
    presets = models.ManyToManyField(AIPreset)
    state = models.CharField(max_length=30, default='new')
    ready = models.BooleanField(default=False)
    enabled = models.BooleanField(default=False, db_index=True)

    def addlog(self, txt):
        AIAgentLog.objects.create(log=txt, agent=self)
    

class AIAgentLog(models.Model):
    agent = models.ForeignKey(AIAgent, on_delete=models.CASCADE)
    timestamp = models.DateTimeField(auto_now_add=True)
    log = models.TextField()
