from rest_framework import serializers, exceptions

from .models import AIAgent, AIAgentLog, AIPreset, AIPresetQA


class AIAgentSerializer(serializers.ModelSerializer):
    state = serializers.ReadOnlyField()
    ready = serializers.ReadOnlyField()

    class Meta:
        model = AIAgent
        exclude = ['website', 'assistant_id']

    def validate(self, values):
        presets = values.get('presets', [])
        if len(presets) > 20:
            raise exceptions.ValidationError('You cannot use more than 20 sources to train your agent')

        website=self.context['request'].current_site
        if website:
      
            for p in presets:
                if p.website != website:
                    raise exceptions.ValidationError(f'Cannot find Preset {p.title}[#{p.pk}]')
                if not p.ready:
                    raise exceptions.ValidationError(f'Preset {p.title}[#{p.pk}] is not ready')

        return values


class AIAgentLogSerializer(serializers.ModelSerializer):
    class Meta:
        model = AIAgentLog
        exclude = ['id']


class AIPresetSerializer(serializers.ModelSerializer):
    file = serializers.FileField(required=False)
    qa_mode = serializers.ReadOnlyField()
    state = serializers.ReadOnlyField()
    ready = serializers.ReadOnlyField()
    class Meta:
        model = AIPreset
        exclude = ['website', 'company', 'file_id']

    def create(self, validated_data):
        _validated_data = {}
        _validated_data.update(validated_data)
        if 'file' in _validated_data.keys():
            del _validated_data['file']
        
        return super(AIPresetSerializer, self).create(_validated_data)


class AIPresetQASerializer(serializers.ModelSerializer):
    class Meta:
        model = AIPresetQA
        fields = ['id', 'question', 'answer', 'notes']
