import time, json
from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer
from django.db.models.signals import post_save, post_delete, pre_delete, pre_save, m2m_changed
from django.dispatch import receiver
channel_layer = get_channel_layer()

from nicechat.utils.constants import WebsocketEventTypes
from apps.ncbot.client import process_bot_next_step, EVENT_NEW_CHAT
from apps.ncbot.tasks import upload_file_openai
from apps.ncbot.serializers import AIPresetSerializer, AIAgentSerializer

from .models import BotChat, AIAgent, AIPreset, AIPresetFile


@receiver(post_save, sender=BotChat)
def botchat_post_save(sender, instance=None, created=False, **kwargs):
    if created:
        process_bot_next_step(instance.chat, instance.botname, EVENT_NEW_CHAT, None)


@receiver(pre_save, sender=AIPreset)
def preset_pre_save(sender, instance=None, **kwargs):
    if instance.state == 'processed':
        instance.ready = True
    if instance.state == 'applying':
        instance.ready = False
        upload_file_openai.delay(instance.pk)


@receiver(post_save, sender=AIPreset)
def preset_post_save(sender, instance=None, **kwargs):
    a_key = "adm-website_mon-%i" % instance.website_id
    async_to_sync(channel_layer.group_send)(a_key, {"type": "websocket.send",  
        "text":  json.dumps({"type": WebsocketEventTypes.BACKEND_CHAT_AI_PRESET,
            "data": dict(AIPresetSerializer(instance).data),
            "pk": instance.pk,
            "created": kwargs.get('created', None)
        })
    })


@receiver(post_save, sender=AIPresetFile)
def preset_file_post_save(sender, instance=None, **kwargs):
    if instance.preset.file_id == 'new':
        upload_file_openai.delay(instance.preset.pk)


@receiver(pre_save, sender=AIAgent)
def agent_pre_save(sender, instance=None, **kwargs):
    if instance.state == 'ready':
        instance.ready = True


@receiver(post_save, sender=AIAgent)
def agent_post_save(sender, instance=None, **kwargs):
    a_key = "adm-website_mon-%i" % instance.website_id
    async_to_sync(channel_layer.group_send)(a_key, {"type": "websocket.send",  
        "text":  json.dumps({"type": WebsocketEventTypes.BACKEND_CHAT_AI_AGENT,
            "data": dict(AIAgentSerializer(instance).data),
            "pk": instance.pk,
            "created": kwargs.get('created', None)
        })
    })


@receiver(post_delete, sender=AIPreset)
def post_delete_aipreset(sender, instance=None, **kwargs):
    pass


@receiver(post_delete, sender=AIAgent)
def post_delete_aiagent(sender, instance=None, **kwargs):
    pass
