from nicechat.celery import app
from django.conf import settings
from apps.ncbot.integration import openaicli
from .models import AIPreset


@app.task
def create_or_update_agent(agent_pk):
    from apps.ncbot.models import AIAgent

    a = AIAgent.objects.filter(pk=agent_pk).first()
    if not a:
        return
    if a.assistant_id.startswith('new_'):
        as_id, state = openaicli.create_assistant(
            a.name,
            a.instructions,
            list(a.presets.all().values_list('file_id', flat=True)),
            a.addlog
        )
        a.assistant_id = as_id
    else:
        as_id, state = openaicli.update_assistant(
            a.assistant_id,
            a.name,
            a.instructions,
            list(a.presets.all().values_list('file_id', flat=True)),
            a.addlog
        )
    a.state = state
    a.save()


@app.task
def check_openai_files(redelay=False):
    from apps.ncbot.models import AIPresetFile, AIPreset

    for file in AIPresetFile.objects.filter(
        preset__file_id='new' 
    ):
        upload_file_openai.delay(file.preset.pk)

    for file in AIPresetFile.objects.filter(
        preset__ready=False
    ).exclude(preset__file_id='new'):
        status = openaicli.get_file_status(file.preset.file_id)

    for preset in AIPreset.objects.filter(
        state = 'applying',  
    ):
        upload_file_openai.delay(preset.pk)


@app.task
def upload_file_openai(preset_pk):
    preset = AIPreset.objects.filter(pk=preset_pk).first()
    if not preset:
        return
    file_id, status = openaicli.upload_file(preset.get_data())
    preset.file_id = file_id
    preset.state = status
    preset.ready = False
    preset.save()