from copy import deepcopy
import ipaddress
from nicechat.admins import ncadmin
from django.contrib import admin
from django import forms
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from sorl.thumbnail.admin import AdminImageMixin
from django.core.exceptions import ValidationError

from apps.users.models import UserSiteRole,  User, WhitelistAddr, UserFiltering


class UserRolesInline(admin.TabularInline):
    model = UserSiteRole


class WhitelistAddrAdminForm(forms.ModelForm):
     fields = '__all__'

     def clean(self):
          cleaned_data = super(WhitelistAddrAdminForm, self).clean()

          if cleaned_data["mask"] != 32:
              try:
                  ipaddress.ip_network(f'{cleaned_data["addr"]}/{cleaned_data["mask"]}')
              except ValueError:
                  raise ValidationError({'mask':'Bad netmask'})

          return cleaned_data
     class Meta:
          fields = '__all__'
          model = WhitelistAddr


class WhitelistAddrAdmin(admin.ModelAdmin):
    list_display = ['req']
    form = WhitelistAddrAdminForm
    fieldsets = (
        (None, {"fields": ("addr", "mask")}),
    )


class UserAdmin(AdminImageMixin, BaseUserAdmin):
    def _is_company_owner(self, instance):
        return instance.is_company_owner
    _is_company_owner.boolean = True

    fieldsets = (
        (None, {"fields": ("email", "password")}),
        (_("Personal info"), {"fields": ("first_name", "last_name")}),
        (
            _("Permissions"),
            {
                "fields": (
                    "is_active",
                    "is_staff",
                    "is_superuser",
                    "groups",
                    "user_permissions",
                ),
            },
        ),
        (_("Important dates"), {"fields": ("last_login", "date_joined")}),
    )


    inlines = [UserRolesInline]
    ordering = ("email",)

    list_display = ('email', 'get_full_name', 'company', '_is_company_owner', 'is_staff',)
    list_filter = ('company', 'is_staff', 'is_superuser', 'is_active',)

    def get_fieldsets(self, request, obj=None):
        fieldset_super = super(UserAdmin, self).get_fieldsets(request, obj)
        fieldset_extended = list(deepcopy(fieldset_super))
        fieldset_extended.insert(1, (_('User info'),
                                     {'fields': ('first_name', 'last_name',
                                                 'company', 'avatar',
                                                 'interface_language')}))
        return tuple(fieldset_extended)


ncadmin.register(User, UserAdmin)
ncadmin.register(WhitelistAddr, WhitelistAddrAdmin)
ncadmin.register(UserFiltering)
