import json
from django.contrib.auth import get_user_model
from django.forms import ModelForm
from django.contrib.auth.forms import PasswordResetForm
from django.utils.encoding import force_bytes
from django.utils.http import urlsafe_base64_encode
from django.conf import settings
from django.template.loader import get_template
from django.core.mail import EmailMultiAlternatives
# from django.forms import CharField

class CustomSignupForm(ModelForm):
    def signup(self, request, user):
        user.first_name = self.cleaned_data['first_name']
        user.last_name = self.cleaned_data['last_name']

        user.save()

    class Meta:
        model = get_user_model()
        fields = ('email', 'first_name', 'last_name',)


class AuthPasswordResetForm(PasswordResetForm):
    def send_mail(self, subject_template_name, email_template_name, context, from_email, to_email,
                  html_email_template_name=None):
        
        print(context)

        context.update({
                'domain': settings.DOMAIN_ACC,
                'site_name': settings.BASE_CHAT_NAME,
                'protocol': settings.RESET_EMAIL_PROTO,
                'uidb64': context['user'].pk
            })
        
        print(context)

        template = get_template('users/email/reset_password.html')
        text = get_template('users/email/reset_password.txt')
        subject = get_template('users/email/reset_password.subject')

        text_content = text.render(context)
        html_content = template.render(context)
        subject_content = subject.render(context)
        msg = EmailMultiAlternatives(
            subject_content, text_content,
            settings.DEFAULT_FROM_EMAIL,
            [to_email])

        msg.attach_alternative(html_content, "text/html")
        msg.send(fail_silently=True)
