import json
from django.db import models


class PropsMixin(object):

    def get_props(self):
        return json.loads(self.props)

    def update_props(self, new_props, save_after=False):
        props = json.loads(self.props)
        props.update(new_props)
        self.props = json.dumps(props)
        if save_after:
            self.save(update_fields=['props'])


class Call(PropsMixin, models.Model):
    chat = models.ForeignKey('chat.Chat', related_name='calls', on_delete=models.CASCADE)
    is_active = models.BooleanField(default=False, db_index=True)
    is_started = models.BooleanField(default=False, db_index=True)
    created_at = models.DateTimeField(auto_now_add=True)
    created_by = models.DateTimeField(auto_now=True)
    provider = models.CharField(max_length=30)
    client_accept = models.BooleanField(default=False)
    props = models.TextField(default='{}')


class CallPeer(PropsMixin, models.Model):
    call = models.ForeignKey(Call, related_name='peers', on_delete=models.CASCADE)
    peer = models.CharField(max_length=100, db_index=True)
    props = models.TextField(default='{}')

    class Meta:
        unique_together = ('call', 'peer',)