from rest_framework import serializers

from apps.visitors.models import Statistics, Order, Client, Browser, OperatingSystem
from apps.website.serializers import ProductSerializer
from ..chat.models import Visitor, VisitedPage, BatamStatistics


class VisitedPageSerializer(serializers.ModelSerializer):
    class Meta:
        model = VisitedPage
        fields = '__all__'


class VisitorSerializer(serializers.ModelSerializer):
    visited_pages = VisitedPageSerializer(many=True, read_only=True, required=False)
    sub_state = serializers.ReadOnlyField()
    active_chat_id = serializers.ReadOnlyField()
    cart = serializers.ReadOnlyField()
    client_name = serializers.ReadOnlyField()
    client_typing = serializers.ReadOnlyField()

    class Meta:
        model = Visitor
        fields = ('id', 'client_id', 'client_name', 'website', 'sys_ip', 'sys_lang', 'sys_lang_full',
                  'sys_os', 'sys_time', 'sys_browser', 'ts_open', 'page_url', 'page_title', 'page_ref',
                  'session', 'state', 'geo_city', 'geo_country', 'geo_lat', 'geo_lon', 'visited_pages',
                  'sub_state', 'active_chat_id', 'client_typing', 'page_utm', 'cart'
                  ) #'sys_browser_ver', 'sys_os_ver', 'sys_device', 


class StatisticsSerializer(serializers.ModelSerializer):
    class Meta:
        model = Statistics
        fields = ('id', 'period', 'ts', 'visitors_count', 'browsers', 'desktop_os', 'mobile_os', 'cities', 'pages',
                  'referrers')

class BatamStatisticsSerializer(serializers.ModelSerializer):
    class Meta:
        model = BatamStatistics
        fields = '__all__'


class VisitorsCountSerializer(serializers.ModelSerializer):
    class Meta:
        model = Statistics
        fields = ('ts', 'visitors_count')


class ClientSerializer(serializers.ModelSerializer):
    class Meta:
        model = Client
        fields = '__all__'


class OrdersSerializer(serializers.ModelSerializer):
    products = ProductSerializer(many=True, required=True)
    client = ClientSerializer(many=False, required=True)

    class Meta:
        model = Order
        fields = '__all__'


class BrowsersSerializer(serializers.ModelSerializer):
    class Meta:
        model = Browser
        fields = '__all__'


class OperationSystemsSerializer(serializers.ModelSerializer):
    class Meta:
        model = OperatingSystem
        fields = '__all__'


class ClientPartialSerializer(serializers.ModelSerializer):
    class Meta:
        model = Client
        fields = ('client_id',  'client_name', 'cart',)
