import json
from django.db.models.signals import post_save
from django.dispatch import receiver
from nicechat.utils.callback import make_callback, EVENT_NEW_ORDER, EVENT_NEW_VISITOR, EVENT_PUSH_FORM
from .models import Order, Client, FormResult
from .serializers import ClientPartialSerializer
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
channel_layer = get_channel_layer()
from nicechat.utils.constants import WebsocketEventTypes
from nicechat.utils.callback import make_callback


@receiver(post_save, sender=Order)
def order_post_save(sender, instance=None, created=False, **kwargs):
    # send push only after second Order.save() (order have M2M fields and we must save object before fill M2M)
    if instance and not created:
        make_callback.apply_async(args=(EVENT_NEW_ORDER, instance.website_id, "Order", instance.pk,), countdown=2)


@receiver(post_save, sender=Client)
def client_post_save(sender, instance=None, created=False, **kwargs):
    if instance and created:
        make_callback.apply_async(args=(EVENT_NEW_VISITOR, instance.website_id, "Client", instance.pk,), countdown=5)
    a_key = "adm-website_mon-%i" % instance.website_id
    msg = {"type": "websocket.send",  "text": json.dumps({"type": WebsocketEventTypes.BACKEND_SITE_CLIENT_UPDATE,
                               "data": dict(ClientPartialSerializer(instance=instance)
                                            .to_representation(instance=instance)),
                               "pk": instance.pk
                               })}
    async_to_sync(channel_layer.group_send)(a_key, msg)


@receiver(post_save, sender=FormResult)
def form_result_post_save(sender, instance=None, created=False, **kwargs):
    if instance and created:
        make_callback.apply_async(args=(EVENT_PUSH_FORM, instance.website_id, "FormResult", instance.pk,), countdown=5)
