import json
import traceback
import platform
import os
import time
from django import http
from django.conf import settings
from channels.generic.websocket import WebsocketConsumer
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
from threading import Thread
from queue import SimpleQueue, Empty
channel_layer = get_channel_layer()
from .utils.conn_info import set_visitor_connection_info, remove_visitor_connection
from .ws_utils.monitor import visitor_monitor_packet, get_session_id_from_ws
from .ws_utils.room import visitor_chat_packet, visitor_chat_close, visitor_send_messages, visitor_send_typing, visitor_send_finform
from apps.chat.models import Chat, Visitor
from nicechat.utils.constants import WebsocketEventTypes
from nicechat.utils.arbiter_sync import WatcherMiddleware
from nicechat import watcher



########################## Waiting WS
# TODO Stas M. add unique token for each client tab!!


ws_main_q = SimpleQueue()
ws_msg_q = SimpleQueue()
ws_inq_qs = []


def process_inq_main():
    print("Start event processing", os.getpid())
    from apps.balancer.ws_monitor import balancer_packet
    while True:
        try:
            if not ws_inq_qs:
                time.sleep(1)
            else:
                try:
                    event, data = ws_inq_qs[0].get(timeout=15)
                except Empty:
                    continue
                if event == "stop_all" and not watcher.can_i_run():
                    break
                balancer_packet(data[0], data[1], settings)
        except Exception as e:
            traceback.print_exception(e)
    print("Stop event processing", os.getpid())


def process_ws_main():
    print("Start agent processing", os.getpid())
    while True:
        try:
            text_data, socket = ws_main_q.get()
            if text_data == "stop_all" and not watcher.can_i_run():
                break
            try:
                dt = json.loads(text_data)
            except ValueError:
                continue
            visitor_monitor_packet(dt, socket.scope['url_route'].get('kwargs', {}), socket)
        except Exception as e:
            traceback.print_exception(e)
    print("Stop agent processing", os.getpid())


def process_ws_msg():
    print("Start visitor processing", os.getpid())
    while True:
        try:
            text_data, socket = ws_msg_q.get()
            if text_data == "stop_all" and not watcher.can_i_run():
                break

            session = socket.scope['session'].get('cli-session', None)
            try:
                dt = json.loads(text_data)
            except ValueError:
                continue
            visitor_chat_packet(dt, socket.scope['url_route'].get('kwargs', {}), socket, session)
        except Exception as e:
            traceback.print_exception(e)
    print("Stop visitor processing", os.getpid())

def activity_monitor():
    if settings.NODE_MODE == settings.NODE_MODE_STANDALONE:
        return
    import time
    c = 5
    
    print("Start activity monitor", os.getpid())
    while watcher.can_i_run():
        time.sleep(5)
        c += 1
        if c > 6:
            c = 0
            from nicechat.utils.arbiter import arbiter_request, arbiter_get_hosts

            hosts = arbiter_get_hosts(settings)
            _stats = WatcherMiddleware.stats
            WatcherMiddleware.stats = {}
            data = {
                'event': 'stats',
                'node': platform.node(),
                'mode': settings.NODE_MODE,
                'pid': os.getpid(),
                'http_req': 0 + WatcherMiddleware.http_req,
                'http_s2s': 0 + WatcherMiddleware.http_s2s,
                'ws_req': 0 + WatcherMiddleware.ws_req,
                'active_ws': 0 + WatcherMiddleware.active_ws,
                'stats': _stats
            }
            WatcherMiddleware.http_s2s = 0
            WatcherMiddleware.http_req = 0
            WatcherMiddleware.ws_req = 0
            arbiter_request(hosts, 'monitor', data)
            _stats.clear()
    print("Stop activity monitor", os.getpid())

# if not 'runserver' in sys.argv:
dthw = Thread(name='worker_activity_monitor', target=activity_monitor)
dthw.setDaemon(True)
dthw.start()
    
ws_main_t = Thread(target=process_ws_main)
ws_main_t.isDaemon = True
ws_main_t.start()
ws_msg_t = Thread(target=process_ws_msg)
ws_msg_t.isDaemon = True
ws_msg_t.start()
ws_inq_t = Thread(target=process_inq_main)
ws_inq_t.isDaemon = True
ws_inq_t.start()


class WidgetMainConsumer(WebsocketConsumer):
    method_mapping = {
        "websocket.connect": "connect",
        "websocket.receive": "receive",
        "websocket.send": "websocket_send",
        "websocket.disconnect": "disconnect",
    }

    def connect(self):
        kwargs = self.scope['url_route'].get('kwargs', {})
        self.scope['session']['site_id'] = kwargs.get('site_id')

        site_id = kwargs.get('site_id')
        session = get_session_id_from_ws(self)
        if not session:
            print('Bad session')
            self.close()
            return

        token = "cli_wait-%s-%s" % (site_id, session)
        cli_site = 'cli_site-%s' % site_id
        website_global = 'website_global-%s' % site_id

        opened_chat = set_visitor_connection_info(site_id, token, self, session)
        if opened_chat == "disabled":
            # ch.send(text_data= '{"type": "not allow"}'})
            print('Bad opened_chat')
            self.close()
            return

        self.accept()

        self.scope['session']['cli-wait'] = token
        self.scope['session']['cli-site'] = cli_site
        self.scope['session']['website-global'] = website_global
        self.scope['session']['cli-session'] = session

        async_to_sync(self.channel_layer.group_add)(token, self.channel_name)
        async_to_sync(self.channel_layer.group_add)(cli_site, self.channel_name)
        async_to_sync(self.channel_layer.group_add)(website_global, self.channel_name)

        if opened_chat:
            async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  'text':  json.dumps({
                "type": WebsocketEventTypes.FRONTEND_CHAT_CREATED, "pk": opened_chat.pk
            })})
        else:
            async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  'text':  json.dumps({
                "type": WebsocketEventTypes.FRONTEND_CHAT_NONE
            })})
        self.send(text_data= '{"type": "ready"}')
        #self.scope['session']['cli-headers'] = self.scope['headers']
        from apps.website.models import AgentStatus
        active = 'true' if AgentStatus.objects.filter(website_id=site_id, is_active=True).count() > 0 else 'false'
        self.send(text_data= '{"type": "is_agents", "val": %s}' % active)

    def receive(self,text_data="", bytes_data=None):
        ws_main_q.put((text_data, self))

    def websocket_send(self, message):
        if message.get('text') == "setoffline":
            self.close()
            return
        self.send(text_data=message.get('text'))
        site_id = self.scope['url_route'].get('kwargs', {}).get('site_id')
        WatcherMiddleware.stats[f"ws:{site_id}"] = 1 + WatcherMiddleware.stats.get(f"ws:{site_id}", 0)

    def disconnect(self, close_code):
        token = self.scope['session'].get('cli-wait', None)
        session = self.scope['session'].get('cli-session', None)
        cli_site = self.scope['session'].get('cli-site', None)
        website_global = self.scope['session'].get('website-global', None)        
        if token:
            async_to_sync(self.channel_layer.group_discard)(token, self.channel_name)
            remove_visitor_connection(token, session)
        if cli_site:
            async_to_sync(self.channel_layer.group_discard)(cli_site, self.channel_name)
        if website_global:
            async_to_sync(self.channel_layer.group_discard)(website_global, self.channel_name)


########################## Speakings WS


class WidgetChatConsumer(WebsocketConsumer):
    method_mapping = {
        "websocket.connect": "connect",
        "websocket.receive": "receive",
        "websocket.send": "websocket_send",
        "websocket.disconnect": "disconnect",
    }

    def connect(self):
        kwargs = self.scope['url_route'].get('kwargs', {})

        session = get_session_id_from_ws(self)
        chat_id = kwargs.get('chat_id', 0)
        #check - have this user access for specified chat-room
        if session and chat_id:
            chat = Chat.objects.filter(pk=chat_id, client__peers__peer=session).first()
            if not chat or  not chat.website or chat.website.online == False:
                self.close()
                return
            self.scope['session']['site_id'] = chat.website_id
            token = "website_global-%s" % chat.website_id
            async_to_sync(self.channel_layer.group_add)(token, self.channel_name)
            visitor = Visitor.objects.filter(client_id=session).first()
            self.accept()
            if chat:
                token = "cli_speak-%s" % chat.pk
                async_to_sync(self.channel_layer.group_add)(token, self.channel_name)
                if visitor:
                    for fld in Chat.fields_for_copy:
                        setattr(chat, fld, getattr(visitor, fld))
                    chat.save(update_fields=Chat.fields_for_copy)
                    if chat.operators_current.count() > 0:
                        visitor.state = Visitor.STATE_CHATING
                    else:
                        visitor.state = Visitor.STATE_CALLING
                    visitor.save(update_fields=('state',))
                self.scope['session']['cli-session'] = session
                visitor_send_typing(chat, self)
                visitor_send_messages(chat, self)
                visitor_send_finform(chat, self)
                if not chat.is_open:
                    self.send(text_data= json.dumps({
                        "type": WebsocketEventTypes.FRONTEND_CHAT_CLOSED, "pk": chat_id
                    }))
                return
            else:
                self.send(text_data= json.dumps({
                    "type": WebsocketEventTypes.FRONTEND_CHAT_WAS_CLOSED, "pk": chat_id
                }))
        self.close()

    def receive(self, text_data="", bytes_data=None):
        if text_data == "setoffline":
            self.close()
            return
        ws_msg_q.put((text_data, self))

    def websocket_send(self, message):
        self.send(text_data=message.get('text'))
        site_id = self.scope['session'].get('site_id')
        WatcherMiddleware.stats[f"ws:{site_id}"] = 1 + WatcherMiddleware.stats.get(f"ws:{site_id}", 0)

    def disconnect(self, close_code):
        kwargs = self.scope['url_route'].get('kwargs', {})
        session = self.scope['session'].get('cli-session', None)
        token = "cli_speak-%s" % kwargs.get('chat_id')
        async_to_sync(self.channel_layer.group_discard)(token, self.channel_name)
        site_id = self.scope['session'].get('site_id')
        if site_id:
            async_to_sync(self.channel_layer.group_discard)("website_global-%s" % site_id, self.channel_name)
        visitor_chat_close(kwargs, session)


########################## Client screen WS
# screen is not session-relative (global for chat room)

class WidgetScreenConsumer(WebsocketConsumer):
    method_mapping = {
        "websocket.connect": "connect",
        "websocket.receive": "receive",
        "websocket.send": "websocket_send",
        "websocket.disconnect": "disconnect",
    }

    def connect(self):
        kwargs = self.scope['url_route'].get('kwargs', {})
        self.scope['session']['chat_id'] = kwargs.get('chat_id')
        self.scope['session']['company_token'] = kwargs.get('company_token')
        self.accept()
        token = "cli_screen-%s-%s" % (self.scope['session']['company_token'], self.scope['session']['chat_id'])
        async_to_sync(self.channel_layer.group_add)(token, self.channel_name)

    def receive(self, text_data="", bytes_data=None):
        token = "adm_screen-%s-%s" % (self.scope['session']['company_token'], self.scope['session']['chat_id'])
        async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  "text": text_data})

    def websocket_send(self, message):
        self.send(text_data=message.get('text'))

    def screen_disconnect(self):
        token = "cli_screen-%s-%s" % (self.scope['session']['company_token'], self.scope['session']['chat_id'])
        async_to_sync(self.channel_layer.group_discard)(token, self.channel_name)