from django.utils import timezone
import json
from datetime import timedelta
from django.conf import settings
from nicechat.analytics import event_track, PLAN_CHANGED, TRIAL_DAYS, PLAN_END_NOTIFY
from nicechat.utils.mail import notify_emails
from django.template.loader import render_to_string


def send_message(template, user, context):
    try:
        subject = render_to_string('%s.subject' % template, context)
        print(user.email, subject)
        notify_emails(
            [user.email],
            subject,
            '%s.txt' % template,
            '%s.html' % template,
            context
        )
    except Exception as e:
        print(e)


def check_payments_status_do():
    from apps.billing.models import CompanyPlan, PlanSettings
    plan_free = PlanSettings.objects.filter(prop=PlanSettings.FREE).first()
    plan_trial = PlanSettings.objects.filter(prop=PlanSettings.TRIAL).first()

    if plan_free:
        for cpl in CompanyPlan.objects.filter(end__lt=timezone.now()):
            cpl.begin = timezone.now()
            cpl.end = timezone.now() + timedelta(weeks=52)
            cpl.plan = plan_free.plan
            cpl.agents_count = 1
            cpl.save()

            event_track(PLAN_CHANGED, user_id=cpl.company.owner.id, params={
                'companyId': cpl.company.pk,
                'plan': {
                    'id': plan_free.plan.id,
                    'billing_id': plan_free.plan.billing_prefix,
                }
            })
            send_message('users/email/plan_changed', cpl.company.owner, {
                'plan': plan_free.plan,
                'user': cpl.company.owner,
                'company': cpl.company.label
            })

    if plan_trial:
        for cpl in CompanyPlan.objects.filter(end__gt=timezone.now(), plan=plan_trial.plan):
            tt = cpl.end - timezone.now()

            try:
                notes = json.loads(cpl.notes)
            except ValueError:
                notes = {}

            if notes.get('days', 100) == tt.days:
                continue
            else:
                event_track(TRIAL_DAYS % tt.days, user_id=cpl.company.owner.id, params={
                    'companyId': cpl.company.pk,
                    'plan': {
                        'days': tt.days
                    }
                })
                if tt.days in settings.TRIAL_DAYS_ALERT:
                    send_message('users/email/trial_alert', cpl.company.owner, {
                        'days': tt.days,
                        'user': cpl.company.owner,
                        'company': cpl.company.label
                    })
                notes['days'] = tt.days
                cpl.notes = json.dumps(notes)
                cpl.save(update_fields=['notes'])

        for cpl in CompanyPlan.objects.filter(end__gt=timezone.now()).exclude(plan=plan_trial.plan):
            tt = cpl.end - timezone.now()

            if plan_free and plan_free.plan == cpl.plan:
                continue

            try:
                notes = json.loads(cpl.notes)
            except ValueError:
                notes = {}

            if tt.days > 10:
                continue

            if notes.get('days', 100) == tt.days:
                continue
            else:
                event_track(PLAN_END_NOTIFY, user_id=cpl.company.owner.id, params={
                    'companyId': cpl.company.pk,
                    'plan': {
                        'id': plan_free.plan.id,
                        'billing_id': plan_free.plan.billing_prefix,
                        'days': tt.days
                    }
                })
                notes['days'] = tt.days
                cpl.notes = json.dumps(notes)
                cpl.save(update_fields=['notes'])
