import re
import time
import requests
from datetime import datetime
from django.db.models import Q
from django.conf import settings
from django.utils import timezone

from apps.website.models import Website, Status, AgentStatus
from apps.chat.models import Chat
from apps.chat.utils import batam_assign_agents_to_chat
from nicechat.analytics import event_track, WEBSITE_IS_NOT_REACHABLE, CODE_IS_FOUND_ON_THE_PAGE, CODE_IS_FOUND_IN_GTM, \
    CODE_IS_NOT_FOUND

SCRIPT_CODE = '%s/widget/widget.js' % settings.DOMAIN_WGT
GOOGLE_TM_CODE = 'www.googletagmanager.com/ns.html'


def _fetch_partner_id(page_source_code):
    results = re.findall('(?:{})\?partner_id=([\w\d]+)\"'.format(SCRIPT_CODE), page_source_code)
    partner_id = None

    if len(results):
        partner_id = results[0]

    return partner_id


def check_website_status_do():
    night_run = datetime.now().hour == 0 and datetime.now().minute < 10 
    beginofhour_run = datetime.now().minute < 10 
    websites = Website.objects.filter(Q(status__widget_installed__exact=False) | Q(status__isnull=True))

    for website in websites:
        Status.objects.update_or_create(website=website,
            defaults={'server_code': 200,
                        'widget_installed': True,
                        'allow_site_access': True})


def check_room2agent_status_do():
    for ch in Chat.objects.exclude(is_open=False):
        try:
            check_room2agent_status_process(ch)
        except Exception as e:
            print(e)


def check_room2agent_status_process(ch):
    if ch.operators_current.count() == 0:
        batam_assign_agents_to_chat(ch.pk)
        return True
    
    to_remove = check_room2agent_status_noreply(ch)
    if to_remove:
        print("No-reply check positive", ch)
        return check_room2agent_status_try_join(ch, to_remove)
    
    to_remove = check_room2agent_status_away(ch)
    if to_remove:
        print("Away check positive", ch, to_remove)
        return check_room2agent_status_try_join(ch, to_remove)
    
    return False


def check_room2agent_status_try_join(ch, no_list):
    if batam_assign_agents_to_chat(ch.pk, no_list=no_list, force=True):
        print("Remove all agents")
        from apps.chat.api import ChatViewSet
        for rc in no_list:
            ChatViewSet.provess_chat_leave(rc, ch.pk)
        return True
    else:
        print("This chat need to be agent's reassigned, but there are not free agent")
        return False


def check_room2agent_status_away(ch):
    ops = []
    for op in ch.operators_current.all():
        ast = op.agent_status.filter(website=ch.website).last()
        if not ast or (ast.state != AgentStatus.STATUS_ONLINE and int(time.time() - ast.state_timestamp) > ch.website.auto_assign_away_tout):
            ops.append(op)
    return ops


def check_room2agent_status_noreply(ch):
    for ms in ch.message_set.order_by('-timestamp'):
        if ms.sender_id != ch.client_id and ms.sender_id != '0':
            # if sender is not client and not system (that Agent - all ok)
            return False
        if ms.sender_id == '0' and ms.text and ms.text.startswith('~!JOINED'):
            # if its a operator join message
            if (timezone.now() - ms.timestamp).total_seconds() > ch.website.auto_assign_noreply_tout:
                return ch.operators_current.all()
            else:
                return False
        if ms.sender_id == ch.client_id:
            # if latest sender client, check the message time
            if (timezone.now() - ms.timestamp).total_seconds() > ch.website.auto_assign_noreply_tout:
                # if more that limit - break
                return ch.operators_current.all()
            else:
                # if less that limit - will wait more
                return False
    return False