import braintree
import logging
from nicechat.analytics import event_track, PAYMENT_METHOD_ADDED, NEW_PAYMENT_METHOD_ERROR
from nicechat.utils.billing import BillingError

braintree_logger = logging.getLogger('billing')


def create_customers_braintree_accounts():
    """Creates corresponding Braintree customer accounts for those companies in db that have no billing_id set"""
    from apps.company.models import Company
    from . import PAYMENT_NAME
    for company in Company.objects.filter(billing_id__isnull=True):
        result = braintree.Customer.create({
            # "id": company.pk,
            "first_name": company.owner.first_name,
            "last_name": company.owner.last_name,
            "company": company.label,
            "email": company.owner.email,
            "phone": company.owner.phone_number,
            # "website": company.websites.first().title    # should be a valid domain
        })

        braintree_logger.debug('[apps.billing.utils.create_customers_braintree_accounts]'
                               '"create customer" call result for company {}:\n{}'.format(company.pk,
                                                                                          str(result)))

        if result.is_success:
            company.billing_id = result.customer.id
            company.billing_provider = PAYMENT_NAME
            company.save()


def on_user_create(user):
    from . import PAYMENT_NAME
    braintree_logger.debug('making "create customer" call for user {} (id={})'.format(str(user),
                                                                                      user.own_company.pk))
    result = braintree.Customer.create({
        # "id": user.own_company.pk,
        "first_name": user.first_name,
        "last_name": user.last_name,
        "company": user.own_company.label,
        "email": user.email,
        "phone": user.phone_number,
        # "website": user.own_company.websites.first().title    # should be a valid domain
    })

    braintree_logger.debug('"create customer" call result for user {}):\n{}'.format(user.own_company.pk,
                                                                                    str(result)))
    if result.is_success:
        user.own_company.billing_id = result.customer.id
        user.own_company.billing_provider = PAYMENT_NAME
        user.own_company.save()
    else:
        # todo decide what to do here)
        pass


def on_user_delete(user):
    pass
    # todo user delete signal:
    # https://developers.braintreepayments.com/guides/customers/python#delete
    # result = braintree.Customer.delete("the_customer_id")
    #
    # result.is_success


def create_payment_method(request_user, validated_data):
    braintree_request_data = {
        'customer_id': request_user.own_company.billing_id,
        'payment_method_nonce': validated_data.pop('nonce'),
        'options': {
            'make_default': True,
            'fail_on_duplicate_payment_method': True,
            'verify_card': True,
        }
    }
    from . import PAYMENT_NAME

    braintree_logger.debug('Create subscription request: {}'.format(braintree_request_data))
    result = braintree.PaymentMethod.create(braintree_request_data)
    braintree_logger.debug('Create subscription request result: {}'.format(result))

    if result.is_success:
        # result.customer.card_type
        try:
            validated_data['card_number'] = result.payment_method.masked_number
        except AttributeError:
            # no 'masked_number' attribute for some card types ('Apple Pay - MasterCard' as an example)
            validated_data['company'].card_number = result.payment_method.last_4
        validated_data['billing_token'] = result.payment_method.token
        validated_data['billing_provider'] = PAYMENT_NAME
        request_user.own_company.save()

        event_track(PAYMENT_METHOD_ADDED, user_id=request_user.pk, params={'companyId': request_user.own_company.pk})
    else:
        # https://github.com/braintree/braintree_flask_example/blob/master/app.py#L71
        event_track(NEW_PAYMENT_METHOD_ERROR, user_id=request_user.pk,
                 params={'companyId': request_user.own_company.pk, 'errorMessage': result.message})
        raise BillingError({'Braintree error': result.message})


def get_client_token(user):
    return braintree.ClientToken.generate()