import logging
from nicechat.analytics import event_track, PAYMENT_METHOD_ADDED, NEW_PAYMENT_METHOD_ERROR
from nicechat.utils.billing import BillingError
billing_logger = logging.getLogger('billing')

def create_customers_stripe_accounts():
    from apps.company.models import Company
    from . import stripe_cli
    from . import PAYMENT_NAME

    for company in Company.objects.filter(billing_id__isnull=True):
        try:
            customer = stripe_cli.Customer.create(
                name="%s %s" % (company.owner.first_name,  company.owner.last_name),
                email=company.owner.email,
                phone=company.owner.phone_number,
                metadata={
                    'id': company.owner.pk,
                    'company_id': company.pk,
                    'company_name': company.label
                }
            )
            result = customer.id
            is_success = True
        except stripe_cli.error.StripeError as e:
            result = e.message
            is_success = False

        billing_logger.debug('[apps.billing.utils.create_customers_stripe_accounts]'
                               '"create customer" call result for company {}:\n{}'.format(company.pk,
                                                                                          str(result)))

        if is_success:
            company.billing_id = result
            company.billing_provider = PAYMENT_NAME
            company.save()


def on_user_create(user):
    billing_logger.debug('making "create customer" call for user {} (id={})'.format(str(user),
                                                                                    user.own_company.pk))
    from . import PAYMENT_NAME
    from . import stripe_cli

    try:
        customer = stripe_cli.Customer.create(
            name="%s %s" % (user.first_name, user.last_name),
            email=user.email,
            phone=user.phone_number,
            metadata={
                'id': user.pk,
                'company_id': user.own_company.pk,
                'company_name': user.own_company.label
            }
        )
        result = customer.id
        is_success = True
    except stripe_cli.error.StripeError as e:
        result = e.message
        is_success = False

    billing_logger.debug('"create customer" call result for user {}):\n{}'.format(user.own_company.pk,
                                                                                    str(result)))
    if is_success:
        user.own_company.billing_id = result
        user.own_company.billing_provider = PAYMENT_NAME
        user.own_company.save()
    else:
        # todo decide what to do here)
        pass


def on_user_delete(user):
    pass
    # todo user delete signal:


def create_payment_method(request_user, validated_data):
    from . import PAYMENT_NAME, stripe_cli

    billing_logger.debug('Create subscription request: {}'.format(validated_data))
    try:
        source = stripe_cli.PaymentMethod.attach(
            validated_data.pop('nonce'),
            customer=request_user.own_company.billing_id,
        )
        is_success = True
        result = source.id
        if source.card:
            masked = '%s %s/%s *****%s' % (source.card.brand, source.card.exp_month, source.card.exp_year, source.card.last4)
        else:
            masked = 'n/a'
    except stripe_cli.error.StripeError as e:
        result = e.message
        masked = ""
        is_success = False
    billing_logger.debug('Create subscription request result: {}'.format(result))


    if is_success:
        validated_data['card_number'] = masked
        validated_data['billing_token'] = result
        validated_data['billing_provider'] = PAYMENT_NAME
        request_user.own_company.save()

        event_track(PAYMENT_METHOD_ADDED, user_id=request_user.pk, params={'companyId': request_user.own_company.pk})
    else:
        event_track(NEW_PAYMENT_METHOD_ERROR, user_id=request_user.pk,
                 params={'companyId': request_user.own_company.pk, 'errorMessage': result.message})
        raise BillingError({'Stripe error': result.message})

    # import stripe as stripe_core
    # from stripe.error import StripeError
    # stripe_core.api_key = settings.STRIPE_SECRET
    # try:
    #     customer = stripe_core.Customer.retrieve(customer_id)
    #     card = customer.sources.retrieve(card_token)
    #     card.delete()
    # except StripeError as e:
    #     print(e)
    #     return False
    # return True


def get_client_token(user):
    from django.conf import settings
    return settings.STRIPE_PUBLIC