import logging
import datetime
# from .utils import convert_braintree_status
from nicechat.analytics import event_track, SUBSCRIPTION_CANCELLED, PLAN_CHANGED, PLAN_CHANGE_FAIL
from braintree.exceptions.braintree_error import BraintreeError
from nicechat.utils.billing import BillingError

billing_logger = logging.getLogger('billing')


def apply_payment(billing_id, validated_plan_id, company_plan_price, billing_token, promo_code_obj, agents_count, request_user):

    from .utils import to_stripe_amount
    from . import stripe_cli, PAYMENT_NAME
    from apps.billing.models import Plan
    from django.conf import settings

    obj = Plan.get_by_billing_id(billing_id)
    if obj:
        plan, interval = obj
        if interval == 12:
            plan_id = '%s-%s-y-%s' % (plan.billing_prefix, to_stripe_amount(plan.annual_price), settings.STRIPE_CURRENCY)
            plan_price = to_stripe_amount(plan.annual_price*12)
        else:
            plan_id = '%s-%s-m-%s' % (plan.billing_prefix, to_stripe_amount(plan.month_price), settings.STRIPE_CURRENCY)
            plan_price = to_stripe_amount(plan.month_price)
    else:
        return False, "No Plan found"


    stripe_plan_id = None
    print(stripe_plan_id)
    for plan in stripe_cli.price.Price.list(unit_amount=plan_price, active=True).data:
        if plan_id == plan.metadata.get('name'):
            stripe_plan_id = plan.id
            break
    print(plan)

    if not stripe_plan_id:
        raise BillingError('No billing plan ID')

    result_code = False
    result_obj = {}

    request_data = dict(
        customer=request_user.own_company.billing_id,
        default_payment_method=billing_token,
        items=[
            {
                "price": stripe_plan_id,
                "quantity": int(agents_count)
            },
        ],
    )

    try:
        billing_logger.debug('Create subscription request: {}'.format(request_data))
        subscription = stripe_cli.subscription.Subscription.create(**request_data)
        billing_logger.debug('Create subscription request result: {}'.format(subscription))

        result_code = True
        result_obj['subscription_id'] = subscription.id
        result_obj['billing_provider'] = PAYMENT_NAME

        event_track(PLAN_CHANGED, user_id=request_user.id, params={
            'companyId': request_user.own_company.pk,
            'plan': {
                'id': validated_plan_id,
                'billing_id': billing_id,
            }
        })
    except stripe_cli.error.StripeError as e:
        event_track(PLAN_CHANGE_FAIL, user_id=request_user.id, params={
            'companyId': request_user.own_company.pk,
            'errorMessage': e.message,
            'plan': {
                'id': validated_plan_id,
                'billing_id': billing_id,
            }
        })
        raise BillingError({'Stripe error': e.message})

    return result_code, result_obj


def cancel_subscription(subscription_id, company_plan):
    from . import stripe_cli

    billing_logger.debug('Cancellation request for subscription with id="{}"'.format(
        subscription_id
    ))
    try:
        subscription = stripe_cli.Subscription.retrieve(subscription_id)
        subscription.delete()
        result = 'success'
        is_success = True
        # write_sub_cancel(subscription)
    except stripe_cli.error.StripeError as e:
        result = e.message
        is_success = False

    billing_logger.debug('Cancellation request for subscription with id="{}" result: {}'.format(
        subscription_id, result
    ))

    event_track(SUBSCRIPTION_CANCELLED, params={
        'companyId': company_plan.company.pk,
        'plan': company_plan.plan.get_billing_id(company_plan.payment_period)
    })

    return is_success

def write_sub_cancel(sub):

    from . import stripe_cli, PAYMENT_NAME
    from apps.billing.models import Invoice, Plan
    from apps.company.models import Company

    payment_plan = None
    payment_period = None

    for rec in sub.items.data:
        if hasattr(rec, 'price') and hasattr(rec.price, 'product'):
            product = stripe_cli.Product.retrieve(rec.price.product)
            payment_plan = Plan.objects.filter(billing_prefix=product.name).first()
            payment_period = rec.price.recurring.interval
            break
    if not payment_plan:
        return
    company = Company.objects.filter(billing_id=sub.customer, billing_provider=PAYMENT_NAME).first()

    Invoice(
        billing_id=sub.id,
        billing_provider=PAYMENT_NAME,
        link='',
        company=company,
        plan_period=payment_period,
        plan=payment_plan,
        number='Subscription cancel',
        user=company.owner,
        timestamp=datetime.datetime.now(),
        amount=0,
        result='',
        status=Invoice.SUCCESS,
    ).save()