from django.conf import settings
from django.http import HttpResponseNotFound, HttpResponse
from django.views.generic import View
from django.views.decorators.csrf import csrf_exempt


class WebHookView(View):
    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        return super(WebHookView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        payload = request.body
        sig_header = request.META['HTTP_STRIPE_SIGNATURE']
        from apps.billing.models import BillingProps
        from . import stripe_cli
        from .utils import do_sync_intent, do_sync_invoice, do_sync_sub

        prop = BillingProps.objects.filter(k='stripe_hook').first()
        if not prop:
            return HttpResponse(status=403)
        endpoint_secret = prop.v

        try:
            event = stripe_cli.Webhook.construct_event(
                payload, sig_header, endpoint_secret, tolerance=600
            )
        except ValueError as e:
            return HttpResponse(status=400)
        except stripe_cli.error.SignatureVerificationError as e:
            return HttpResponse(status=400)

        tp = event.data.object.object

        if tp == 'invoice':
            do_sync_invoice(stripe_cli.Invoice.retrieve(event.data.object.id))
        elif tp == 'subscription':
            do_sync_sub(stripe_cli.Subscription.retrieve(event.data.object.id))
        elif tp == 'payment_intent':
            do_sync_intent(stripe_cli.PaymentIntent.retrieve(event.data.object.id))

        return HttpResponse('ok')
