import os
import threading
import sys
from django.utils.module_loading import import_string
from channels.generic.websocket import WebsocketConsumer
from channels.routing import ProtocolTypeRouter, URLRouter
from channels.sessions import SessionMiddlewareStack
from django.core.asgi import get_asgi_application
from django.urls import path

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
import django
django.setup()
from django.conf import settings
from apps.chat.consumers import MessagesConsumer
from apps.ncbot.consumers import BotConsumer
from apps.connector.consumers import ConnectorConsumer
from apps.visitors.ws import WidgetScreenConsumer, WidgetChatConsumer, WidgetMainConsumer
from apps.website.ws import EventStreamConsumer
from nicechat.utils.arbiter_sync import WatcherMiddleware

use_ws = settings.NODE_MODE in [settings.NODE_MODE_WORKER, settings.NODE_MODE_STANDALONE]

router = [
            path("<str:worker_node>/bot-api/<int:site_id>/", BotConsumer.as_asgi()),
            path("<str:worker_node>/api/v1/chat/<int:chat_id>/wsmessages/", MessagesConsumer.as_asgi()),
            path("<str:worker_node>/api/v1/websites/<int:site_id>/event-stream/", EventStreamConsumer.as_asgi()),
            path("<str:worker_node>/widget/<int:site_id>/<int:chat_id>/screen/", WidgetScreenConsumer.as_asgi()),
            path("<str:worker_node>/widget/<int:site_id>/<int:chat_id>/", WidgetChatConsumer.as_asgi()),
            path("<str:worker_node>/widget/<int:site_id>/", WidgetMainConsumer.as_asgi()),
        ] if use_ws else []

if settings.CONNECTOR_ENABLED and use_ws:
    router.append(
        path("<str:worker_node>/api/v1/connector/<slug:connector>/<int:site_id>/<str:key>/", ConnectorConsumer.as_asgi()),
    )
    router.append(
        path("<str:worker_node>/api/v1/connector/<slug:connector>/<int:site_id>/", ConnectorConsumer.as_asgi()),
    )

type_router = {
    "http": WatcherMiddleware(get_asgi_application())
}
if router and use_ws:
    type_router["websocket"] = WatcherMiddleware(SessionMiddlewareStack(URLRouter(router)))
application = ProtocolTypeRouter(type_router)

ws_cli_routes = {}

for up in settings.UPDATE_WS:
    cls = import_string(up)
    update_cli_ws = getattr(cls, 'update_cli_ws', None)
    if update_cli_ws:
        ws_routes = update_cli_ws()
        if ws_routes:
            ws_cli_routes.update(ws_routes)

setattr(WebsocketConsumer, 'ws_cli_routes', ws_cli_routes)

if not '--no-cron' in sys.argv:
    from nicechat.utils.periodical_task import clean_zombie_visitors
    d = threading.Thread(name='reactor_run', target=clean_zombie_visitors)
    d.setDaemon(True)
    d.start()


if settings.ENV_SENTRY_DSN:
    import os
    print("Setup sentry", os.getpid(), flush=True)
    import sentry_sdk

    if sentry_sdk.init(
        dsn=settings.ENV_SENTRY_DSN,
        enable_tracing=True,
    ):
        settings.SENTRY_TRACE = sentry_sdk.capture_exception