import multiprocessing
import socket
import os
import sys
import time
import json
import signal
import traceback
import logging

workers = multiprocessing.cpu_count()
bind = '0.0.0.0:8000'
accesslog = "-"


wsock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
wsock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

from .udpmon import udp_bcserver
multiprocessing.log_to_stderr(logging.DEBUG)
process = multiprocessing.Process(target=udp_bcserver)
process.start()

def broadcast_error(msg, new=False):
    hostname = socket.gethostname()
    _msg = json.dumps(msg)
    _msg = f'{hostname}:{_msg}'.encode()
    _acpt = b'nc_newnode_' if new else b'nc_error_'
    wsock.sendto(_acpt + _msg, ('127.0.0.1', 5006))


def on_starting(server):
    import threading

    def thread_prefork():
        import os
        from django.core.management import call_command
        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
        import django
        django.setup()
        call_command('migrate')

        from django.conf import settings
        if settings.NODE_MODE == settings.NODE_MODE_WORKER:
            from nicechat.utils.arbiter_sync import node_syncing
            print("Worker Syncing...")
            node_syncing()

    # dthw = threading.Thread(name='thread_prefork', target=thread_prefork)
    # dthw.start()
    # dthw.join()

    thread_prefork()

def when_ready(server):
    print("server", server)

    import threading
    def broadcasting():    
        print('Start broadcasting.', os.getpid())
        wsock.bind(("0.0.0.0", 5005))
        wsock.sendto(b"boot", ('127.0.0.1', 5006))

        os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
        import django
        import json
        django.setup()
        from django.conf import settings
        from nicechat import watcher

        if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
            msg = b'iamarbiter' 
        elif  settings.NODE_MODE == settings.NODE_MODE_WORKER:
            msg = f"iamworker:{settings.NODE_PUBLIC_KEY}".encode()
        else:
            msg = b'nc_meetyou'

        wsock.sendto(msg, ('127.0.0.1', 5006))
        wsock.settimeout(2.0)
        c = 0
        while watcher.can_i_run():
            c += 1
            if c > 35:
                c = 0
                try:
                    if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
                        wsock.sendto(b'iamarbiter', ('127.0.0.1', 5006))
                    if settings.NODE_MODE == settings.NODE_MODE_WORKER:
                        msg = f"iamworker:{settings.NODE_PUBLIC_KEY}".encode()
                        wsock.sendto(_msg, ('127.0.0.1', 5006))
                except Exception as e:
                    pass
            try:
                data, addr = wsock.recvfrom(1024)
            except socket.timeout:
                continue
            sys.stderr.write(f"{data}, {addr}\n")
            wsock.sendto(f"@{addr[0]}".encode(), ('127.0.0.1', 5006))
            watcher.event_bc(data, addr)
            if data and (data in [b'nc_meetyou', b'iambalancer'] or data.startswith(b'nc_newnode')):
                wsock.sendto(msg, (addr[0], 5005))
            if data and data.startswith(b'confupdate'):
                if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
                    if data == b'confupdate:all' or data == b'confupdate:arbiter':
                        time.sleep(1)
                        sys.stderr.write(f'Restarting.... {os.getpid()}\n')
                        _num_workers = server.num_workers
                        server.num_workers = 0
                        server.kill_workers(9)
                        server.stop()
                        sys.stderr.write(f'Stoping.... {os.getpid()}\n')
                        os._exit(0)
                elif settings.NODE_MODE == settings.NODE_MODE_WORKER:
                    if data == b'confupdate:all':
                        time.sleep(20)
                        sys.stderr.write(f'Restarting.... {os.getpid()}\n')
                        _num_workers = server.num_workers
                        server.num_workers = 0
                        server.kill_workers(9)
                        server.stop()
                        sys.stderr.write(f'Stoping.... {os.getpid()}\n')
                        os._exit(0)
                    elif data == f'confupdate:{settings.WORKER_NODE_NAME}'.encode():
                        sys.stderr.write(f'Restarting.... {os.getpid()}\n')
                        _num_workers = server.num_workers
                        server.num_workers = 0
                        server.kill_workers(9)
                        server.stop()
                        sys.stderr.write(f'Stoping.... {os.getpid()}\n')
                        os._exit(0)

        print('Stop broadcasting.', os.getpid())

    bcth = threading.Thread(name='broadcasting', target=broadcasting)
    bcth.setDaemon(True)
    bcth.start()

def pre_fork(server, worker):
    from nicechat import watcher
    watcher.add_bc_pre(worker)

def post_worker_init(worker):
    from nicechat import watcher
    watcher.add_bc_post(worker)

def worker_exit(server, worker):
    print(server, worker)
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "nicechat.settings")
    import django
    import platform
    django.setup()
    from django.conf import settings
    from nicechat import watcher

    watcher.apply_stop()
    watcher.clear_bc(worker)
    from nicechat.utils.arbiter import arbiter_request, arbiter_get_hosts, mark_node_status
    data = {
        'event': 'stopworker',
        'node': platform.node(),
        'mode': settings.NODE_MODE,
        'pid': os.getpid(),
    }
    if settings.NODE_MODE == settings.NODE_MODE_ARBITER:
        mark_node_status(data)
    elif settings.NODE_MODE == settings.NODE_MODE_WORKER:
        hosts = arbiter_get_hosts(settings)
        print(hosts)
        arbiter_request(hosts, 'monitor', data)
    print('DOWN', worker.pid, flush=True)

def on_exit(server):
    wsock.sendto(b"exit", ('127.0.0.1', 5006))
    print('DOWN-ALL',flush=True)
    wsock.close()
    try:
        wsock.shutdown(1)
    except OSError:
        pass
    import threading
    for t in threading.enumerate():
        print(t, flush=True)
    process.terminate()
    sys.exit(0)
