from django.core.management.base import BaseCommand
import string
import random
import argparse
from django.contrib.auth import get_user_model
from django.conf import settings


class Command(BaseCommand):
    help = 'Run on empty-DB'

    BROWSERS = [{ "name": "Unknown Browser", "type": "d", "icon": "browser/unknown.png"}, { "name": "AndroidBrowser", "type": "d", "icon": "browser/android.png"}, { "name": "BrowserNG", "type": "d", "icon": "browser/ng.png"}, { "name": "Chrome", "type": "d", "icon": "browser/chrome.png"}, { "name": "Firefox", "type": "d", "icon": "browser/firefox.png"}, { "name": "Galeon", "type": "d", "icon": "browser/galeon.png"}, { "name": "Konqueror", "type": "d", "icon": "browser/opera.png"}, { "name": "Microsoft Internet Explor", "type": "d", "icon": "browser/ie.png"}, { "name": "MSEdge", "type": "d", "icon": "browser/edge.png"}, { "name": "NetFront", "type": "d", "icon": "browser/netfront.png"}, { "name": "Netscape", "type": "d", "icon": "browser/netscape.png"}, { "name": "Opera", "type": "d", "icon": "browser/opera.png"}, { "name": "Safari", "type": "d", "icon": "browser/safari.png"}, { "name": "SeaMonkey", "type": "d", "icon": "browser/seamonkey.png"}, { "name": "Other", "type": "d", "icon": "browser/other.png"}]
    OS = [{ "name": "Android", "type": "m", "icon": "os/android.png"}, { "name": "Blackberry", "type": "m", "icon": "os/blackberry.png"}, { "name": "ChromeOS", "type": "d", "icon": "os/chrome.png"}, { "name": "Debian", "type": "d", "icon": "os/linux.png"}, { "name": "Ipad", "type": "m", "icon": "os/apple.png"}, { "name": "iPhone", "type": "m", "icon": "os/apple.png"}, { "name": "iOS", "type": "m", "icon": "os/apple.png"}, { "name": "Linux", "type": "d", "icon": "os/linux.png"}, { "name": "Macintosh", "type": "d", "icon": "os/apple.png"}, { "name": "Mac", "type": "d", "icon": "os/apple.png"}, { "name": "MacOS", "type": "d", "icon": "os/apple.png"}, { "name": "PlayStation", "type": "d", "icon": "os/android.png"}, { "name": "Symbian", "type": "m", "icon": "os/android.png"}, { "name": "Ubuntu", "type": "d", "icon": "os/linux.png"}, { "name": "Unknown OS", "type": "d", "icon": "os/unknown.png"}, { "name": "Windows Mobile", "type": "m", "icon": "os/windows.png"}, { "name": "Windows Server", "type": "d", "icon": "os/windows.png"}, { "name": "Windows Phone", "type": "m", "icon": "os/windows.png"}, { "name": "Windows", "type": "d", "icon": "os/windows.png"}]
    THEMES = [ { 'name': 'cacce8', 'icon_color': '#656ab3', 'buttons_color': '#656ab3', 'buttons_text_color': '#ffffff', 'system_text_color': '#4e5169', 'bg_color': '#cacce8', 'notifications_text_color': '#222222', 'border_color': '#ffffff', 'outchat_bg_color': '#686b89', 'intchat_bg_color': '#ffffff', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#586778', 'operator_icon_bg': '#4e5169', 'operator_border': '#ffffff' }, { 'name': 'eae9d5', 'icon_color': '#ff9360', 'buttons_color': '#ff9360', 'buttons_text_color': '#ffffff', 'system_text_color': '#666464', 'bg_color': '#eae9d5', 'notifications_text_color': '#222222', 'border_color': '#ffffff', 'outchat_bg_color': '#666464', 'intchat_bg_color': '#f0f3f6', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#666464', 'operator_icon_bg': '#666464', 'operator_border': '#ffffff' }, { 'name': 'cad8e8', 'icon_color': '#2c9ff9', 'buttons_color': '#2c9ff9', 'buttons_text_color': '#ffffff', 'system_text_color': '#586778', 'bg_color': '#cad8e8', 'notifications_text_color': '#222222', 'border_color': '#ffffff', 'outchat_bg_color': '#586778', 'intchat_bg_color': '#f0f3f5', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#586778', 'operator_icon_bg': '#586778', 'operator_border': '#ffffff' }, { 'name': 'd4ecd8', 'icon_color': '#08c255', 'buttons_color': '#08c255', 'buttons_text_color': '#ffffff', 'system_text_color': '#395b47', 'bg_color': '#d4ecd8', 'notifications_text_color': '#222222', 'border_color': '#ffffff', 'outchat_bg_color': '#5e8d72', 'intchat_bg_color': '#f0f3f5', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#395b47', 'operator_icon_bg': '#395b47', 'operator_border': '#ffffff' }, { 'name': 'e3d0e8', 'icon_color': '#a66ab2', 'buttons_color': '#a66ab2', 'buttons_text_color': '#ffffff', 'system_text_color': '#5a455f', 'bg_color': '#e3d0e8', 'notifications_text_color': '#222222', 'border_color': '#ffffff', 'outchat_bg_color': '#8c7391', 'intchat_bg_color': '#ffffff', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#5a455f', 'operator_icon_bg': '#5a455f', 'operator_border': '#ffffff' }, { 'name': 'e9e9ee', 'icon_color': '#62696f', 'buttons_color': '#62696f', 'buttons_text_color': '#ffffff', 'system_text_color': '#586778', 'bg_color': '#e9e9ee', 'notifications_text_color': '#222222', 'border_color': '#ffffff', 'outchat_bg_color': '#62696f', 'intchat_bg_color': '#f0f3f5', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#586778', 'operator_icon_bg': '#586778', 'operator_border': '#ffffff' }, { 'name': '1d242b', 'icon_color': '#2a3642', 'buttons_color': '#ffffff', 'buttons_text_color': '#1d242b', 'system_text_color': '#c3cdd9', 'bg_color': '#1d242b', 'notifications_text_color': '#ffffff', 'border_color': '#2a3642', 'outchat_bg_color': '#2a3642', 'intchat_bg_color': '#a3bacc', 'input_bg_color': '#2a3642', 'input_text_color': '#ffffff', 'input_buttons_color': '#c3cdd9', 'operator_icon_bg': '#2a3642', 'operator_border': '#c3cdd9' }, { 'name': '394868', 'icon_color': '#3a4868', 'buttons_color': '#ffffff', 'buttons_text_color': '#394868', 'system_text_color': '#cadcf1', 'bg_color': '#394868', 'notifications_text_color': '#ffffff', 'border_color': '#252e42', 'outchat_bg_color': '#252e42', 'intchat_bg_color': '#acbfec', 'input_bg_color': '#252e42', 'input_text_color': '#ffffff', 'input_buttons_color': '#cadcf1', 'operator_icon_bg': '#252e42', 'operator_border': '#cadcf1' }, { 'name': '4fbdae', 'icon_color': '#4fbdae', 'buttons_color': '#ffffff', 'buttons_text_color': '#4fbdae', 'system_text_color': '#e2fffb', 'bg_color': '#4fbdae', 'notifications_text_color': '#ffffff', 'border_color': '#ffffff', 'outchat_bg_color': '#1c675d', 'intchat_bg_color': '#bbf6ee', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#252e42', 'operator_icon_bg': '#1c675d', 'operator_border': '#ffffff' }, { 'name': 'cd6060', 'icon_color': '#cd6060', 'buttons_color': '#ffffff', 'buttons_text_color': '#cd6060', 'system_text_color': '#ffe7e7', 'bg_color': '#cd6060', 'notifications_text_color': '#ffffff', 'border_color': '#ffffff', 'outchat_bg_color': '#813d3d', 'intchat_bg_color': '#f0f3f5', 'input_bg_color': '#ffffff', 'input_text_color': '#222222', 'input_buttons_color': '#813d3d', 'operator_icon_bg': '#813d3d', 'operator_border': '#ffffff' } ]
    FEATURES = [{ "name": "products_add",  "title": "Create Products" }, { "name": "products_import",  "title": "Products import (csv/google merchant)" }, { "name": "file_sharing",  "title": "Files sharing" }, { "name": "add_forms",  "title": "Add forms" }, { "name": "custom_forms",  "title": "Customizable forms" }, { "name": "add_quick_answers",  "title": "Add quick reply templates" }, { "name": "quick_answers",  "title": "Quick reply templates" }, { "name": "suggest_product",  "title": "Suggest products" }, { "name": "typing_preview",  "title": "Typing preview" }]

    def add_arguments(self, parser):
        parser.add_argument('--nosuperuser', action=argparse.BooleanOptionalAction)

    def handle(self, *args, **options):
        nosuperuser = options['nosuperuser']

        from apps.visitors.models import OperatingSystem, Browser
        from apps.website.models import WidgetTheme
        from apps.billing.models import PlanFeature, Plan, PlanSettings

        if settings.NODE_MODE not in [settings.NODE_MODE_ARBITER, settings.NODE_MODE_STANDALONE]:
            self.stdout.write("Not-Arbiter node. Finish.")
            exit()

        if Browser.objects.count() > 0:
            self.stdout.write("Browsers already exists. Skip.")
        else:
            for rec in self.BROWSERS:
                Browser.objects.create(**rec)

        if OperatingSystem.objects.count() > 0:
            self.stdout.write("OperatingSystem already exists. Skip.")
        else:
            for rec in self.OS:
                OperatingSystem.objects.create(**rec)

        if WidgetTheme.objects.count() > 0:
            self.stdout.write("WidgetThemes already exists. Skip.")
        else:
            for rec in self.THEMES:
                WidgetTheme.objects.create(**rec)

        if PlanFeature.objects.count() > 0:
            self.stdout.write("PlanFeatures already exists. Skip.")
        else:
            for rec in self.FEATURES:
                PlanFeature.objects.create(**rec)
        
        if Plan.objects.count() > 0:
            self.stdout.write("Plan already exist. Skip.")
        else:
            pln = Plan.objects.create(
                name="Default",
                month_price=0,
                annual_price=0,
                description="Default plan",
                promotion="Default plan"
            )
            for ft in PlanFeature.objects.all():
                pln.features.add(ft)
            PlanSettings.objects.create(plan=pln, prop=PlanSettings.FREE)
            PlanSettings.objects.create(plan=pln, prop=PlanSettings.TRIAL)

        if nosuperuser:
            return

        User = get_user_model()
        if User.objects.count() > 0:
            self.stdout.write("Users already exists. Skip.")
        else:
            email = f'ncadmin@{settings.ENV_MAIN_DOMAIN}'
            usr = User(
                email=email,
                is_staff=True,
                is_superuser=True
            )
            letters = string.digits + string.ascii_uppercase + string.ascii_lowercase
            pwd = ''.join(random.choice(letters) for i in range(12))

            OKGREEN = '\033[92m'
            FAIL = '\033[91m'
            ENDC = '\033[0m'
            BOLD = '\033[1m'
            usr.set_password(pwd)
            usr.save()

            print(f"{OKGREEN}==========================================")
            self.stdout.write("\n\n\n\n")
            print(f"    {FAIL} ## Superuser created ##")
            print(f"    {ENDC}{FAIL} Login: {BOLD}{email}")
            print(f"    {ENDC}{FAIL} Password: {BOLD}{pwd}")
            self.stdout.write("\n\n\n\n")
            print(f"{ENDC}{OKGREEN}=========================================={ENDC}")
