
from django.core.management.base import BaseCommand
import string
import random
from datetime import timedelta
from django.utils import timezone


class Command(BaseCommand):
    help = 'Create company for load-testing'

    def add_arguments(self, parser):
        parser.add_argument("agents_count", nargs=1, type=int)#, default=25)

    @staticmethod
    def get_rndstr():
        return ''.join(random.choice(string.digits + string.ascii_uppercase + string.ascii_lowercase) for i in range(10))

    def handle(self, *args, **options):
        agents_count = options['agents_count'].pop()

        from apps.billing.models import PlanSettings, CompanyPlan
        from apps.company.models import Company
        from apps.users.models import User, UserSiteRole, UserRoles
        from apps.website.models import Website, Status
        from apps.website.tasks import create_website_celery

        email = 'load_test-admin_' + self.get_rndstr() + '@loadtest.com'
        pwd = self.get_rndstr()
        usr = User(email=email)
        usr.set_password(pwd)
        usr.is_active = True
        usr.save()
        cmp_tok = self.get_rndstr()
        cmpg = Company(
            name="LoadTest-company_" + cmp_tok,
            owner=usr
        )
        cmpg.save()
        ps = PlanSettings.objects.filter(prop=PlanSettings.FREE).first()
        CompanyPlan(
            company=cmpg, 
            plan=ps.plan, 
            begin=timezone.now(),
            end=timezone.now() + timedelta(days=3650),
            agents_count=agents_count+1
        ).save()

        usr.company = cmpg
        usr.save()

        ws = Website(domain='company-%s.com' % cmp_tok, title=cmpg.name, company=cmpg)
        ws.save()
        create_website_celery.run(ws.pk)

        Status.objects.update_or_create(website=ws, defaults={'server_code': 200, 'widget_installed': True, 'allow_site_access':True})
        UserSiteRole.objects.update_or_create(user=usr, site=ws, defaults={'role': UserRoles.SITE_ADMIN})
        ws.users.add(usr)

        print("Company_ID", cmpg.pk)
        print("Website_ID", ws.pk)
        print("Admin", email, pwd)
        print('Employees:')


        for j in range(0, agents_count):
            e_email = 'load_test_' + self.get_rndstr() + '@loadtest.com'
            e_pwd = self.get_rndstr()
            _usr = User(email=e_email)
            _usr.set_password(e_pwd)
            _usr.is_active = True
            _usr.company = cmpg
            _usr.save()
            UserSiteRole.objects.update_or_create(user=_usr, site=ws, defaults={'role': UserRoles.SITE_AGENT})
            ws.users.add(_usr)
            print(f"{e_email};{e_pwd}")
