"""
Django settings for nicechat project.

Generated by 'django-admin startproject' using Django 1.10.2.

For more information on this file, see
https://docs.djangoproject.com/en/1.10/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.10/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
import sys
import time

from celery.schedules import crontab
from datetime import timedelta
from django.conf.global_settings import STATICFILES_FINDERS

BASE_DIR = os.path.dirname(os.path.abspath(__file__))

if os.getenv('USE_DOTENV_FILE', default=False):
    from nicechat.utils.readenv import read_env
    read_env(os.getenv('USE_DOTENV_FILE'))
elif os.path.exists('/var/lib/nicechat/nicechat.conf'):
    from nicechat.utils.readenv import read_env
    read_env('/var/lib/nicechat/nicechat.conf')
elif os.path.exists('/etc/nicechat.conf'):
    from nicechat.utils.readenv import read_env
    read_env('/etc/nicechat.conf')

ENV_PUBLIC_ROOT = os.getenv('PUBLIC_ROOT', default='/nc/public/')

ENV_SECRET_KEY = os.getenv('SECRET_KEY', default='QWEasd123QWEasd123QWEasd123QWEasd123')
ENV_DATA_KEY = os.getenv('DATA_KEY', default='111222333444555666777888999000aaabbbcccdddeeefff111222333444555666777888999000aaabbbcccdddeeefff')

ENV_POSTGRES_NAME = os.getenv('POSTGRES_NAME', default='nicechat')
ENV_POSTGRES_USER = os.getenv('POSTGRES_USER', default='nicechat')
ENV_POSTGRES_PASSWORD = os.getenv('POSTGRES_PASSWORD', default='nicechat')
ENV_POSTGRES_HOST = os.getenv('POSTGRES_HOST', default='postgres')

ENV_ARBITER_CA = os.getenv('ARBITER_CA', default=None)
ENV_ARBITER_HOST = os.getenv('ARBITER_HOST', default=None)
ENV_ARBITER_HOSTKEY = os.getenv('ARBITER_HOSTKEY', default=None)
ENV_BALANCER_SIGN = os.getenv('BALANCER_SIGN', default='')
ENV_STANDALONE = os.getenv('STANDALONE', default='0') in ['1', 'true', 'yes']

NODE_MODE_ARBITER = 'arbiter'
NODE_MODE_WORKER = 'worker'
NODE_MODE_STANDALONE = 'standalone'

NODE_MODE = None
NODE_PRIVATE_KEY = None
NODE_PUBLIC_KEY = None
NODE_ARBITER_PUB = None
NODE_ARBITER_HOST = []
WORKER_NODE_NAME = None

SESSION_COOKIE_SECURE = True
CSRF_COOKIE_SECURE = True

SENTRY_TRACE = None

from nicechat.utils.arbiter import load_private_key, load_public_key, arbiter_request, dump_public_key
from nicechat.utils.crypt import get_data_keys, decrypt_dict


if ENV_ARBITER_CA:
    NODE_MODE = NODE_MODE_ARBITER
    ROOT_URLCONF = 'nicechat.urls_arbiter'
    NODE_PRIVATE_KEY = load_private_key(ENV_ARBITER_CA)
    NODE_PUBLIC_KEY = dump_public_key(NODE_PRIVATE_KEY, True)
    NODE_ARBITER_PUB = NODE_PUBLIC_KEY
    db = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': ENV_POSTGRES_NAME,
            'USER': ENV_POSTGRES_USER,
            'PASSWORD': ENV_POSTGRES_PASSWORD,
            'HOST': ENV_POSTGRES_HOST,
            'PORT': '5432',
            'CONN_MAX_AGE': 30,
            'OPTIONS': {}
        }
    }
    from .settingvalidation import read_db
    code, data, conn = read_db(db)
    if code == 0:
        data = decrypt_dict(data, ENV_DATA_KEY)

        ENV_SENTRY_DSN = os.getenv('SENTRY_DSN', default=None)
        ENV_OPENAI_APIKEY = os.getenv('OPENAI_APIKEY', default=None)
        ENV_OPENAI_ORG = os.getenv('OPENAI_ORG', default=None)

        ENV_DEBUG_EMAILS = os.getenv('DEBUG_EMAILS', default='0') in ['1', 'true', 'yes']

        ENV_ALL_DOMAINS = data['domains']

        ENV_MAIN_DOMAIN = ENV_ALL_DOMAINS[0]

        ENV_MAIN_NAME = data.get('main_name', 'ChatUsNow')
        ENV_GOOGLE_MAP_KEY = data.get('google_map_key', '')

        ENV_SEND_ERRORS_EMAILS = data.get('send_errors_emails', '').split(',')
        ENV_INNER_NOTIFICATION_EMAILS = data.get('inner_notification_emails', '').split(',')

        ENV_SMTP_HOST = data.get('smtp_host', 'smtp.gmail.com')
        try:
            ENV_SMTP_PORT = int(data.get('smtp_port', '587'))
        except Exception:
            ENV_SMTP_PORT = 587
        ENV_SMTP_USER = data.get('smtp_user', '')
        ENV_SMTP_PASSWORD = data.get('smtp_password', '')
        ENV_SMTP_MODE = data.get('smtp_mode', 'tls')
        ENV_EMAIL_FROM = data.get('email_from', 'Notify <no_reply@nice.chat>')

        ENV_MFA = data.get('mfa', 'none')
        ENV_DEBUG = data.get('debug', '0') in ['1', 'true', 'yes']

        ENV_REDIS_HOST = data.get('redis_host', 'redis')
        ENV_REDIS_PORT = data.get('redis_port', '6379')
        ENV_REDIS_DB = data.get('redis_db', '0')
        ENV_REDIS_SSL = data.get('redis_ssl', '0') in ['1', 'true', 'yes']
        redis_user = data.get('redis_user', '')
        redis_password = data.get('redis_password', '')
        ENV_ATTACH_SIGN_SECRET = data.get('attach_sign_secret', 'FH70BfQ9AKuqohZplaBaRtLHVg3ONheUL1CsLTvg3WIj4z3oNtFytCKYIXj')
        ENV_REDIS_AUTH = f'{redis_user}:{redis_password}@' if redis_user else ''

        ENV_SENTRY_DSN = data.get('sentry_dsn', None)
        ENV_OPENAI_APIKEY = data.get('openai_apikey', None)
        ENV_OPENAI_ORG = data.get('openai_org', None)

        ENV_DEBUG_EMAILS = data.get('debug_emails', '0') in ['1', 'true', 'yes']

    else:
        from .settingvalidation import base_server
        sys.stderr.write(data)
        base_server(data)
        exit()

if ENV_ARBITER_HOSTKEY:
    NODE_MODE = NODE_MODE_WORKER
    from .settingvalidation import HostManager
    ROOT_URLCONF = 'nicechat.urls_worknode'
    if isinstance(ENV_ARBITER_HOSTKEY, str) and '.' in ENV_ARBITER_HOSTKEY:
        n, w, a = ENV_ARBITER_HOSTKEY.split('.', 2)
        NODE_ARBITER_PUB = load_public_key(a)
        NODE_PRIVATE_KEY = load_private_key(w)
        NODE_PUBLIC_KEY = dump_public_key(NODE_PRIVATE_KEY, False)
        WORKER_NODE_NAME = n
        if NODE_ARBITER_PUB and NODE_PRIVATE_KEY:
            NODE_ARBITER_HOST = HostManager()
            arb_data = None
            if not NODE_ARBITER_HOST:
                print('There are no known arbiters.', flush=True)
                import socket
                wsock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
                wsock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
                wsock.bind(("0.0.0.0", 5005))
                wsock.settimeout(5.0)
                while True:
                    print('Try to find arbiter..', flush=True)
                    wsock.sendto(b'nc_meetyou', ('127.0.0.1', 5006))
                    try:
                        data, addr = wsock.recvfrom(1024)
                    except socket.timeout:
                        continue
                    if data == b'iamarbiter':
                        NODE_ARBITER_HOST.add_host(addr[0])
                        print('Found arbiter!', flush=True)
                        sys.exit(0)
            c = 0
            while not arb_data:
                arb_data = arbiter_request(NODE_ARBITER_HOST, 'init', keys=[NODE_PRIVATE_KEY, NODE_ARBITER_PUB, NODE_PUBLIC_KEY])
                if not arb_data:
                    print('Wait for arbiter...', flush=True)
                    time.sleep(3)
                    c += 1
                    if c > 20:
                        NODE_ARBITER_HOST.empty()
                        os._exit(0)

            if arb_data:
                ENV_ALL_DOMAINS = arb_data.get('ALL_DOMAINS', [])
                ENV_MAIN_DOMAIN = arb_data.get('MAIN_DOMAIN')
                ENV_MAIN_NAME = arb_data.get('MAIN_NAME')
                ENV_GOOGLE_MAP_KEY = arb_data.get('GOOGLE_MAP_KEY')

                ENV_SEND_ERRORS_EMAILS = arb_data.get('SEND_ERRORS_EMAILS', [])
                ENV_INNER_NOTIFICATION_EMAILS = arb_data.get('INNER_NOTIFICATION_EMAILS', [])

                ENV_SMTP_HOST = arb_data.get('SMTP_HOST')
                ENV_SMTP_MODE = arb_data.get('SMTP_MODE', 'tls')
                ENV_SMTP_PORT = arb_data.get('SMTP_PORT')
                ENV_SMTP_USER = arb_data.get('SMTP_USER')
                ENV_SMTP_PASSWORD = arb_data.get('SMTP_PASSWORD')
                ENV_EMAIL_FROM = arb_data.get('EMAIL_FROM')

                ENV_POSTGRES_NAME = arb_data.get('POSTGRES_NAME')
                ENV_POSTGRES_USER = arb_data.get('POSTGRES_USER')
                ENV_POSTGRES_PASSWORD = arb_data.get('POSTGRES_PASSWORD')
                ENV_POSTGRES_HOST = arb_data.get('POSTGRES_HOST')

                ENV_DEBUG = arb_data.get('DEBUG', False)
                ENV_MFA = arb_data.get('MFA', 'none')

                ENV_REDIS_HOST = arb_data.get('REDIS_HOST')
                ENV_REDIS_PORT = arb_data.get('REDIS_PORT')
                ENV_REDIS_SSL = arb_data.get('REDIS_SSL', False)
                ENV_REDIS_DB = arb_data.get('REDIS_DB')
                ENV_SENTRY_DSN = arb_data.get('SENTRY_DSN', None)
                ENV_PUBLIC_ROOT = arb_data.get('PUBLIC_ROOT', '/nc/public/')

                _REDIS_USER = arb_data.get('REDIS_USER')
                _REDIS_PASS = arb_data.get('REDIS_PASS')
                ENV_REDIS_AUTH = f'{_REDIS_USER}:{_REDIS_PASS}@' if _REDIS_USER else ''

                ENV_ATTACH_SIGN_SECRET = arb_data.get('ATTACH_SIGN_SECRET', 'FH70BfQ9AKuqohZplaBaRtLHVg3ONheUL1CsLTvg3WIj4z3oNtFytCKYIXj')
    
                ENV_SENTRY_DSN = arb_data.get('SENTRY_DSN', None)
                ENV_OPENAI_APIKEY = arb_data.get('OPENAI_APIKEY', None)
                ENV_OPENAI_ORG = arb_data.get('OPENAI_ORG', None)

                ENV_DEBUG_EMAILS = arb_data.get('DEBUG_EMAILS', '0') in ['1', 'true', 'yes']

                ENV_SECRET_KEY = arb_data.get('SECRET_KEY')
                _arb_wn = arb_data.get('NODE', None)
                if WORKER_NODE_NAME != _arb_wn:
                    print(f'Wrong worker node name. Arbiter return {_arb_wn}, at configs {WORKER_NODE_NAME}')
                    exit()
            else:
                print('Cannot read data from Aribter')
                exit()

NODE_NOT_SYNC = False
BALANCER_SIGN = ENV_BALANCER_SIGN

if ENV_STANDALONE:
    NODE_MODE = NODE_MODE_STANDALONE
    WORKER_NODE_NAME = 'chatting'
    NODE_NOT_SYNC = True
    ROOT_URLCONF = 'nicechat.urls_standalone'
    domains = os.getenv('MAIN_DOMAIN', default='nice.chat')
    if ',' in domains:
        ENV_ALL_DOMAINS = domains.split(',')
    else:
        ENV_ALL_DOMAINS = [domains]

    ENV_MAIN_DOMAIN = ENV_ALL_DOMAINS[0]

    SESSION_COOKIE_SECURE = True
    CSRF_COOKIE_SECURE = True

    ENV_MAIN_NAME = os.getenv('MAIN_NAME', default='Nice.Chat')
    ENV_GOOGLE_MAP_KEY = os.getenv('GOOGLE_MAP_KEY', default='')

    ENV_SEND_ERRORS_EMAILS = os.getenv('SEND_ERRORS_EMAILS', default='').split(',')
    ENV_INNER_NOTIFICATION_EMAILS = os.getenv('INNER_NOTIFICATION_EMAILS', default='').split(',')

    ENV_SMTP_HOST = os.getenv('SMTP_HOST', default='smtp.gmail.com')
    try:
        ENV_SMTP_PORT = int(os.getenv('SMTP_PORT', default='587'))
    except Exception:
        ENV_SMTP_PORT = 587
    ENV_SMTP_USER = os.getenv('SMTP_USER', default='')
    ENV_SMTP_PASSWORD = os.getenv('SMTP_PASSWORD', default='')
    ENV_SMTP_MODE = os.getenv('SMTP_MODE', default='tls')
    ENV_EMAIL_FROM = os.getenv('EMAIL_FROM', default='Notify <no_reply@nice.chat>')

    ENV_MFA = os.getenv('MFA', default='none')
    ENV_DEBUG = os.getenv('DEBUG', default='0') in ['1', 'true', 'yes']

    ENV_REDIS_HOST = os.getenv('REDIS_HOST', default='redis')
    ENV_REDIS_PORT = os.getenv('REDIS_PORT', default='6379')
    ENV_REDIS_DB = os.getenv('REDIS_DB', default='0')
    ENV_REDIS_SSL = os.getenv('REDIS_SSL', default='0') in ['1', 'true', 'yes']
    ENV_REDIS_AUTH = os.getenv('REDIS_AUTH', default='')
    ENV_ATTACH_SIGN_SECRET = os.getenv('ATTACH_SIGN_SECRET', default='FH70BfQ9AKuqohZplaBaRtLHVg3ONheUL1CsLTvg3WIj4z3oNtFytCKYIXj')
    ENV_REDIS_AUTH = f'{ENV_REDIS_AUTH}@' if ENV_REDIS_AUTH else ''

    ENV_SENTRY_DSN = os.getenv('SENTRY_DSN', default=None)
    ENV_OPENAI_APIKEY = os.getenv('OPENAI_APIKEY', default=None)
    ENV_OPENAI_ORG = os.getenv('OPENAI_ORG', default=None)

    ENV_DEBUG_EMAILS = os.getenv('DEBUG_EMAILS', default='0') in ['1', 'true', 'yes']


if not NODE_PRIVATE_KEY and not ENV_STANDALONE:
    print("Unknown mode. Please set ARBITER_CA or ENV_ARBITER_HOST&ENV_ARBITER_HOSTKEY variables")
    from .gconfig import broadcast_error
    import sys
    if sys.argv[0].endswith('celery'):
        import time
        time.sleep(10)
        exit()
    broadcast_error("", new=True)
    from .settingvalidation import base_server
    base_server()
    exit()

if len(ENV_DATA_KEY) != 96 and not NODE_MODE == NODE_MODE_STANDALONE:
    print('DATA_KEY have wrong length 96 hex required')
    exit()

DATA_KEY_HS, DATA_KEY_IV = get_data_keys(ENV_DATA_KEY) if not NODE_MODE == NODE_MODE_STANDALONE else (None, None)

PROJECT_NAME = ENV_MAIN_NAME
# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.10/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = ENV_SECRET_KEY

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = ENV_DEBUG

ALLOWED_HOSTS = ['*']

SITE_ID = 1

DOMAIN_BASE = ENV_MAIN_DOMAIN
DOMAIN_ACC = f'account.{ENV_MAIN_DOMAIN}'
DOMAIN_MGT = f'manage.{ENV_MAIN_DOMAIN}'
DOMAIN_DIR = f'direct.{ENV_MAIN_DOMAIN}'
DOMAIN_WGT = f'widget.{ENV_MAIN_DOMAIN}'
DOMAIN_CDN = f'widget.{ENV_MAIN_DOMAIN}'
DOMAIN_SUP = f'support.{ENV_MAIN_DOMAIN}'

GOOGLE_MAP_KEY = ENV_GOOGLE_MAP_KEY

BASE_CHAT_NAME = ENV_MAIN_NAME

# Application definition

INSTALLED_APPS = [
    'daphne',
    'apps.users',

    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',

    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'sorl.thumbnail',
    'rest_framework',
    'rest_framework.authtoken',
    'dj_rest_auth',
    'dj_rest_auth.registration',
    'drf_yasg',
    'channels',
    'nicechat',

    'apps.billing',
    'apps.chat',
    'apps.company',
    'apps.website',
    'apps.visitors',
    'apps.marketplaces',
    'apps.video',
    'apps.ncbot',
    'apps.connector',
    'apps.balancer',
]

REST_AUTH = {
    'USE_JWT': True,
    'SESSION_LOGIN': False,
}

AUTH_USER_MODEL = 'users.User'

REDIS_URL = f'rediss://{ENV_REDIS_AUTH}{ENV_REDIS_HOST}:{ENV_REDIS_PORT}/{ENV_REDIS_DB}?ssl_cert_reqs=none' if \
    ENV_REDIS_SSL else f'redis://{ENV_REDIS_AUTH}{ENV_REDIS_HOST}:{ENV_REDIS_PORT}/{ENV_REDIS_DB}'

OLD_PASSWORD_FIELD_ENABLED = True

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels_redis.pubsub.RedisPubSubChannelLayer",
#        "BACKEND": "channels_redis.core.RedisChannelLayer",
        "CONFIG": {
            "hosts": [REDIS_URL],
        },
    },
}

AUTHENTICATION_BACKENDS = (
    # Needed to login by username in Django admin, regardless of `allauth`
    'django.contrib.auth.backends.ModelBackend',
    'nicechat.middleware.NCJSONWebTokenAuthentication',
    # `allauth` specific authentication methods, such as login by e-mail
    "allauth.account.auth_backends.AuthenticationBackend",
)

SIMPLE_JWT = {
    "ACCESS_TOKEN_LIFETIME": timedelta(hours=12),
    "REFRESH_TOKEN_LIFETIME": timedelta(days=14),
}

REST_FRAMEWORK = {
    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework.renderers.JSONRenderer',
        # 'rest_framework.renderers.BrowsableAPIRenderer',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'nicechat.middleware.NCJSONWebTokenAuthentication',
    ),
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema',
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.LimitOffsetPagination',
    'PAGE_SIZE': 20
}

ADMINS = [
 ('admin', eml.strip()) for eml in ENV_SEND_ERRORS_EMAILS
]
MANAGERS = ADMINS

LOGIN_REDIRECT_URL = '/'

ACCOUNT_USER_MODEL_USERNAME_FIELD = None
ACCOUNT_EMAIL_REQUIRED = True
ACCOUNT_USERNAME_REQUIRED = False
ACCOUNT_AUTHENTICATION_METHOD = 'email'
ACCOUNT_CONFIRM_EMAIL_ON_GET = True
ACCOUNT_LOGOUT_ON_GET = True
ACCOUNT_SIGNUP_FORM_CLASS = 'apps.users.forms.CustomSignupForm'
ACCOUNT_ADAPTER='apps.users.adapter.AllAuthBasedAdapter'
USERNAME_FIELD='email'

AGENT_CONFIRM_URL = f'https://{DOMAIN_ACC}/users/agent-confirm-email/:token/'
AUTH_CONFIRM_URL = f'https://{DOMAIN_ACC}/registration/account-confirm-email/:token/'

VISITOR_COOKIE_NAME = "nicechat"

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'nicechat.middleware.CheckOriginMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'nicechat.middleware.LanguageCookieMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'nicechat.middleware.CurrentSiteMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

TFA = {
    'awssns': 'nicechat.tfa.TfaAwsSnS',
    'gtotp': 'nicechat.tfa.TfaTOtp',
    'email': 'nicechat.tfa.TfaEmail',
}
TFA_ENABLE = ENV_MFA in TFA.keys()
TFA_MODULES = [TFA[ENV_MFA]] if TFA_ENABLE else [] 

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',

                'nicechat.context_processors.head_body_settings',
            ],
        },
    },
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': ENV_POSTGRES_NAME,
        'USER': ENV_POSTGRES_USER,
        'PASSWORD': ENV_POSTGRES_PASSWORD,
        'HOST': ENV_POSTGRES_HOST,
        'PORT': '5432',
        'CONN_MAX_AGE': 30,
    }
}

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

LANGUAGES = (
    ('en', 'English'),
    ('ru', 'Russian'),
)

DEFAULT_LANGUAGE = 'en'
LANGUAGE_COOKIE_NAME = 'language'

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale'),
)

STATIC_URL = '/djstatic/'
MEDIA_URL = f'https://{DOMAIN_CDN}/media/'

PUBLIC_ROOT = os.path.abspath(ENV_PUBLIC_ROOT)
MEDIA_ROOT = os.path.join(PUBLIC_ROOT, 'media')
STATIC_ROOT = os.path.join(PUBLIC_ROOT, 'djstatic')
TRY_TO_UPDATE_EXISTS_STAT = False

STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'static'),
)

RELEASE_ID = None
DESC_ID = '123'

TRIAL_DAYS_ALERT = [5,1]

LOGGING = {
'version': 1,
'disable_existing_loggers': False,
'filters': {
    'require_debug_false': {
        '()': 'django.utils.log.RequireDebugFalse',
    },
    'require_debug_true': {
        '()': 'django.utils.log.RequireDebugTrue',
    },
},
'formatters': {
    'django.server': {
        '()': 'django.utils.log.ServerFormatter',
        'format': '[%(server_time)s] %(message)s',
    }
},
'handlers': {
    'console': {
        'level': 'INFO',
        'filters': ['require_debug_true'],
        'class': 'logging.StreamHandler',
    },
    # Custom handler which we will use with logger 'django'.
    # We want errors/warnings to be logged when DEBUG=False
    'console_on_not_debug': {
        'level': 'WARNING',
        'filters': ['require_debug_false'],
        'class': 'logging.StreamHandler',
    },
    'django.server': {
        'level': 'INFO',
        'class': 'logging.StreamHandler',
        'formatter': 'django.server',
    },
    'mail_admins': {
        'level': 'ERROR',
        'filters': ['require_debug_false'],
        'class': 'django.utils.log.AdminEmailHandler'
    }
},
'loggers': {
    'django': {
        'handlers': ['console', 'mail_admins', 'console_on_not_debug'],
        'level': 'INFO',
    },
    'django.server': {
        'handlers': ['django.server'],
        'level': 'INFO',
        'propagate': False,
    },
}
}

OPENAI_APIKEY = ENV_OPENAI_APIKEY
OPENAI_ORG = ENV_OPENAI_ORG

NOT_USE_WIDGET_SSL = False

AUTO_OPEN_LINK_SECRET = ENV_SECRET_KEY

WIDGET_SESSION_IN_GET_PARAM = "nc_client"
VISITOR_UPLOAD_ROOT = os.path.join(PUBLIC_ROOT, 'visitor_upload')

GLOBAL_WELCOME_MESSAGE = ('Hello!', 'How can I help you?',)
ALLOW_CHAT_AUTOCREATE = False

CHAT_UPLOAD_STORAGE = 'django.core.files.storage.FileSystemStorage'
ALLOWED_UPLOAD_MODULES = ['django.core.files.storage.FileSystemStorage']

# WS notification
CHAT_SERIALIZER = 'apps.chat.serializers.ChatSerializer'
MESSAGE_SERIALIZER = 'apps.chat.serializers.MessageSerializer'
VISITOR_SERIALIZER = 'apps.visitors.serializers.VisitorSerializer'

ALLOWED_BOTS = {
    'ai': 'apps.ncbot.bot.BotProcessor'
}
AI_BOT = 'ai'

WATCH_WS_ENABLE = False
WATCH_WS_TIMEDELTA_S = 120
WATCH_WS_SECRET = ''
WATCH_WS_GET_USER_PK = lambda a: a


VISITOR_REMOVE_TIMEOUT = 30  # wait X seconds before remove Visitor from list (after it was disconnected)
REMOVE_CLOSED_CHAT_M = 0
REMOVE_CLOSED_CHAT_ONLY_CLI = True
ARCHIVE_UNACTIVE_CHAT_M = 60
ARCHIVE_UNACTIVE_CHAT_CONN_M = 60 * 24 * 14  # 2 week
VISITOR_ATTACH_LIMIT = 1.5
AGENT_ATTACH_LIMIT = 3.0
AGENT_LEAVE_TIMEOUT_S = 20
CHAT_AGENT_CHECK_FIRST = 30
CHAT_AGENT_CHECK_SECOND = 90

#TODO - self-host
BOT_HOST = 'ws://backend:8000'

BROKER_URL = REDIS_URL
CELERY_RESULT_BACKEND = REDIS_URL

CELERY_TASK_RESULT_EXPIRES = 18000
CELERY_ACCEPT_CONTENT = ['pickle', 'json', 'msgpack', 'yaml']
# CELERY_ALWAYS_EAGER = True

CELERY_TIMEZONE = 'UTC'

STATISTICS_ROTATE_INTERVAL = 900  # 15 min
CELERYBEAT_SCHEDULE = {
    'prepare-visitors-statistics': {
        'task': 'apps.visitors.tasks.prepare_visitors_statistics',
        'schedule': crontab(minute=(5, 20, 35, 50,))
    },
    'prepare-batam-visitors-statistics': {
        'task': 'apps.visitors.tasks.prepare_batam_visitors_statistics',
        'schedule': crontab(minute=(10, 25, 40, 55,))
    },
    'produce-hourly-statistics': {
        'task': 'apps.visitors.tasks.produce_hourly_statistics',
        'schedule': crontab(minute=(10,))
    },
    'produce-daily-statistics': {
        'task': 'apps.visitors.tasks.produce_daily_statistics',
        'schedule': crontab(minute=(10,), hour='*/2')
    },
    'produce-weekly-statistics': {
        'task': 'apps.visitors.tasks.produce_weekly_statistics',
        'schedule': crontab(minute=20, hour=1)
    },
    'produce-monthly-statistics': {
        'task': 'apps.visitors.tasks.produce_monthly_statistics',
        'schedule': crontab(day_of_month='1')
    },
    'check-room2agent-status': {
        'task': 'apps.website.tasks.check_room2agent_status',
        'schedule': crontab()
    },
    'check-website-status': {
        'task': 'apps.website.tasks.check_website_status',
        'schedule': crontab(minute='*/5')
    },
    'clean-visitors': {
        'task': 'apps.visitors.tasks.clean_visitors',
        'schedule': crontab(minute='*/5')
    },
    'clean-chatrooms': {
        'task': 'apps.website.tasks.clean_chatrooms',
        'schedule': crontab(minute='*/15')
    },
} if NODE_MODE == NODE_MODE_WORKER else {
    'check-payments-status': {
        'task': 'apps.website.tasks.check_payments_status',
        'schedule': crontab(minute='*/10')
    },
    'cn2-check_openai_files': {
        'task': 'apps.ncbot.tasks.check_openai_files',
        'schedule': crontab(minute='*/10')
    },
    'transfer-watcher': {
        'task': 'apps.balancer.tasks.website_transfer_executor',
        'schedule': crontab(minute='*/1')
    },
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.redis.RedisCache',
        'LOCATION': REDIS_URL,
    },
}
WATCH_THREADS = False
IS_DJANGO_DEBUG_RUN = 'runserver' in sys.argv

DEFAULT_AUTO_FIELD = 'django.db.models.AutoField'

ALLOW_ORIGIN = [
    'http://localhost:4000',
    'http://127.0.0.1:4000',
    'http://localhost:3000',
    'http://127.0.0.1:3000',
    f'https://{DOMAIN_ACC}',
    f'https://{DOMAIN_MGT}',
    f'https://{DOMAIN_BASE}',
    f'https://{DOMAIN_WGT}',
]

for _domain in ENV_ALL_DOMAINS:
    ALLOW_ORIGIN.append(f'https://account.{_domain}')
    ALLOW_ORIGIN.append(f'https://manage.{_domain}')
    ALLOW_ORIGIN.append(f'https://direct.{_domain}')
    ALLOW_ORIGIN.append(f'https://widget.{_domain}')
    ALLOW_ORIGIN.append(f'https://{_domain}')

CSRF_TRUSTED_ORIGINS = ALLOW_ORIGIN

if ENV_DEBUG_EMAILS:
    EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

DEFAULT_FROM_EMAIL = ENV_EMAIL_FROM

SERVER_EMAIL=ENV_EMAIL_FROM
EMAIL_HOST = ENV_SMTP_HOST
EMAIL_HOST_USER = ENV_SMTP_USER
EMAIL_HOST_PASSWORD = ENV_SMTP_PASSWORD
EMAIL_PORT = ENV_SMTP_PORT
EMAIL_FROM = ENV_EMAIL_FROM
EMAIL_TIMEOUT = 15
EMAIL_USE_TLS = True if ENV_SMTP_MODE not in ['ssl', 'none'] else False
EMAIL_USE_SSL = True if ENV_SMTP_MODE == 'ssl' else False

GEOIP_PATH = os.path.join(BASE_DIR, 'data', 'geoip')
GEOIP_COUNTRY = os.path.join(GEOIP_PATH, 'GeoLite2-Country.mmdb')
GEOIP_CITY = os.path.join(GEOIP_PATH, 'GeoLite2-City.mmdb')

#MIXPANEL_API_TOKEN = ''

MAX_IMAGE_SIZE = [1000, 1000]

THUMBNAIL_PRESERVE_FORMAT = True

BILLING_ENABLED = False
BILLING_BACKEND = 'billproviders.nicechat_mocks'

DEFAULT_WIDGET_POSITION = 'left'
DEFAULT_WIDGET_SHAPE = 'square'
DEFAULT_WIDGET_SIZE = 'm'
DEFAULT_WIDGET_IDLE_TIME_M = 180

PRODUCTS_PROPS_CHECK_MAX_COUNT = 16
PRODUCTS_PROPS_CHECK_KEY_LEN = 96
PRODUCTS_PROPS_CHECK_VALUE_LEN = 512

NEW_COMPANY_NOTIFY_TO = ENV_INNER_NOTIFICATION_EMAILS

RESET_EMAIL_PROTO = 'https'

AWS_SNS_ACCESS_KEY = None
AWS_SNS_SECRET_KEY = None

UPDATE_URLS = []
UPDATE_WS = []

DISABLE_CLIENT_AUTOCREATE=False
WIDGET_SESSION_TOKEN_KEY=None
WIDGET_DEBUG_URL=None

NC_VIDEO_ENABLE = False
NC_VIDEO_BACKEND = 'videoproviders.twilionc'
TWILIO_VIDEO_ACC = ''
TWILIO_VIDEO_KEY = ''
TWILIO_VIDEO_SECRET = ''

CONNECTOR_ENABLED = True
CONNECTOR_ALLOW_AUTOCREATE = True
CONNECTOR_REPEAT_TIMEOUT = 180

OPERATORS_INFO = False
DEFAULT_CLIENT_NAME = 'Guest'
ARCHIVE_ALL = False

NOTIFICATOR_ENABLE = False
NOTIFICATOR_REPLY = None
NOTIFICATOR_ALLOW_ATTACH = False
NOTIFICATOR_STEP_MIN = 4
NOTIFICATOR_IGNORE_MIN = 240
NOTIFICATOR_PLUS_REPLY = True
NOTIFICATOR_TARGET = lambda user: getattr(user, 'email', None)

POP3_SERVER = None
POP3_USER = None
POP3_PASS = None

ASGI_APPLICATION = "nicechat.asgi.application"
WSGI_APPLICATION = 'nicechat.wsgi.application'

SECURE_PROXY_SSL_HEADER = ('HTTP_X_FORWARDED_PROTO', 'https',)

ATTACH_SIGN_SECRET = ENV_ATTACH_SIGN_SECRET

try:
    from .local import *
except ImportError as e:
    pass

import socket
WSOCKA = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  
WSOCKA.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

from .settingvalidation import validate_settings

code, _msg = validate_settings(DATABASES, REDIS_URL, PUBLIC_ROOT, MEDIA_ROOT, STATIC_ROOT)
if code != 0:
    from .gconfig import broadcast_error
    broadcast_error(f"{code}:{NODE_MODE}:{NODE_PUBLIC_KEY or ''}:{_msg}")
    exit(code)
