import sys, os, copy, tempfile, errno, ssl, ipaddress, logging, json, threading, socket, time, traceback
from datetime import datetime, timedelta
from urllib.parse import parse_qs


def broadcasting():
    wsock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  
    wsock.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
    wsock.bind(("0.0.0.0", 5005))
    wsock.settimeout(2.0)
    sys.stderr.write("Bind watcher at UDP 5005\n")
    sys.stderr.flush()

    hostname = socket.gethostname()
    _msg = f'nc_newnode_{hostname}:'.encode()

    wsock.sendto(_msg, ('127.0.0.1', 5006))
    c = 0
    while True:
        c += 1
        if c > 29:
            c = 0
            try:
                wsock.sendto(_msg, ('127.0.0.1', 5006))
            except Exception as e:
                pass
        try:
            data, addr = wsock.recvfrom(1024)
        except socket.timeout:
            continue
        sys.stderr.write(f"{data} {addr}\n")
        if data == b"iambalancer":
            wsock.sendto(_msg, (addr[0], 5005))
        if data == b"iamarbiter":
            wsock.sendto(b"iamnewnode", (addr[0], 5005))


base_data_store = {}

def base_server(e_base=None):
    sys.stderr.write("Starting server...\n")
    sys.stderr.flush()
    
    from http.server import HTTPServer, BaseHTTPRequestHandler
    class WebRequestHandler(BaseHTTPRequestHandler):
        logger = logging.getLogger("http")
        form = """<html><head><title>Setup arbiter</title></head><body>
            %(error)s
            <form action="/"
                method="POST"
                enctype="application/x-www-form-urlencoded">
                <table border=0>
                <tr><td>Main domain *</td><td><input type="text" name="first_domain" required value="%(first_domain)s" /><br/><small>You will can add more later</small> </td></tr>
                <tr><td colspan=2>Arbiter Postgres database </td></tr>
                <tr><td>Host *</td><td><input type="text" name="postgres_host" required value="%(postgres_host)s" /> </td></tr>
                <tr><td>Port</td><td><input type="text" name="postgres_port" value="%(postgres_port)s" /></td></tr>
                <tr><td>DBname *</td><td><input type="text" name="postgres_dbname" required value="%(postgres_dbname)s" /> </td></tr>
                <tr><td>User *</td><td><input type="text" name="postgres_user" required value="%(postgres_user)s" /> </td></tr>
                <tr><td>Password *</td><td><input type="password" name="postgres_password" required value="%(postgres_password)s" /> </td></tr>
                <tr><td colspan=2>Arbiter redis </td></tr>
                <tr><td>Host *</td><td><input type="text" name="redis_host" required value="%(redis_host)s" /> </td></tr>
                <tr><td>Port *</td><td><input type="text" name="redis_port" value="%(redis_port)s" /> </td></tr>
                <tr><td>DBnumber *</td><td><input type="number" name="redis_dbnumber" required value="%(redis_dbnumber)s" /> </td></tr>
                <tr><td>Use SSL</td><td><input type="checkbox" name="redis_ssl" value="checked" %(redis_ssl)s /> </td></tr>
                <tr><td>User</td><td><input type="text" name="redis_user" value="%(redis_user)s" /> </td></tr>
                <tr><td>Password</td><td><input type="password" name="redis_password" value="%(redis_password)s" /> </td></tr>
                <tr><td colspan=2>
                <input type="submit" value="Submit" /> </td></tr>
                </table>
            </form>
        </body></html>
        """
        pg = """
        <html><head><title>Setup arbiter..</title><meta http-equiv="refresh" content="30; url=/arbiter/first-run/"></head><body>...</body></html>
        """
        def do_GET(self):
            sys.stderr.write(self.path)
            if e_base:
                self.send_response(200)
                self.send_header("Content-Type", "text/html")
                self.end_headers()
                self.wfile.write(e_base.encode())
            if self.path in ['', '/']:
                self.exec_write_form()
                return
            if self.path in ['/arbiter/first-run/', '/arbiter/first-run']:
                self.exec_first_run()
                return
            self.send_response(404)
            self.end_headers()

        def do_POST(self):
            if e_base:
                self.send_response(405)
                self.end_headers()
                return

            sys.stderr.write(f"{self.path}\n\n")
            if self.path in ['', '/']:
                self.exec_write_settings()
                return
            if self.path in ['/initassign', '/initassign/']:
                self.exec_initassign()
                return
            if self.path in ['/applyassign/arbiter', '/applyassign/arbiter/']:
                self.exec_applyassign(False)
                return
            if self.path in ['/applyassign/worker', '/applyassign/worker/']:
                self.exec_applyassign(True)
                return
            if self.path in ['/applyreboot', '/applyreboot/']:
                self.exec_applyreboot()
                return
        
            self.send_response(404)
            self.end_headers()

        def exec_first_run(self):
            self.send_response(200)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            self.wfile.write(self.pg.encode())

        def exec_write_form(self, data=None):
            self.send_response(200)
            self.send_header("Content-Type", "text/html")
            self.end_headers()
            first_domain = ''
            for h in self.headers:
                if h and f"{h}".lower() == 'host':
                    first_domain = self.headers[h]
                    break
            _data = {
                'first_domain': first_domain, 'error': '',
                'postgres_host': '','postgres_port': '5432','postgres_dbname': '','postgres_user': '','postgres_password': '',
                'redis_host': '','redis_port': '6379','redis_dbnumber': '','redis_ssl': '','redis_user': '','redis_password': '',
            }
            if data:
                _data.update(data)
            sys.stderr.write(str(_data))
            form = self.form % _data
            self.wfile.write(form.encode())

        def exec_initassign(self):
            self.send_response(202)
            self.send_header("Content-Type", "application/binary")
            self.end_headers()

            from cryptography.hazmat.primitives import serialization
            from cryptography.hazmat.primitives.asymmetric import x448

            private_key = x448.X448PrivateKey.generate()
            base_data_store['privkey'] = private_key
            pubkey = private_key.public_key().public_bytes(
                encoding=serialization.Encoding.Raw,
                format=serialization.PublicFormat.Raw
            )
            base_data_store['pubkey'] = pubkey
            self.wfile.write(pubkey)

        def exec_applyreboot(self, is_worker):
            # if os.path.exists('/var/lib/nicechat/nicechat.conf'):
            #     sys.exit(0)
            self.send_response(201)
            self.send_header("Content-Type", "application/json")
            self.end_headers()
            self.wfile.write(b'{"status": "ok"}')

        def exec_applyassign(self, is_worker):
            content_len = int(self.headers.get('Content-Length'))
            post_body = self.rfile.read(content_len)
            a_pbkey = None
            for h in self.headers:
                if h and f"{h}".lower() == 'x-arbiter':
                    a_pbkey = self.headers[h]
                    break
            from cryptography.hazmat.primitives import serialization
            from cryptography.hazmat.primitives.asymmetric import x448
            import base64
            if a_pbkey and base_data_store.get('privkey'):
                from nicechat.utils.arbiter import arb_encrypt, arb_decrypt
                try:
                    _e_data, _is_correct = arb_decrypt(post_body, base_data_store['privkey'], a_pbkey)
                    if _is_correct:
                        _data = json.loads(_e_data)
                except Exception as e:
                    traceback.print_exception(e)
                    _data = None
                sys.stderr.write(f"{_data} {is_worker}\n\n")
                if _is_correct and _data:
                    # wsocka = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  
                    # wsocka.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)

                    if not is_worker:
                        sys.stderr.write("Save configs...\n")
                        with open('/var/lib/nicechat/nicechat.conf', 'w') as f:
                            for k, v in _data.items():
                                f.write(f"{k}={v}\n")
                        sys.stderr.write("Sending notification...\n")
                        self.send_response(201)
                        self.send_header("Content-Type", "application/json")
                        self.end_headers()
                        self.wfile.write(b'{"status": "ok"}')
                    else:
                        from nicechat.utils.arbiter import get_fullnode_key
                        nodekey = base_data_store['privkey'].private_bytes(
                            encoding=serialization.Encoding.Raw,
                            format=serialization.PrivateFormat.Raw,
                            encryption_algorithm=serialization.NoEncryption(),
                        )
                        nodekey = base64.b32encode(b'\xb3\xa2\xa2D' + nodekey).decode()
 
                        pubkey = a_pbkey
                        nodename = _data.get('NODE')
                        settingskey = f"{nodename}.{nodekey}.{pubkey}"
                        sys.stderr.write("Save configs...\n")
                        with open('/var/lib/nicechat/nicechat.conf', 'w') as f:
                            f.write(f"ARBITER_HOSTKEY={settingskey}\n")
                        self.send_response(201)
                        self.send_header("Content-Type", "application/json")
                        self.end_headers()
                        self.wfile.write(b'{"status": "ok"}')
                    sys.exit(0)

            self.send_response(400)
            self.end_headers()

        def exec_write_settings(self):    
            content_len = int(self.headers.get('Content-Length'))
            post_body = self.rfile.read(content_len)
            
            data, res = self._validate_data(post_body)
            if not res:
                self.exec_write_form(data)
                return
            
            sys.stderr.write("Data is valid\n")
            db_dict = {
                'default': {
                    'ENGINE': 'django.db.backends.postgresql_psycopg2',
                    'NAME': data.get('postgres_dbname', ''),
                    'USER': data.get('postgres_user', ''),
                    'PASSWORD': data.get('postgres_password', ''),
                    'HOST': data.get('postgres_host', ''),
                    'PORT': '5432',
                    'CONN_MAX_AGE': 30,
                }
            }
            code, dt, cnn = read_db(db_dict)
            sys.stderr.write(f"{code} {dt} {cnn}")
            if code != 0:
                data['error'] = dt
                self.exec_write_form(data)
                return
            sys.stderr.write("Writing configs...\n")

            import string
            import random
            import base64
            from cryptography.hazmat.primitives import serialization
            from cryptography.hazmat.primitives.asymmetric import x448
            from nicechat.utils.crypt import encrypt_dict

            hex_chars = '0123456789abcdef'
            secret_chars = string.ascii_letters + string.digits
            secret_key = ''.join(random.choice(secret_chars) for _ in range(64))
            balancer_sign = ''.join(random.choice(secret_chars) for _ in range(32))

            c_k = ''.join(random.choice(hex_chars) for _ in range(96))

            private_key = x448.X448PrivateKey.generate()
            private_bytes = private_key.private_bytes(
                encoding=serialization.Encoding.Raw,
                format=serialization.PrivateFormat.Raw,
                encryption_algorithm=serialization.NoEncryption(),
            )

            arbiter_ca = base64.b32encode(b'\x04C1\x00' + private_bytes).decode()

            try:
                c = cnn.cursor()
                c.execute("DELETE FROM arbiter_settings;")
                c.execute("DELETE FROM arbiter_hostdomains;")
                cnn.commit()
                c = cnn.cursor()
                d = copy.deepcopy(data)
                d['first_run'] = '1'
                for k, v in encrypt_dict(d, c_k).items():
                    if k not in ['redis_host', 'redis_port','redis_dbnumber','redis_ssl', 'redis_user','redis_password']:
                        continue
                    sys.stderr.write(f"{k}: {v}\n")
                    c.execute("insert into arbiter_settings (key, data) values(%s, %s);", (k,v,))
                crt, key = generate_selfsigned_cert(data['first_domain'], add_sub=True)
                dtt = json.dumps(encrypt_dict({'crt': crt.decode(), 'key': key.decode()}, c_k))
                c.execute('insert into arbiter_hostdomains (domain, data) values(%s, %s);', (data['first_domain'], dtt,))
                cnn.commit()
            except Exception as e:
                data['error'] = f'DB Error {e}'
                self.exec_write_form(data)
                return

            with open('/var/lib/nicechat/nicechat.conf', 'w') as f:
                f.write(f"ARBITER_CA={arbiter_ca}\n")
                f.write(f"BALANCER_SIGN={balancer_sign}\n")
                f.write(f"SECRET_KEY={secret_key}\n")
                f.write(f"DATA_KEY={c_k}\n")
                f.write(f"POSTGRES_NAME={data['postgres_dbname']}\n")
                f.write(f"POSTGRES_USER={data['postgres_user']}\n")
                f.write(f"POSTGRES_PASSWORD={data['postgres_password']}\n")
                f.write(f"POSTGRES_HOST={data['postgres_host']}\n")

            sys.stderr.write("Sending notification...\n")
            wsocka = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  
            wsocka.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            wsocka.sendto(b'up_arbiter', ('127.0.0.1', 5006))
            self.exec_first_run()
            sys.exit(0)
            return
    
        def _validate_data(self, post_body):
            _data = {
                k.decode():v[0].decode() for k, v in parse_qs(post_body).items() if k and v
            }
            err = []
            for k in ['redis_host', 'redis_dbnumber', 'first_domain', 'postgres_dbname', 'postgres_host', 'postgres_user', 'postgres_password']:
                if not _data.get(k):
                    err.append(f'"{k}" is required')
            if not err:
                public = os.getenv('PUBLIC_ROOT', default='/nc/public/')
                _redis_user = _data.get('redis_user', '')
                _redis_pass = _data.get('redis_pass', '')
                _redis_host = _data.get('redis_host', '')
                _redis_port = _data.get('redis_port', '6379')
                _redis_dbnumber = _data.get('redis_dbnumber', '')
                _redis_ssl = True if _data.get('redis_ssl', None) else False

                redis_auth = f'{_redis_user}:{_redis_pass}@' if _redis_user else ''
                redis_url = f'rediss://{redis_auth}{_redis_host}:{_redis_port}/{_redis_dbnumber}?ssl_cert_reqs=none' if \
                    _redis_ssl else f'redis://{redis_auth}{_redis_host}:{_redis_port}/{_redis_dbnumber}'

                db_dict = {
                    'default': {
                        'ENGINE': 'django.db.backends.postgresql_psycopg2',
                        'NAME': _data.get('postgres_dbname', ''),
                        'USER': _data.get('postgres_user', ''),
                        'PASSWORD': _data.get('postgres_password', ''),
                        'HOST': _data.get('postgres_host', ''),
                        'PORT': '5432',
                        'CONN_MAX_AGE': 30,
                    }
                }

                code, text = validate_settings(db_dict, redis_url, public)
                if code != 0:
                    err.append(text)
            _data['error'] = ''.join(err)
            return _data, True if not err else False

    server = HTTPServer(("0.0.0.0", 8000), WebRequestHandler)
    crt, key = generate_selfsigned_cert('chat')
    with tempfile.TemporaryDirectory() as tmpdirname:
        fp_k = open(os.path.join(tmpdirname, 'key.pem'), 'wb')
        fp_c = open(os.path.join(tmpdirname, 'crt.pem'), 'wb')
        fp_k.write(key)
        fp_k.close()
        fp_c.write(crt)
        fp_c.close()

        server.socket = ssl.wrap_socket (server.socket,
            keyfile=fp_k.name,
            certfile=fp_c.name,
            server_side=True)

        sys.stderr.write("Started server on port 8000\n")
        sys.stderr.flush()
        bcth = threading.Thread(name='broadcasting', target=broadcasting)
        bcth.setDaemon(True)
        bcth.start()

        server.serve_forever()


def generate_selfsigned_cert(hostname, ip_addresses=None, key=None, add_sub=False):
    """Generates self signed certificate for a hostname, and optional IP addresses."""
    from cryptography import x509
    from cryptography.x509.oid import NameOID
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.backends import default_backend
    from cryptography.hazmat.primitives import serialization
    from cryptography.hazmat.primitives.asymmetric import rsa

    if key is None:
        key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend(),
        )
    
    name = x509.Name([
        x509.NameAttribute(NameOID.COMMON_NAME, hostname)
    ])
 
    alt_names = [x509.DNSName(hostname)]
    if add_sub:
        alt_names.append(x509.DNSName(f'account.{hostname}'))
        alt_names.append(x509.DNSName(f'direct.{hostname}'))
        alt_names.append(x509.DNSName(f'manage.{hostname}'))

    if ip_addresses:
        for addr in ip_addresses:
            alt_names.append(x509.DNSName(addr))
            alt_names.append(x509.IPAddress(ipaddress.ip_address(addr)))
    
    san = x509.SubjectAlternativeName(alt_names)

    days = 35 if not add_sub else 3650

    basic_contraints = x509.BasicConstraints(ca=True, path_length=0)
    now = datetime.utcnow()
    cert = (
        x509.CertificateBuilder()
        .subject_name(name)
        .issuer_name(name)
        .public_key(key.public_key())
        .serial_number(1000)
        .not_valid_before(now)
        .not_valid_after(now + timedelta(days=days))
        .add_extension(basic_contraints, False)
        .add_extension(san, False)
        .sign(key, hashes.SHA256(), default_backend())
    )
    cert_pem = cert.public_bytes(encoding=serialization.Encoding.PEM)
    key_pem = key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption(),
    )

    return cert_pem, key_pem

def read_db(db_dict):
    data = {}
    try:
        db = copy.deepcopy(db_dict)

        from django.db.utils import load_backend
        backend = load_backend(db['default']["ENGINE"])
        dw = backend.DatabaseWrapper(db, 'default')

        setattr(dw, 'settings_dict', db['default'])
        if not db['default'].get('OPTIONS'):
            db['default']['OPTIONS'] = {}

        conn_params = dw.get_connection_params()
        cnn = dw.get_new_connection(conn_params)

        try:
            c = cnn.cursor()
            c.execute("CREATE TABLE IF NOT EXISTS arbiter_settings (key varchar(45) NOT NULL, data text, PRIMARY KEY (key));")
            c.execute("CREATE TABLE IF NOT EXISTS arbiter_hostdomains (domain varchar(150) NOT NULL, data text, PRIMARY KEY (domain));")
            cnn.commit()
            c = cnn.cursor()
            c.execute("select key, data from arbiter_settings;")
            rows=c.fetchall()
            cnn.commit()
            for row in rows:
                data[row[0]] = row[1]
            # sys.stderr.write(f"{rows}")
            data['domains'] = []
            c.execute("select domain from arbiter_hostdomains;")
            rows=c.fetchall()
            cnn.commit()
            # sys.stderr.write(f"{rows}")
            for row in rows:
                data['domains'].append(row[0])
        except OperationalError:
            connected = False
        else:
            connected = True
            
        if not connected:
            return 12, "DB is not established", None
        else:
            print("DB - ok")
            return 0, data, cnn
    except Exception as e:
        traceback.print_exception(e)
        return 10, f"DB - error: {e}", None
    

def validate_settings(db_dict, redis_url, public, media=None, static=None):

    code, data, cnn = read_db(db_dict)
    if code != 0:
        return code, data

    try:
        import redis
        url_connection = redis.from_url(redis_url, socket_timeout=15, socket_connect_timeout=15)
        if not url_connection.ping():
            return 12, "Redis is not established"
        else:
            print("Redis - ok")
    except Exception as e:
        return 20, f"Redis - error:{e}"


    if not os.path.exists(public) or not os.path.isdir(public):
        return 30, f"{public} is not mounted"
    if media and static:
        if not os.path.exists(media) or not os.path.exists(static):
            try:
                testfile = tempfile.TemporaryFile(dir = public)
                testfile.close()
            except OSError as e:
                if e.errno == errno.EACCES:
                    return 31, f"{public} is not writable"

    print("Media-dirs - ok")

    return 0, None


class HostManager(list):
    PTH = '/var/lib/nicechat/arb.cache'
    def __init__(self, *args, **kwargs):
        super(HostManager, self).__init__(*args, **kwargs)
        # try:
        if os.path.exists(self.PTH):
            with open(self.PTH, 'r') as f:
                data = f.read()
                if data:
                    data = json.loads(data)
                    self.extend(set(data))

    def add_host(self, host):
        items = set(self)
        itemsa = set(self)
        items.add(host)
        self.clear()
        self.extend(items)
        if items.symmetric_difference(itemsa):
            with open(self.PTH, 'w') as f:
                f.write(json.dumps(self))
    
    def rem_host(self):
        if host in self:
            self.remove(host)
            with open(self.PTH, 'w') as f:
                f.write(json.dumps(self))

    def empty(self):
        self.clear()
        with open(self.PTH, 'w') as f:
            f.write(json.dumps([]))
