from django import template
from django.conf import settings
from django.utils.safestring import mark_safe


register = template.Library()


class RevisionGet(object):
    pass


def filter_by_extension(bundle, extension):
    for chunk in bundle:
        if chunk['name'].endswith('.{0}'.format(extension)):
            yield chunk


def render_as_tags(bundle, attrs):
    tags = []
    if not hasattr(RevisionGet, 'REV'):
        label = settings.RELEASE_ID
        label = label.strip() if label and isinstance(label, str) else '#'
        setattr(RevisionGet, 'REV', label)

    for chunk in bundle:
        url = chunk['url'].replace("{%REV%}", getattr(RevisionGet, 'REV', 'latest'))

        if chunk['name'].endswith(('.js', '.js.gz')):
            tags.append((
                '<script type="text/javascript" src="{0}" {1}></script>'
            ).format(url, attrs))
        elif chunk['name'].endswith(('.css', '.css.gz')):
            tags.append((
                '<link type="text/css" href="{0}" rel="stylesheet" {1}/>'
            ).format(url, attrs))
    return mark_safe('\n'.join(tags))


def get_prod_bundle(bundle_name, extension):
    bundles = settings.WEBPACK_TPLS.get(bundle_name, [])
    bundles = filter_by_extension(bundles, extension)
    bundles = [prepare_url(chunk) for chunk in bundles]
    return bundles


def prepare_url(chunk):
    if not chunk['url'].startswith(settings.STATIC_URL):
        chunk['url'] = settings.STATIC_URL + chunk['url']
    return chunk

@register.simple_tag
def dettag(tag):
    print(dir(tag))
    return 'ok'

@register.simple_tag
def print_version():
    if not hasattr(RevisionGet, 'DSC'):
        try:
            if settings.DESC_ID is None:
                import subprocess
                descr = subprocess.check_output(["git", "describe"])
            else:
                descr = settings.DESC_ID
            descr = descr.strip() if descr and isinstance(descr, str) else '#'
        except subprocess.CalledProcessError:
            descr = '#'
        setattr(RevisionGet, 'DSC', descr)

    return "%s-%s" % (RevisionGet.DSC, getattr(RevisionGet, 'REV', '-'))