from rest_framework.exceptions import ValidationError
import traceback
from django.conf import settings

PAY_CALLS = {}
PAYMENT_NAME = None

def register_payments(pay_module, api_patterns):
    if not settings.BILLING_ENABLED:
        return

    update_route = getattr(pay_module, 'update_route', None)
    global PAYMENT_NAME

    PAYMENT_NAME = getattr(pay_module, 'PAYMENT_NAME', None)
    if not PAYMENT_NAME:
        PAYMENT_NAME = 'disabled'
        return

    if update_route:
        update_route(api_patterns)

    for k in dir(pay_module):
        if callable(getattr(pay_module, k)):
            PAY_CALLS[k] = getattr(pay_module, k)

def execute_billing(name, args=None, kwargs=None):
    if not settings.BILLING_ENABLED:
        return

    cl = PAY_CALLS.get(name)
    _args = args if args else []
    _kwargs = kwargs if kwargs else {}

    try:
        if cl:
            return cl(*_args, **_kwargs)
    except Exception as x:
        traceback.print_exc()
        raise BillingError(x.message)

    raise KeyError('method {} not found'.format(name))


def get_billing_provider():
    global PAYMENT_NAME
    return PAYMENT_NAME


class BillingError(ValidationError):
    pass
