from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.padding import PKCS7
import base64


def ncencrypt(body, hs, iv):
    padding = PKCS7(128).padder()
    padded_data = padding.update(body)
    padded_data += padding.finalize()
    cipher = Cipher(algorithms.AES(hs), modes.CBC(iv))
    encryptor = cipher.encryptor()
    ct = encryptor.update(padded_data) + encryptor.finalize()
    return base64.b64encode(ct).decode()


def ncdecrypt(credentials, hs, iv, tostr=False):
    cipher = Cipher(algorithms.AES(hs), modes.CBC(iv))
    decryptor = cipher.decryptor()
    data = decryptor.update(base64.b64decode(credentials)) + decryptor.finalize()
    unpadding = PKCS7(128).unpadder()
    udata = unpadding.update(data)
    udata += unpadding.finalize()
    return udata if not tostr else udata.decode()

def get_data_keys(data):
    return bytes.fromhex(data[0:64]), bytes.fromhex(data[64:96])

def decrypt_dict(dct, key):
    hs, iv = get_data_keys(key)
    return decrypt_dict_v(dct, hs, iv)

def decrypt_dict_v(dct, hs, iv):
    ret = {}
    for k, v in dct.items():
        ret[k] = ncdecrypt(v, hs, iv).decode() if isinstance(v, str) else v
    return ret

def encrypt_dict(dct, key):
    hs, iv = get_data_keys(key)
    return encrypt_dict_v(dct, hs, iv)

def encrypt_dict_v(dct, hs, iv):
    ret = {}
    for k, v in dct.items():
        ret[k] = ncencrypt(f"{v}".encode(), hs, iv)
    return ret

    
