from django.contrib.gis import geoip2
#from geoip2.errors import AddressNotFoundError
import socket
import user_agents
from apps.chat.models import Visitor

from django.conf import settings
g = geoip2.GeoIP2()


def get_info_by_ip(ip):
    ip_data = {}
    if ip:
        try:
            ip_data['city'] = g.city(ip)['city']
            ip_data['country'] = g.country_name(ip)
            lon_lat = g.lon_lat(ip)
            if not lon_lat:
                ip_data['lon'] = -90
                ip_data['lat'] = -180
            else:
                ip_data['lon'], ip_data['lat'] = lon_lat
        except Exception:
            ip_data = {
                'city': 'N/A',
                'country': 'N/A',
                'lon': -90,
                'lat': -180
            }
        return ip_data
    raise Exception('IP address needed')



_IP_HEADERS_LIST = [
    # apadted for daphne-server
    'cf-connecting-ip',
    'x-forwarded-for',
    'client-ip',
    'x-real-ip',
    'x-forwarded',
    'x-cluster-client-ip',
    'forwarded-for',
    'forwarded',
    'via',
    'client',
]


def parse_info(ch, visitor, model_name=Visitor):
    headers = ch.scope['headers']
    headers = {v[0].decode(): v[1].decode() for v in headers if isinstance(headers, list) and len(headers) > 2}
    for client_ip in ch.scope.get('client', []):
        # get first record if exixsts
        headers['client'] = client_ip
        break

    main_ip = ''
    other = ''
    lang = ''
    ua_data = user_agents.parse(headers.get('user-agent', 'Mozilla/5.0 (Unknown)'))

    os = ua_data.os.family
    browser = ua_data.browser.family
    os_ver = ua_data.os.version_string
    browser_ver = ua_data.browser.version_string
    device = ua_data.device.family

    langs = parse_accept_language(headers.get('accept-language', 'en'))
    for lang, q in langs:
        break

    for key in _IP_HEADERS_LIST:
        value = headers.get(key, '').strip()
        if not value:
            value = str(ch.scope.get(key, '')).strip()
        if value != '':
            ips = [ip.strip().lower() for ip in value.split(',')]
            for ip_str in ips:
                if ip_str:
                    if not main_ip and _is_valid_ip(ip_str):
                        main_ip = ip_str
                    other = "%s %s: %s\n" % (other, key, ip_str)
    main_ip = 'n/a' if not main_ip else main_ip

    if not visitor:
        visitor = model_name()
    visitor.sys_ip = main_ip[0:30] if main_ip and isinstance(main_ip, str) else None
    visitor.sys_lang = lang[0:30] if lang and isinstance(lang, str) else None
    visitor.sys_lang_full = lang
    visitor.sys_os = os[0:30] if os and isinstance(os, str) else None
    visitor.sys_browser = browser[0:30] if browser and isinstance(browser, str) else None
    visitor.sys_info = other
    visitor.sys_os_ver = os_ver
    visitor.sys_browser_ver = browser_ver
    visitor.sys_device = device


    return visitor


def _is_valid_ipv4(ip_str):
    try:
        socket.inet_pton(socket.AF_INET, ip_str)
    except AttributeError:
        try:
            socket.inet_aton(ip_str)
        except socket.error:
            return False
        return ip_str.count('.') == 3
    except socket.error:
        return False
    return True


def _is_valid_ipv6(ip_str):
    try:
        socket.inet_pton(socket.AF_INET6, ip_str)
    except socket.error:
        return False
    return True


def _is_valid_ip(ip_str):
    return _is_valid_ipv4(ip_str) or _is_valid_ipv6(ip_str)


def parse_accept_language(accept_language):
    languages = accept_language.split(",")
    locale_q_pairs = []

    for language in languages:
        if language.split(";")[0] == language:
            # no q => q = 1
            locale_q_pairs.append((language.strip(), "1"))
        else:
            locale = language.split(";")[0].strip()
            q = language.split(";")[1].split("=")[1]
            locale_q_pairs.append((locale, q))

    return locale_q_pairs
