import time
from datetime import timedelta

from django.core.exceptions import AppRegistryNotReady
from django.utils import timezone
from django.conf import settings
from django.db import OperationalError, models
from django.core.cache import caches, InvalidCacheBackendError


def clean_zombie_visitors(use_reactor=True):
    while True:
        try:
            from apps.chat.models import Visitor
            Visitor.objects.all().delete()
            print("visitors cleared")
            from apps.website.models import AgentStatus
            AgentStatus.objects.filter(is_active=True).delete()
            print("operators cleared")
            break
        except AppRegistryNotReady:
            time.sleep(0.1)


def clean_chatrooms(use_reactor=True):
    if not use_reactor:
        time.sleep(10)
    try:

        print("Start Chat cleaner")
        del_serialized = timezone.now() - timedelta(minutes=settings.REMOVE_CLOSED_CHAT_M)
        pack_unclosed = timezone.now() - timedelta(minutes=settings.ARCHIVE_UNACTIVE_CHAT_M)
        pack_unclosed_connector = timezone.now() - timedelta(minutes=settings.ARCHIVE_UNACTIVE_CHAT_CONN_M)

        from apps.chat.models import Chat
        from apps.chat.utils import move_chat_to_archive
        r = 0

        # look all Chat that are closed, but not archived
        for room in Chat.objects.filter(is_open=False, is_serialized=False, client__mode=1):
            print("Serialize chat room #%i" % room.pk)
            move_chat_to_archive(room, force_del=True)

        # remove chat from "Chat" tab not immediately (after it closed and archived), but after some time
        for room in Chat.objects.filter(is_serialized=True, ts_close__lt=del_serialized, client__mode=1):
            try:
                room.joinedbot.all().delete()
                room.delete()
                r += 1
            except OperationalError:
                pass

        if not settings.REMOVE_CLOSED_CHAT_ONLY_CLI:
            for room in Chat.objects.filter(is_serialized=True, ts_close__lt=del_serialized):
                try:
                    room.joinedbot.all().delete()
                    room.delete()
                    r += 1
                except OperationalError:
                    pass

        print("Delete %i closed chat(s)" % r)

        # find all chat in which was not activity for a long time (check open time and last message time)
        items = list(Chat.objects.filter(is_open=True, is_serialized=False, ts_open__lt=pack_unclosed, client__mode=1, is_connector=False))
        items.extend(list(Chat.objects.filter(is_open=True, is_serialized=False, ts_open__lt=pack_unclosed_connector, client__mode=1, is_connector=True)))
        for room in items:
            msg = room.message_set.last()
            if (msg and msg.timestamp < pack_unclosed) or not msg:
                room.is_open = False
                room.ts_close = timezone.now()
                room.save()
                room.put_tech_message("~!CLOSEDBYSERVER:")
                move_chat_to_archive(room, force_del=True)

                from apps.chat.models import Visitor
                visitor = Visitor.objects.filter(client_id=room.client_id, website_id=room.website_id).first()
                if visitor:
                    models.signals.post_save.send(sender=Visitor, instance=visitor, created=False, update_fields=[])
                print("Close and serialize chat room #%i" % room.pk)

        print("End Chat cleaner")
    except AppRegistryNotReady:
        pass
