import time
import json
from channels.layers import get_channel_layer
from asgiref.sync import async_to_sync
channel_layer = get_channel_layer()
from nicechat.utils.constants import WebsocketEventTypes

VIDEO_NAME = None
VIDEO_MODULE = object()


def register_video(vid_module, api_patterns):
    update_route = getattr(vid_module, 'update_route', None)
    global VIDEO_MODULE
    global VIDEO_NAME
    VIDEO_MODULE = vid_module

    VIDEO_NAME = getattr(vid_module, 'VIDEO_NAME', None)
    if not VIDEO_NAME:
        VIDEO_NAME = 'disabled'
        return

    if update_route:
        update_route(api_patterns)


def get_video_provider():
    global VIDEO_NAME
    return VIDEO_NAME


def get_video_module():
    global VIDEO_MODULE
    return VIDEO_MODULE


def video_stop_call(call, client, session):
    call.is_active = False
    txt = "~!CALL_CANCEL:1"
    if call.is_started:
        txt = "~!CALL_END:1"
        call.is_started = False
        if hasattr(VIDEO_MODULE, 'stop_call'):
            VIDEO_MODULE.stop_call(call.get_props())
    call.save()
    call.chat.put_tech_message_wdata(txt, {
        'provider': VIDEO_NAME,
        'time': time.time()
    })
    call.chat.clear_call_messages()

    website_id = call.chat.website_id
    for peer in call.peers.all():
        if peer.peer:
            if peer.peer.startswith('ag_'):
                token = "adm-website_mon-u-%s-%s" % (website_id, peer.peer[3:])
                async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  "text":  json.dumps(
                    {
                        "type": WebsocketEventTypes.BACKEND_SITE_WS_CALL_STOP,
                    })
                })
            else:
                token = "cli_wait-%s-%s" % (website_id, peer.peer)
                async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  "text":  json.dumps(
                    {
                        "type": WebsocketEventTypes.FRONTEND_CHAT_WS_CALL_STOP,
                    })
                })


def video_start_call(call, chat):
    if hasattr(VIDEO_MODULE, 'start_call') and hasattr(VIDEO_MODULE, 'get_user_props'):
        room_unq = 'room_nc_%s-%s' % (call.pk, chat.pk)
        room_props = VIDEO_MODULE.start_call(room_unq)
        call.update_props(room_props, save_after=True)
        for peer in call.peers.all():
            if peer.peer:
                cli_props = VIDEO_MODULE.get_user_props(room_unq, peer.peer)
                cli_props.update(room_props)
                peer.update_props(cli_props, save_after=True)
                if peer.peer.startswith('ag_'):
                    token = "adm-website_mon-u-%s-%s" % (chat.website_id, peer.peer[3:])
                    async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  "text":  json.dumps(
                        {
                            "type": WebsocketEventTypes.BACKEND_SITE_WS_CALL_ROOM,
                            "data": cli_props
                        })
                    })
                else:
                    token = "cli_wait-%s-%s" % (chat.website_id, peer.peer)
                    cli_props['type'] = WebsocketEventTypes.FRONTEND_CHAT_WS_CALL
                    async_to_sync(channel_layer.group_send)(token, {"type": "websocket.send",  "text":  json.dumps(cli_props)})
        chat.clear_call_messages()
        chat.put_tech_message_wdata("~!CALL_START:1", {
            'provider': VIDEO_NAME
        })
